@@ -1,5 +1,5 @@
 /*
- *  $Id: chg-scsi-chio.c,v 1.1.2.3 1999/09/08 23:26:02 jrj Exp $
+ *  $Id: chg-scsi-chio.c,v 1.1.2.3.6.3 2003/01/26 19:20:56 martinea Exp $
  *
  *  chg-scsi.c -- generic SCSI changer driver
  *
@@ -242,7 +242,7 @@ int read_config(char *configfile, changer_t *chg)
   FILE *file;
   int init_flag = 0;
   int drivenum=0;
-  char linebuffer[256];
+  char *linebuffer = NULL;
   int token;
   char *value;
 
@@ -252,8 +252,9 @@ int read_config(char *configfile, changer_t *chg)
   if (NULL==(file=fopen(configfile,"r"))){
     return (-1);
   }
-  while (!feof(file)){
-    if (NULL!=fgets(linebuffer,255,file)){
+
+  amfree(linebuffer);
+  while (NULL!=(linebuffer=agets(file))){
       parse_line(linebuffer,&token,&value);
       if (token != -1){
         if (0==init_flag) {
@@ -277,13 +278,13 @@ int read_config(char *configfile, changer_t *chg)
           chg->sleep = atoi(value);
           break;
         case CHANGERDEV:
-          chg->device = strdup(value);
+          chg->device = stralloc(value);
           break;
         case SCSITAPEDEV:
-          chg->conf[drivenum].scsitapedev = strdup(value);
+          chg->conf[drivenum].scsitapedev = stralloc(value);
           break;
         case TAPESTATFILE:
-          chg->conf[drivenum].tapestatfile = strdup(value);
+          chg->conf[drivenum].tapestatfile = stralloc(value);
           break;
         case CLEANMAX:
           chg->cleanmax = atoi(value);
@@ -328,7 +329,7 @@ int read_config(char *configfile, changer_t *chg)
           break;
         case DEVICE:
           if (drivenum < numconf){
-            chg->conf[drivenum].device = strdup(value);
+            chg->conf[drivenum].device = stralloc(value);
           } else {
             fprintf(stderr,"Error: drive is not less than number_drives"\
                     " device ignored\n");
@@ -336,7 +337,7 @@ int read_config(char *configfile, changer_t *chg)
           break;
         case STATFILE:
           if (drivenum < numconf){
-            chg->conf[drivenum].slotfile = strdup(value);
+            chg->conf[drivenum].slotfile = stralloc(value);
           } else {
             fprintf(stderr,"Error: drive is not less than number_drives"\
                     " slotfile ignored\n");
@@ -344,7 +345,7 @@ int read_config(char *configfile, changer_t *chg)
           break;
         case CLEANFILE:
           if (drivenum < numconf){
-            chg->conf[drivenum].cleanfile = strdup(value);
+            chg->conf[drivenum].cleanfile = stralloc(value);
           } else {
             fprintf(stderr,"Error: drive is not less than number_drives"\
                     " cleanfile ignored\n");
@@ -352,7 +353,7 @@ int read_config(char *configfile, changer_t *chg)
           break;
         case USAGECOUNT:
           if (drivenum < numconf){
-            chg->conf[drivenum].timefile = strdup(value);
+            chg->conf[drivenum].timefile = stralloc(value);
           } else {
             fprintf(stderr,"Error: drive is not less than number_drives"\
                     " usagecount ignored\n");
@@ -363,8 +364,9 @@ int read_config(char *configfile, changer_t *chg)
           break;
         }
       }
-    }
+    amfree(linebuffer);
   }
+  amfree(linebuffer);
 
   fclose(file);
   return 0;
@@ -656,23 +658,23 @@ int main(int argc, char *argv[])
     drive_num    = chg.conf[confnum].drivenum;
     need_eject   = chg.eject;
     need_sleep   = chg.sleep;
-    clean_file   = strdup(chg.conf[confnum].cleanfile);
+    clean_file   = stralloc(chg.conf[confnum].cleanfile);
     clean_slot   = chg.conf[confnum].cleanslot;
     maxclean     = chg.cleanmax;
     if (NULL != chg.conf[confnum].timefile)
-      time_file = strdup(chg.conf[confnum].timefile);
+      time_file = stralloc(chg.conf[confnum].timefile);
     if (NULL != chg.conf[confnum].slotfile)
-      changer_file = strdup(chg.conf[confnum].slotfile);
+      changer_file = stralloc(chg.conf[confnum].slotfile);
     else
       changer_file = NULL;
     if (NULL != chg.conf[confnum].device)
-      tape_device  = strdup(chg.conf[confnum].device);
+      tape_device  = stralloc(chg.conf[confnum].device);
     if (NULL != chg.device)
-      changer_dev  = strdup(chg.device); 
+      changer_dev  = stralloc(chg.device); 
     if (NULL != chg.conf[confnum].scsitapedev)
-      scsitapedevice = strdup(chg.conf[confnum].scsitapedev);
+      scsitapedevice = stralloc(chg.conf[confnum].scsitapedev);
     if (NULL != chg.conf[confnum].tapestatfile)
-      tapestatfile = strdup(chg.conf[confnum].tapestatfile);
+      tapestatfile = stralloc(chg.conf[confnum].tapestatfile);
     dump_changer_struct(chg);
     /* get info about the changer */
     if (-1 == (fd = OpenDevice(changer_dev, "changer_dev"))) {
@@ -687,12 +689,12 @@ int main(int argc, char *argv[])
 
     if (tape_device == NULL)
       {
-        tape_device = strdup(changer_dev);
+        tape_device = stralloc(changer_dev);
       }
 
     if (scsitapedevice == NULL)
       {
-         scsitapedevice = strdup(tape_device);
+         scsitapedevice = stralloc(tape_device);
       }
 
     if ((chg.conf[confnum].end == -1) || (chg.conf[confnum].start == -1)){
