@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1998, 2001 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: getfsent.c,v 1.20.4.2 2001/03/30 02:40:21 martinea Exp $
+ * $Id: getfsent.c,v 1.20.4.1.2.2.2.6 2003/03/17 18:50:11 martinea Exp $
  *
  * generic version of code to read fstab
  */
@@ -143,25 +143,49 @@ generic_fsent_t *fsent;
 
 #include <mntent.h>
 
+#if defined(HAVE_ENDMNTENT)
+#define AMCLOSE_MNTENT(x)	endmntent(x)
+#else
+#define AMCLOSE_MNTENT(x)	fclose(x)
+#endif
+
 static FILE *fstabf1 = NULL;		/* /proc/mounts */
-static FILE *fstabf2 = NULL;		/* MNTTAB */
+static FILE *fstabf2 = NULL;		/* MOUNTED */
+static FILE *fstabf3 = NULL;		/* MNTTAB */
 
 int open_fstab()
 {
     close_fstab();
 #if defined(HAVE_SETMNTENT)
     fstabf1 = setmntent("/proc/mounts", "r");
-    fstabf2 = setmntent(MNTTAB, "r");
+# if defined(MOUNTED)
+    fstabf2 = setmntent(MOUNTED, "r");
+# endif
+# if defined(MNTTAB)
+    fstabf3 = setmntent(MNTTAB, "r");
+# endif
 #else
-    fstabf2 = fopen(MNTTAB, "r");
+# if defined(MNTTAB)
+    fstabf3 = fopen(MNTTAB, "r");
+# endif
 #endif
-    return (fstabf1 != NULL || fstabf2 != NULL);
+    return (fstabf1 != NULL || fstabf2 != NULL || fstabf3 != NULL);
 }
 
 void close_fstab()
 {
-    afclose(fstabf1);
-    afclose(fstabf2);
+    if (fstabf1) {
+	AMCLOSE_MNTENT(fstabf1);
+	fstabf1 = NULL;
+    }
+    if (fstabf2) {
+	AMCLOSE_MNTENT(fstabf2);
+	fstabf2 = NULL;
+    }
+    if (fstabf3) {
+	AMCLOSE_MNTENT(fstabf3);
+	fstabf3 = NULL;
+    }
 }
 
 int get_fstab_nextentry(fsent)
@@ -172,13 +196,22 @@ generic_fsent_t *fsent;
     if(fstabf1) {
 	sys_fsent = getmntent(fstabf1);
 	if(!sys_fsent) {
-	    afclose(fstabf1);
+	    AMCLOSE_MNTENT(fstabf1);
+	    fstabf1 = NULL;
 	}
     }
     if(!sys_fsent && fstabf2) {
 	sys_fsent = getmntent(fstabf2);
 	if(!sys_fsent) {
-	    afclose(fstabf2);
+	    AMCLOSE_MNTENT(fstabf2);
+	    fstabf2 = NULL;
+	}
+    }
+    if(!sys_fsent && fstabf3) {
+	sys_fsent = getmntent(fstabf3);
+	if(!sys_fsent) {
+	    AMCLOSE_MNTENT(fstabf3);
+	    fstabf3 = NULL;
 	}
     }
     if(!sys_fsent) {
@@ -317,7 +350,7 @@ generic_fsent_t *fsent;
 {
     struct statfs fsd;
     char typebuf[FSTYPSZ];
-    struct mnttab mnt;
+    static struct mnttab mnt;
     char *dp, *ep;
 
     if(!fread (&mnt, sizeof mnt, 1, fstabf))
@@ -330,9 +363,10 @@ generic_fsent_t *fsent;
     if (statfs (fsent->mntdir, &fsd, sizeof fsd, 0) != -1
         && sysfs (GETFSTYP, fsd.f_fstyp, typebuf) != -1) {
        dp = typebuf;
-       ep = fsent->fstype;
+       ep = fsent->fstype = malloc(strlen(typebuf)+2);
        while (*dp)
             *ep++ = tolower(*dp++);
+       *ep=0;
     }
 
     if ( mnt.mt_ro_flg == MNT_READONLY ) {
@@ -427,10 +461,17 @@ struct stat stats[3], *estat;
   return 0;
 }
 
-int search_fstab(name, fsent)
-char *name;
-generic_fsent_t *fsent;
+int search_fstab(name, fsent, check_dev)
+     char *name;
+     generic_fsent_t *fsent;
+     int check_dev;
 {
+#ifdef IGNORE_FSTAB
+  /* There is no real mount table so this will always fail and
+   * we are using GNU tar so we can just return here.
+   */
+  return 0;
+#else
   struct stat stats[3];
   char *fullname = NULL;
   char *rdev = NULL;
@@ -476,8 +517,8 @@ generic_fsent_t *fsent;
     if(fsent->fsname != NULL) {
       sfs = stat(fsent->fsname, &fsstat);
       sfsr = stat((rdev = dev2rdev(fsent->fsname)), &fsrstat);
-      if(sfs == -1 && sfsr == -1)
-        continue;
+      if(check_dev == 1 && sfs == -1 && sfsr == -1)
+	continue;
     }
 
     if((fsent->mntdir != NULL &&
@@ -496,6 +537,7 @@ generic_fsent_t *fsent;
   amfree(rdev);
   close_fstab();
   return rc;
+#endif /* !IGNORE_FSTAB */
 }
 
 int is_local_fstype(fsent)
@@ -520,8 +562,9 @@ char *str;
 {
     generic_fsent_t fsent;
 
-    if(search_fstab(str, &fsent))
-      if (fsent.fsname != NULL)
+    if(search_fstab(str, &fsent, 1) && fsent.fsname != NULL)
+	str = fsent.fsname;
+    else if(search_fstab(str, &fsent, 0) && fsent.fsname != NULL)
 	str = fsent.fsname;
 
     return dev2rdev(str);
@@ -532,8 +575,9 @@ char *str;
 {
     generic_fsent_t fsent;
 
-    if(search_fstab(str, &fsent))
-      if (fsent.mntdir != NULL)
+    if(search_fstab(str, &fsent, 1) && fsent.mntdir != NULL)
+	str = fsent.mntdir;
+    else if(search_fstab(str, &fsent, 0) && fsent.mntdir != NULL)
 	str = fsent.mntdir;
 
     return stralloc(str);
@@ -544,7 +588,7 @@ char *str;
 {
     generic_fsent_t fsent;
 
-    if (!search_fstab(str, &fsent))
+    if (!search_fstab(str, &fsent, 1) && !search_fstab(str, &fsent, 0))
       return stralloc("");
 
     return stralloc(fsent.fstype);
@@ -603,7 +647,7 @@ int main(argc, argv)
     close_fstab();
 
     name = newstralloc(name, "/usr");
-    if(search_fstab(name, &fsent)) {
+    if(search_fstab(name, &fsent, 1) || search_fstab(name, &fsent, 0)) {
 	printf("Found %s mount for %s:\n",
 	       is_local_fstype(&fsent)? "local" : "remote", name);
 	print_entry(&fsent);
@@ -612,7 +656,7 @@ int main(argc, argv)
 	printf("Mount for %s not found\n", name);
 
     name = newstralloc(name, "/");
-    if(search_fstab(name, &fsent)) {
+    if(search_fstab(name, &fsent, 1) || search_fstab(name, &fsent, 0)) {
 	printf("Found %s mount for %s:\n",
 	       is_local_fstype(&fsent)? "local" : "remote", name);
 	print_entry(&fsent);
