@@ -25,18 +25,21 @@
  *			   University of Maryland at College Park
  */
 /* 
- * $Id: selfcheck.c,v 1.40.2.4 2001/03/15 02:17:34 jrjackson Exp $
+ * $Id: selfcheck.c,v 1.40.2.3.4.4.2.22 2003/11/18 18:04:26 martinea Exp $
  *
  * do self-check and send back any error messages
  */
 
 #include "amanda.h"
+#include "clock.h"
 #include "statfs.h"
 #include "version.h"
 #include "getfsent.h"
 #include "amandates.h"
 #include "util.h"
 #include "pipespawn.h"
+#include "amfeatures.h"
+#include "client_util.h"
 
 #ifdef SAMBA_CLIENT
 #include "findpass.h"
@@ -56,12 +59,17 @@ int need_runtar=0;
 int need_gnutar=0;
 int need_compress_path=0;
 
+static am_feature_t *our_features = NULL;
+static char *our_feature_string = NULL;
+static g_option_t *g_options = NULL;
+
 /* local functions */
 int main P((int argc, char **argv));
 
-static void check_options P((char *program, char *disk, char *str));
-static void check_disk P((char *program, char *disk, int level));
+static void check_options P((char *program, char *disk, char *device, option_t *options));
+static void check_disk P((char *program, char *disk, char *amdevice, int level));
 static void check_overall P((void));
+static void check_access P((char *filename, int mode));
 static void check_file P((char *filename, int mode));
 static void check_dir P((char *dirname, int mode));
 static void check_suid P((char *filename));
@@ -75,12 +83,15 @@ char **argv;
     char *line = NULL;
     char *program = NULL;
     char *disk = NULL;
+    char *device = NULL;
     char *optstr = NULL;
-    char *s;
+    char *err_extra = NULL;
+    char *s, *fp;
     int ch;
     int fd;
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
+    option_t *options;
 
     /* initialize */
 
@@ -102,16 +113,34 @@ char **argv;
 
     erroutput_type = (ERR_INTERACTIVE|ERR_SYSLOG);
     dbopen();
+    startclock();
     dbprintf(("%s: version %s\n", argv[0], version()));
 
+    our_features = am_init_feature_set();
+    our_feature_string = am_feature_to_string(our_features);
+
     /* handle all service requests */
 
     for(; (line = agets(stdin)) != NULL; free(line)) {
-#define sc "OPTIONS"
+#define sc "OPTIONS "
 	if(strncmp(line, sc, sizeof(sc)-1) == 0) {
 #undef sc
-	    /* we don't recognize any options yet */
-	    printf("OPTIONS ;\n");
+	    g_options = parse_g_options(line+8, 1);
+	    if(!g_options->hostname) {
+		g_options->hostname = alloc(MAX_HOSTNAME_LENGTH+1);
+		gethostname(g_options->hostname, MAX_HOSTNAME_LENGTH);
+		g_options->hostname[MAX_HOSTNAME_LENGTH] = '\0';
+	    }
+
+	    printf("OPTIONS ");
+	    if(am_has_feature(g_options->features, fe_rep_options_features)) {
+		printf("features=%s;", our_feature_string);
+	    }
+	    if(am_has_feature(g_options->features, fe_rep_options_hostname)) {
+		printf("hostname=%s;", g_options->hostname);
+	    }
+	    printf("\n");
+	    fflush(stdout);
 	    continue;
 	}
 
@@ -134,14 +163,29 @@ char **argv;
 	skip_non_whitespace(s, ch);
 	s[-1] = '\0';				/* terminate the disk name */
 
-	skip_whitespace(s, ch);			/* find level number */
+	skip_whitespace(s, ch);                 /* find the device or level */
+	if (ch == '\0') {
+	    goto err;				/* no device or level */
+	}
+	if(!isdigit((int)s[-1])) {
+	    fp = s - 1;
+	    skip_non_whitespace(s, ch);
+	    s[-1] = '\0';			/* terminate the device */
+	    device = stralloc(fp);
+	    skip_whitespace(s, ch);		/* find level number */
+	}
+	else {
+	    device = stralloc(disk);
+	}
+
+						/* find level number */
 	if (ch == '\0' || sscanf(s - 1, "%d", &level) != 1) {
 	    goto err;				/* bad level */
 	}
 	skip_integer(s, ch);
 
-#define sc "OPTIONS"
 	skip_whitespace(s, ch);
+#define sc "OPTIONS "
 	if (ch && strncmp (s - 1, sc, sizeof(sc)-1) == 0) {
 	    s += sizeof(sc)-1;
 	    ch = s[-1];
@@ -153,8 +197,15 @@ char **argv;
 	    optstr = s - 1;
 	    skip_non_whitespace(s, ch);
 	    s[-1] = '\0';			/* terminate the options */
-	    check_options(program, disk, optstr);
-	    check_disk(program, disk, level);
+	    options = parse_options(optstr, disk, device, g_options->features, 1);
+	    check_options(program, disk, device, options);
+	    check_disk(program, disk, device, level);
+	    free_sl(options->exclude_file);
+	    free_sl(options->exclude_list);
+	    free_sl(options->include_file);
+	    free_sl(options->include_list);
+	    amfree(options->str);
+	    amfree(options);
 	} else if (ch == '\0') {
 	    /* check all since no option */
 	    need_samba=1;
@@ -170,23 +221,32 @@ char **argv;
 	    need_runtar=1;
 	    need_gnutar=1;
 	    need_compress_path=1;
-	    check_disk(program, disk, level);
+	    check_disk(program, disk, device, level);
 	} else {
 	    goto err;				/* bad syntax */
 	}
+	amfree(device);
     }
 
     check_overall();
 
     amfree(line);
+    amfree(our_feature_string);
+    am_release_feature_set(our_features);
+    our_features = NULL;
+    am_release_feature_set(g_options->features);
+    g_options->features = NULL;
+    amfree(g_options->str);
+    amfree(g_options->hostname);
+    amfree(g_options);
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
     if(malloc_size_1 != malloc_size_2) {
 #if defined(USE_DBMALLOC)
-	extern int db_fd;
+	extern int dbfd;
 
-	malloc_list(db_fd, malloc_hist_1, malloc_hist_2);
+	malloc_list(dbfd(), malloc_hist_1, malloc_hist_2);
 #endif
     }
 
@@ -194,30 +254,73 @@ char **argv;
     return 0;
 
  err:
-    amfree(line);
     printf("ERROR [BOGUS REQUEST PACKET]\n");
-    dbprintf(("REQ packet is bogus\n"));
+    dbprintf(("%s: REQ packet is bogus%s%s\n",
+	      debug_prefix_time(NULL),
+	      err_extra ? ": " : "",
+	      err_extra ? err_extra : ""));
     dbclose();
     return 1;
 }
 
 
 static void
-check_options(program, disk, str)
-     char *program, *disk, *str;
+check_options(program, disk, device, options)
+    char *program, *disk, *device;
+    option_t *options;
 {
-    int as_index = 0;
-
-    if(strstr(str,"index") != NULL)
-	as_index=1;
     if(strcmp(program,"GNUTAR") == 0) {
 	need_gnutar=1;
-        if(disk[0] == '/' && disk[1] == '/')
-	  need_samba=1;
-	else
-	  need_runtar=1;
+        if(disk[0] == '/' && disk[1] == '/') {
+	    if(options->exclude_file && options->exclude_file->nb_element > 1) {
+		printf("ERROR [samba support only one exclude file]\n");
+	    }
+	    if(options->exclude_list && options->exclude_list->nb_element > 0 &&
+	       options->exclude_optional==0) {
+		printf("ERROR [samba does not support exclude list]\n");
+	    }
+	    if(options->include_file && options->include_file->nb_element > 0) {
+		printf("ERROR [samba does not support include file]\n");
+	    }
+	    if(options->include_list && options->include_list->nb_element > 0 &&
+	       options->include_optional==0) {
+		printf("ERROR [samba does not support include list]\n");
+	    }
+	    need_samba=1;
+	}
+	else {
+	    int nb_exclude = 0;
+	    int nb_include = 0;
+	    char *file_exclude = NULL;
+	    char *file_include = NULL;
+
+	    if(options->exclude_file) nb_exclude += options->exclude_file->nb_element;
+	    if(options->exclude_list) nb_exclude += options->exclude_list->nb_element;
+	    if(options->include_file) nb_include += options->include_file->nb_element;
+	    if(options->include_list) nb_include += options->include_list->nb_element;
+
+	    if(nb_exclude > 0) file_exclude = build_exclude(disk, device, options, 1);
+	    if(nb_include > 0) file_include = build_include(disk, device, options, 1);
+
+	    amfree(file_exclude);
+	    amfree(file_include);
+
+	    need_runtar=1;
+	}
     }
     if(strcmp(program,"DUMP") == 0) {
+	if(options->exclude_file && options->exclude_file->nb_element > 0) {
+	    printf("ERROR [DUMP does not support exclude file]\n");
+	}
+	if(options->exclude_list && options->exclude_list->nb_element > 0) {
+	    printf("ERROR [DUMP does not support exclude list]\n");
+	}
+	if(options->include_file && options->include_file->nb_element > 0) {
+	    printf("ERROR [DUMP does not support include file]\n");
+	}
+	if(options->include_list && options->include_list->nb_element > 0) {
+	    printf("ERROR [DUMP does not support include list]\n");
+	}
 #ifdef USE_RUNDUMP
 	need_rundump=1;
 #endif
@@ -231,7 +334,7 @@ check_options(program, disk, str)
 	{
 	    need_vdump=1;
 	    need_rundump=1;
-	    if (as_index)
+	    if (options->createindex)
 		need_vrestore=1;
 	}
 	else
@@ -245,7 +348,7 @@ check_options(program, disk, str)
 	{
 	    need_xfsdump=1;
 	    need_rundump=1;
-	    if (as_index)
+	    if (options->createindex)
 		need_xfsrestore=1;
 	}
 	else
@@ -258,50 +361,50 @@ check_options(program, disk, str)
 #endif
 	{
 	    need_vxdump=1;
-	    if (as_index)
+	    if (options->createindex)
 		need_vxrestore=1;
 	}
 	else
 #endif /* VXDUMP */
 	{
 	    need_dump=1;
-	    if (as_index)
+	    if (options->createindex)
 		need_restore=1;
 	}
 #else
 	/* AIX backup program */
 	need_dump=1;
-	if (as_index)
+	if (options->createindex)
 	    need_restore=1;
 #endif
     }
-    if(strstr(str, "compress") != NULL)
+    if(options->compress == COMPR_BEST || options->compress == COMPR_FAST) 
 	need_compress_path=1;
-    if(strstr(str, "index") != NULL) {
-	/* do nothing */
-    }
 }
 
-static void check_disk(program, disk, level)
-char *program, *disk;
+static void check_disk(program, disk, amdevice, level)
+char *program, *disk, *amdevice;
 int level;
 {
     char *device = NULL;
     char *err = NULL;
     char *user_and_password = NULL, *domain = NULL;
+    char *share = NULL, *subdir = NULL;
+    int lpass = 0;
     int amode;
     int access_result;
     char *access_type;
     char *extra_info = NULL;
 
-    dbprintf(("%s: checking disk %s\n", get_pname(), disk));
+    dbprintf(("%s: checking disk %s\n", debug_prefix_time(NULL), disk));
 
     if (strcmp(program, "GNUTAR") == 0) {
-        if(disk[0] == '/' && disk[1] == '/') {
+        if(amdevice[0] == '/' && amdevice[1] == '/') {
 #ifdef SAMBA_CLIENT
 	    int nullfd, checkerr;
 	    int passwdfd;
 	    char *pwtext;
+	    int pwtext_len;
 	    int checkpid;
 	    amwait_t retstat;
 	    char number[NUM_STR_SIZE];
@@ -310,31 +413,57 @@ int level;
 	    char *line;
 	    char *sep;
 	    FILE *ferr;
+	    char *pw_fd_env;
+	    int errdos;
 
-	    if ((user_and_password = findpass(disk, &domain)) == NULL) {
-		err = stralloc2("cannot find password for ", disk);
+	    parsesharename(amdevice, &share, &subdir);
+	    if (!share) {
+		err = stralloc2("cannot parse for share/subdir disk entry ", amdevice);
 		goto common_exit;
 	    }
+	    if ((subdir) && (SAMBA_VERSION < 2)) {
+		err = vstralloc("subdirectory specified for share '",
+				amdevice,
+				"' but samba not v2 or better",
+				NULL);
+		goto common_exit;
+	    }
+	    if ((user_and_password = findpass(share, &domain)) == NULL) {
+		err = stralloc2("cannot find password for ", amdevice);
+		goto common_exit;
+	    }
+	    lpass = strlen(user_and_password);
 	    if ((pwtext = strchr(user_and_password, '%')) == NULL) {
-		err = stralloc2("password field not \'user%pass\' for ", disk);
+		err = stralloc2("password field not \'user%pass\' for ", amdevice);
 		goto common_exit;
 	    }
 	    *pwtext++ = '\0';
-	    if ((device = makesharename(disk, 0)) == NULL) {
-		err = stralloc2("cannot make share name of ", disk);
+	    pwtext_len = strlen(pwtext);
+	    if ((device = makesharename(share, 0)) == NULL) {
+		err = stralloc2("cannot make share name of ", share);
 		goto common_exit;
 	    }
 
 	    nullfd = open("/dev/null", O_RDWR);
+	    if (pwtext_len > 0) {
+		pw_fd_env = "PASSWD_FD";
+	    } else {
+		pw_fd_env = "dummy_PASSWD_FD";
+	    }
 	    checkpid = pipespawn(SAMBA_CLIENT, STDERR_PIPE|PASSWD_PIPE,
 				 &nullfd, &nullfd, &checkerr,
-				 "PASSWD_FD", &passwdfd,
+				 pw_fd_env, &passwdfd,
 				 "smbclient",
 				 device,
-				 "-U", user_and_password,	/* user only */
+				 *user_and_password ? "-U" : skip_argument,
+				 *user_and_password ? user_and_password : skip_argument,
 				 "-E",
 				 domain ? "-W" : skip_argument,
 				 domain ? domain : skip_argument,
+#if SAMBA_VERSION >= 2
+				 subdir ? "-D" : skip_argument,
+				 subdir ? subdir : skip_argument,
+#endif
 				 "-c", "quit",
 				 NULL);
 	    if (domain) {
@@ -342,24 +471,28 @@ int level;
 		amfree(domain);
 	    }
 	    aclose(nullfd);
-	    if (fullwrite(passwdfd, pwtext, strlen(pwtext)) < 0) {
+	    if (pwtext_len > 0 && fullwrite(passwdfd, pwtext, pwtext_len) < 0) {
 		err = vstralloc("password write failed: ",
-				disk,
+				amdevice,
 				": ",
 				strerror(errno),
 				NULL);
 		aclose(passwdfd);
 		goto common_exit;
 	    }
-	    memset(user_and_password, '\0', strlen(user_and_password));
+	    memset(user_and_password, '\0', lpass);
 	    amfree(user_and_password);
 	    aclose(passwdfd);
 	    ferr = fdopen(checkerr, "r");
 	    sep = "";
+	    errdos = 0;
 	    for(sep = ""; (line = agets(ferr)) != NULL; free(line)) {
 		strappend(extra_info, sep);
 		strappend(extra_info, line);
 		sep = ": ";
+		if(strstr(line, "ERRDOS") != NULL) {
+		    errdos = 1;
+		}
 	    }
 	    afclose(ferr);
 	    checkerr = -1;
@@ -384,10 +517,10 @@ int level;
 		    strappend(err, number);
 		}
 	    }
-	    if (rc != 0) {
+	    if (errdos != 0 || rc != 0) {
 		err = newvstralloc(err,
 				   "samba access error: ",
-				   disk,
+				   amdevice,
 				   ": ",
 				   extra_info ? extra_info : "",
 				   err,
@@ -395,33 +528,33 @@ int level;
 		amfree(extra_info);
 	    }
 #else
-	    err = stralloc2("This client is not configured for samba: ", disk);
+	    err = stralloc2("This client is not configured for samba: ", amdevice);
 #endif
 	    goto common_exit;
 	}
 	amode = F_OK;
-	device = amname_to_dirname(disk);
+	device = amname_to_dirname(amdevice);
     } else {
-	if(disk[0] == '/' && disk[1] == '/') {
+	if(amdevice[0] == '/' && amdevice[1] == '/') {
 	    err = vstralloc("The DUMP program cannot handle samba shares,",
 			    " use GNUTAR: ",
-			    disk,
+			    amdevice,
 			    NULL);
 	    goto common_exit;
 	}
 #ifdef VDUMP								/* { */
 #ifdef DUMP								/* { */
-        if (strcmp(amname_to_fstype(disk), "advfs") == 0)
+        if (strcmp(amname_to_fstype(amdevice), "advfs") == 0)
 #else									/* }{ */
 	if (1)
 #endif									/* } */
 	{
-	    device = amname_to_dirname(disk);
+	    device = amname_to_dirname(amdevice);
 	    amode = F_OK;
 	} else
 #endif									/* } */
 	{
-	    device = amname_to_devname(disk);
+	    device = amname_to_devname(amdevice);
 #ifdef USE_RUNDUMP
 	    amode = F_OK;
 #else
@@ -430,27 +563,32 @@ int level;
 	}
     }
 
-    dbprintf(("%s: device %s\n", get_pname(), device));
+    dbprintf(("%s: device %s\n", debug_prefix_time(NULL), device));
 
+    /* skip accessability test if this is an AFS entry */
+    if(strncmp(device, "afs:", 4) != 0) {
 #ifdef CHECK_FOR_ACCESS_WITH_OPEN
-    access_result = open(device, O_RDONLY);
-    access_type = "open";
+	access_result = open(device, O_RDONLY);
+	access_type = "open";
 #else
-    access_result = access(device, amode);
-    access_type = "access";
+	access_result = access(device, amode);
+	access_type = "access";
 #endif
-    if(access_result == -1) {
-	err = vstralloc("could not ", access_type, " ", device,
+	if(access_result == -1) {
+	    err = vstralloc("could not ", access_type, " ", device,
 			" (", disk, "): ", strerror(errno), NULL);
-    }
+	}
 #ifdef CHECK_FOR_ACCESS_WITH_OPEN
-    aclose(access_result);
+	aclose(access_result);
 #endif
+    }
 
 common_exit:
 
+    amfree(share);
+    amfree(subdir);
     if(user_and_password) {
-	memset(user_and_password, '\0', strlen(user_and_password));
+	memset(user_and_password, '\0', lpass);
 	amfree(user_and_password);
     }
     if(domain) {
@@ -460,14 +598,19 @@ common_exit:
 
     if(err) {
 	printf("ERROR [%s]\n", err);
-	dbprintf(("%s: %s\n", get_pname(), err));
+	dbprintf(("%s: %s\n", debug_prefix_time(NULL), err));
 	amfree(err);
     } else {
+	printf("OK %s\n", disk);
+	dbprintf(("%s: disk \"%s\" OK\n", debug_prefix_time(NULL), disk));
+	printf("OK %s\n", amdevice);
+	dbprintf(("%s: amdevice \"%s\" OK\n",
+		  debug_prefix_time(NULL), amdevice));
 	printf("OK %s\n", device);
-	dbprintf(("%s: OK\n", get_pname()));
+	dbprintf(("%s: device \"%s\" OK\n", debug_prefix_time(NULL), device));
     }
     if(extra_info) {
-	dbprintf(("%s: extra info: %s\n", get_pname(), extra_info));
+	dbprintf(("%s: extra info: %s\n", debug_prefix_time(NULL), extra_info));
 	amfree(extra_info);
     }
     amfree(device);
@@ -617,7 +760,7 @@ static void check_overall()
         check_file("/etc/vdumpdates", F_OK);
     }
 
-    check_file("/dev/null", R_OK|W_OK);
+    check_access("/dev/null", R_OK|W_OK);
     check_space(AMANDA_TMPDIR, 64);	/* for amandad i/o */
 
 #ifdef AMANDA_DBGDIR
@@ -642,7 +785,7 @@ long kbytes;
 	printf("OK %s has more than %ld KB available.\n", dir, kbytes);
 }
 
-static void check_file(filename, mode)
+static void check_access(filename, mode)
 char *filename;
 int mode;
 {
@@ -663,18 +806,41 @@ int mode;
 	printf("OK %s %s\n", filename, adjective);
 }
 
+static void check_file(filename, mode)
+char *filename;
+int mode;
+{
+    struct stat stat_buf;
+    if(!stat(filename, &stat_buf)) {
+	if(!S_ISREG(stat_buf.st_mode)) {
+	    printf("ERROR [%s is not a file]\n", filename);
+	}
+    }
+    check_access(filename, mode);
+}
+
 static void check_dir(dirname, mode)
 char *dirname;
 int mode;
 {
-    char *dir = stralloc2(dirname, "/.");
-    check_file(dir, mode);
+    struct stat stat_buf;
+    char *dir;
+
+    if(!stat(dirname, &stat_buf)) {
+	if(!S_ISDIR(stat_buf.st_mode)) {
+	    printf("ERROR [%s is not a directory]\n", dirname);
+	}
+    }
+    dir = stralloc2(dirname, "/.");
+    check_access(dir, mode);
     amfree(dir);
 }
 
 static void check_suid(filename)
 char *filename;
 {
+/* The following is only valid for real Unixs */
+#ifndef IGNORE_UID_CHECK
     struct stat stat_buf;
     if(!stat(filename, &stat_buf)) {
 	if(stat_buf.st_uid != 0 ) {
@@ -687,4 +853,5 @@ char *filename;
     else {
 	printf("ERROR [can not stat %s]\n",filename);
     }
+#endif
 }
