@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id: sendbackup-gnutar.c,v 1.56.2.16 2001/03/15 02:17:34 jrjackson Exp $
+ * $Id: sendbackup-gnutar.c,v 1.56.2.15.4.4.2.11 2003/10/30 18:04:45 martinea Exp $
  *
  * send backup data using GNU tar
  */
@@ -32,6 +32,7 @@
 #include "amanda.h"
 #include "sendbackup.h"
 #include "amandates.h"
+#include "clock.h"
 #include "util.h"
 #include "getfsent.h"			/* for amname_to_dirname lookup */
 #include "version.h"
@@ -49,74 +50,78 @@
 
 static regex_t re_table[] = {
   /* tar prints the size in bytes */
-  { DMP_SIZE, 
-	"^Total bytes written: [0-9][0-9]*",				1},
+  AM_SIZE_RE("^ *Total bytes written: [0-9][0-9]*", 1),
 
-  { DMP_NORMAL, "^Elapsed time:", 1 },
-  { DMP_NORMAL, "^Throughput", 1 },
+  AM_NORMAL_RE("^Elapsed time:"),
+  AM_NORMAL_RE("^Throughput"),
 
   /* GNU tar 1.13.17 will print this warning when (not) backing up a
      Unix named socket.  */
-  { DMP_NORMAL, ": socket ignored$", 1 },
+  AM_NORMAL_RE(": socket ignored$"),
 
   /* GNUTAR produces a few error messages when files are modified or
      removed while it is running.  They may cause data to be lost, but
      then they may not.  We shouldn't consider them NORMAL until
      further investigation.  */
 #ifdef IGNORE_TAR_ERRORS
-  { DMP_NORMAL, ": File .* shrunk by [0-9][0-9]* bytes, padding with zeros", 1 },
-  { DMP_NORMAL, ": Cannot add file .*: No such file or directory$", 1},
-  { DMP_NORMAL, ": Error exit delayed from previous errors", 1},
+  AM_NORMAL_RE(": File .* shrunk by [0-9][0-9]* bytes, padding with zeros"),
+  AM_NORMAL_RE(": Cannot add file .*: No such file or directory$"),
+  AM_NORMAL_RE(": Error exit delayed from previous errors"),
 #endif
   
   /* samba may produce these output messages */
-  { DMP_NORMAL, "^[Aa]dded interface", 1},
-  { DMP_NORMAL, "^session request to ", 1},
-  { DMP_NORMAL, "^tar: dumped [0-9][0-9]* (tar )?files", 1},
+  AM_NORMAL_RE("^[Aa]dded interface"),
+  AM_NORMAL_RE("^session request to "),
+  AM_NORMAL_RE("^tar: dumped [0-9][0-9]* (tar )?files"),
 
 #if SAMBA_VERSION < 2
-  { DMP_NORMAL, "^doing parameter", 1},
-  { DMP_NORMAL, "^pm_process\\(\\)", 1},
-  { DMP_NORMAL, "^adding IPC", 1},
-  { DMP_NORMAL, "^Opening", 1},
-  { DMP_NORMAL, "^Connect", 1},
-  { DMP_NORMAL, "^Domain=", 1},
-  { DMP_NORMAL, "^max", 1},
-  { DMP_NORMAL, "^security=", 1},
-  { DMP_NORMAL, "^capabilities", 1},
-  { DMP_NORMAL, "^Sec mode ", 1},
-  { DMP_NORMAL, "^Got ", 1},
-  { DMP_NORMAL, "^Chose protocol ", 1},
-  { DMP_NORMAL, "^Server ", 1},
-  { DMP_NORMAL, "^Timezone ", 1},
-  { DMP_NORMAL, "^received", 1},
-  { DMP_NORMAL, "^FINDFIRST", 1},
-  { DMP_NORMAL, "^FINDNEXT", 1},
-  { DMP_NORMAL, "^dos_clean_name", 1},
-  { DMP_NORMAL, "^file", 1},
-  { DMP_NORMAL, "^getting file", 1},
-  { DMP_NORMAL, "^Rejected chained", 1},
-  { DMP_NORMAL, "^nread=", 1},
-  { DMP_NORMAL, "^\\([0-9][0-9]* kb/s\\)", 1},
-  { DMP_NORMAL, "^\\([0-9][0-9]*\\.[0-9][0-9]* kb/s\\)", 1},
-  { DMP_NORMAL, "^[ \t]*[0-9][0-9]* \\([ \t]*[0-9][0-9]*\\.[0-9][0-9]* kb/s\\)", 1},
-  { DMP_NORMAL, "^[ \t]*directory ", 1},
-  { DMP_NORMAL, "^load_client_codepage", 1},
+  AM_NORMAL_RE("^doing parameter"),
+  AM_NORMAL_RE("^pm_process\\(\\)"),
+  AM_NORMAL_RE("^adding IPC"),
+  AM_NORMAL_RE("^Opening"),
+  AM_NORMAL_RE("^Connect"),
+  AM_NORMAL_RE("^Domain="),
+  AM_NORMAL_RE("^max"),
+  AM_NORMAL_RE("^security="),
+  AM_NORMAL_RE("^capabilities"),
+  AM_NORMAL_RE("^Sec mode "),
+  AM_NORMAL_RE("^Got "),
+  AM_NORMAL_RE("^Chose protocol "),
+  AM_NORMAL_RE("^Server "),
+  AM_NORMAL_RE("^Timezone "),
+  AM_NORMAL_RE("^received"),
+  AM_NORMAL_RE("^FINDFIRST"),
+  AM_NORMAL_RE("^FINDNEXT"),
+  AM_NORMAL_RE("^dos_clean_name"),
+  AM_NORMAL_RE("^file"),
+  AM_NORMAL_RE("^getting file"),
+  AM_NORMAL_RE("^Rejected chained"),
+  AM_NORMAL_RE("^nread="),
+  AM_NORMAL_RE("^\\([0-9][0-9]* kb/s\\)"),
+  AM_NORMAL_RE("^\\([0-9][0-9]*\\.[0-9][0-9]* kb/s\\)"),
+  AM_NORMAL_RE("^[ \t]*[0-9][0-9]* \\([ \t]*[0-9][0-9]*\\.[0-9][0-9]* kb/s\\)"),
+  AM_NORMAL_RE("^[ \t]*directory "),
+  AM_NORMAL_RE("^load_client_codepage"),
 #endif
 
 #ifdef IGNORE_SMBCLIENT_ERRORS
   /* This will cause amanda to ignore real errors, but that may be
    * unavoidable when you're backing up system disks.  It seems to be
    * a safe thing to do if you know what you're doing.  */
-  { DMP_NORMAL, "^ERRDOS - ERRbadshare opening remote file", 1},
-  { DMP_NORMAL, "^ERRDOS - ERRbadfile opening remote file", 1},
-  { DMP_NORMAL, "^ERRDOS - ERRnoaccess opening remote file", 1},
-  { DMP_NORMAL, "^ERRSRV - ERRaccess setting attributes on file", 1},
-  { DMP_NORMAL, "^ERRDOS - ERRnoaccess setting attributes on file", 1},
+  AM_NORMAL_RE("^ERRDOS - ERRbadshare opening remote file"),
+  AM_NORMAL_RE("^ERRDOS - ERRbadfile opening remote file"),
+  AM_NORMAL_RE("^ERRDOS - ERRnoaccess opening remote file"),
+  AM_NORMAL_RE("^ERRSRV - ERRaccess setting attributes on file"),
+  AM_NORMAL_RE("^ERRDOS - ERRnoaccess setting attributes on file"),
+#endif
+
+#if SAMBA_VERSION >= 2
+  /* Backup attempt of nonexisting directory */
+  AM_ERROR_RE("ERRDOS - ERRbadpath (Directory invalid.)"),
 #endif
 
   /* catch-all: DMP_STRANGE is returned for all other lines */
-  { DMP_STRANGE, NULL, 0}
+  AM_STRANGE_RE(NULL)
 };
 
 int cur_level;
@@ -127,9 +132,9 @@ time_t cur_dumptime;
 static char *incrname = NULL;
 #endif
 
-static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
+static void start_backup(host, disk, amdevice, level, dumpdate, dataf, mesgf, indexf)
     char *host;
-    char *disk;
+    char *disk, *amdevice;
     int level, dataf, mesgf, indexf;
     char *dumpdate;
 {
@@ -142,17 +147,20 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
     struct tm *gmtm;
     amandates_t *amdates;
     time_t prev_dumptime;
+    char *error_pn = NULL;
+
+    error_pn = stralloc2(get_pname(), "-smbclient");
 
     fprintf(stderr, "%s: start [%s:%s level %d]\n",
 	    get_pname(), host, disk, level);
 
     NAUGHTY_BITS;
 
-    if(compress) {
+    if(options->compress == COMPR_FAST || options->compress == COMPR_BEST) {
 	char *compopt = skip_argument;
 
 #if defined(COMPRESS_BEST_OPT) && defined(COMPRESS_FAST_OPT)
-	if(compress == COMPR_BEST) {
+	if(options->compress == COMPR_BEST) {
 	    compopt = COMPRESS_BEST_OPT;
 	} else {
 	    compopt = COMPRESS_FAST_OPT;
@@ -161,8 +169,8 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
 	comppid = pipespawn(COMPRESS_PATH, STDIN_PIPE,
 			    &dumpout, &dataf, &mesgf,
 			    COMPRESS_PATH, compopt, NULL);
-	dbprintf(("%s-gnutar: pid %ld: %s",
-		  get_pname(), (long)comppid, COMPRESS_PATH));
+	dbprintf(("%s: pid %ld: %s",
+		  debug_prefix_time("-gnutar"), (long)comppid, COMPRESS_PATH));
 	if(compopt != skip_argument) {
 	    dbprintf((" %s", compopt));
 	}
@@ -174,7 +182,7 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
 
 #ifdef GNUTAR_LISTED_INCREMENTAL_DIR					/* { */
 #ifdef SAMBA_CLIENT							/* { */
-    if (disk[0] == '/' && disk[1]=='/')
+    if (amdevice[0] == '/' && amdevice[1]=='/')
 	amfree(incrname);
     else
 #endif									/* } */
@@ -224,12 +232,12 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
 	    if ((in = fopen(inputname, "r")) == NULL) {
 		int save_errno = errno;
 
-		dbprintf(("%s-gnutar: error opening %s: %s\n",
-			  get_pname(),
+		dbprintf(("%s: error opening %s: %s\n",
+			  debug_prefix_time("-gnutar"),
 			  inputname,
 			  strerror(save_errno)));
 		if (baselevel < 0) {
-		    error("error [opening %s: %s]", inputname, strerror(errno));
+		    error("error [opening %s: %s]", inputname, strerror(save_errno));
 		}
 	    }
 	}
@@ -260,8 +268,8 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
 	}
 	out = NULL;
 
-	dbprintf(("%s-gnutar: doing level %d dump as listed-incremental",
-		  get_pname(), level));
+	dbprintf(("%s: doing level %d dump as listed-incremental",
+		  debug_prefix_time("-gnutar"), level));
 	if(baselevel >= 0) {
 	    dbprintf((" from %s", inputname));
 	}
@@ -293,10 +301,10 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
 		gmtm->tm_year + 1900, gmtm->tm_mon+1, gmtm->tm_mday,
 		gmtm->tm_hour, gmtm->tm_min, gmtm->tm_sec);
 
-    dbprintf(("%s-gnutar: doing level %d dump from date: %s\n",
-	      get_pname(), level, dumptimestr));
+    dbprintf(("%s: doing level %d dump from date: %s\n",
+	      debug_prefix_time("-gnutar"), level, dumptimestr));
 
-    dirname = amname_to_dirname(disk);
+    dirname = amname_to_dirname(amdevice);
 
     cur_dumptime = time(0);
     cur_level = level;
@@ -315,18 +323,38 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
 
 #ifdef SAMBA_CLIENT							/* { */
     /* Use sambatar if the disk to back up is a PC disk */
-   if (disk[0] == '/' && disk[1]=='/') {
+    if (amdevice[0] == '/' && amdevice[1]=='/') {
 	char *sharename = NULL, *user_and_password = NULL, *domain = NULL;
+	char *share = NULL, *subdir = NULL;
 	char *pwtext;
 	char *taropt;
 	int passwdf;
 	int lpass;
+	int pwtext_len;
+	char *pw_fd_env;
 
-	if ((user_and_password = findpass(disk, &domain)) == NULL) {
+	parsesharename(amdevice, &share, &subdir);
+	if (!share) {
+	     amfree(share);
+	     amfree(subdir);
+	     set_pname(error_pn);
+	     amfree(error_pn);
+	     error("cannot parse disk entry '%s' for share/subdir", disk);
+	}
+	if ((subdir) && (SAMBA_VERSION < 2)) {
+	     amfree(share);
+	     amfree(subdir);
+	     set_pname(error_pn);
+	     amfree(error_pn);
+	     error("subdirectory specified for share '%s' but samba not v2 or better", disk);
+	}
+	if ((user_and_password = findpass(share, &domain)) == NULL) {
 	    if(domain) {
 		memset(domain, '\0', strlen(domain));
 		amfree(domain);
 	    }
+	    set_pname(error_pn);
+	    amfree(error_pn);
 	    error("error [invalid samba host or password not found?]");
 	}
 	lpass = strlen(user_and_password);
@@ -337,22 +365,26 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
 		memset(domain, '\0', strlen(domain));
 		amfree(domain);
 	    }
-	    error("%s-smbtar: password field not \'user%%pass\' for %s",
-		  get_pname(), disk);
+	    set_pname(error_pn);
+	    amfree(error_pn);
+	    error("password field not \'user%%pass\' for %s", disk);
 	}
 	*pwtext++ = '\0';
-	if ((sharename = makesharename(disk, 0)) == 0) {
+	pwtext_len = strlen(pwtext);
+	if ((sharename = makesharename(share, 0)) == 0) {
 	    memset(user_and_password, '\0', lpass);
 	    amfree(user_and_password);
 	    if(domain) {
 		memset(domain, '\0', strlen(domain));
 		amfree(domain);
 	    }
-	    error("error [can't make share name of %s]", disk);
+	    set_pname(error_pn);
+	    amfree(error_pn);
+	    error("error [can't make share name of %s]", share);
 	}
 
 	taropt = stralloc("-T");
-	if (estr != NULL && *estr != '\0') {
+	if(options->exclude_file && options->exclude_file->nb_element == 1) {
 	    strappend(taropt, "X");
 	}
 #if SAMBA_VERSION >= 2
@@ -361,95 +393,149 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
 	strappend(taropt, "c");
 	if (level != 0) {
 	    strappend(taropt, "g");
-	} else if (!no_record) {
+	} else if (!options->no_record) {
 	    strappend(taropt, "a");
 	}
 
-	dbprintf(("%s-gnutar: backup of %s\n", get_pname(), sharename));
+	dbprintf(("%s: backup of %s", debug_prefix_time("-gnutar"), sharename));
+	if (subdir) {
+	    dbprintf(("/%s",subdir));
+	}
+	dbprintf(("\n"));
 
 	program->backup_name = program->restore_name = SAMBA_CLIENT;
 	cmd = stralloc(program->backup_name);
 	write_tapeheader();
 
-	start_index(createindex, dumpout, mesgf, indexf, indexcmd);
+	start_index(options->createindex, dumpout, mesgf, indexf, indexcmd);
 
+	if (pwtext_len > 0) {
+	    pw_fd_env = "PASSWD_FD";
+	} else {
+	    pw_fd_env = "dummy_PASSWD_FD";
+	}
 	dumppid = pipespawn(cmd, STDIN_PIPE|PASSWD_PIPE,
 			    &dumpin, &dumpout, &mesgf,
-			    "PASSWD_FD", &passwdf,
+			    pw_fd_env, &passwdf,
 			    "smbclient",
 			    sharename,
-			    "-U", user_and_password,		/* user only */
+			    *user_and_password ? "-U" : skip_argument,
+			    *user_and_password ? user_and_password : skip_argument,
 			    "-E",
 			    domain ? "-W" : skip_argument,
 			    domain ? domain : skip_argument,
+#if SAMBA_VERSION >= 2
+			    subdir ? "-D" : skip_argument,
+			    subdir ? subdir : skip_argument,
+#endif
 			    "-d0",
 			    taropt,
 			    "-",
-			    estr ? estr : skip_argument,
+			    options->exclude_file && options->exclude_file->nb_element == 1 ? options->exclude_file->first->name : skip_argument,
 			    NULL);
 	if(domain) {
 	    memset(domain, '\0', strlen(domain));
 	    amfree(domain);
 	}
-	if(fullwrite(passwdf, pwtext, strlen(pwtext)) < 0) {
+	if(pwtext_len > 0 && fullwrite(passwdf, pwtext, pwtext_len) < 0) {
 	    int save_errno = errno;
 
 	    aclose(passwdf);
 	    memset(user_and_password, '\0', lpass);
 	    amfree(user_and_password);
+	    set_pname(error_pn);
+	    amfree(error_pn);
 	    error("error [password write failed: %s]", strerror(save_errno));
 	}
 	memset(user_and_password, '\0', lpass);
 	amfree(user_and_password);
 	aclose(passwdf);
 	amfree(sharename);
+	amfree(share);
+	amfree(subdir);
 	amfree(taropt);
 	tarpid = dumppid;
     } else
 #endif									/* } */
     {
+	int nb_exclude = 0;
+	int nb_include = 0;
+	char **my_argv;
+	int i = 0;
+	char *file_exclude = NULL;
+	char *file_include = NULL;
+
+	if(options->exclude_file) nb_exclude+=options->exclude_file->nb_element;
+	if(options->exclude_list) nb_exclude+=options->exclude_list->nb_element;
+	if(options->include_file) nb_include+=options->include_file->nb_element;
+	if(options->include_list) nb_include+=options->include_list->nb_element;
+
+	if(nb_exclude > 0) file_exclude = build_exclude(disk, amdevice, options, 0);
+	if(nb_include > 0) file_include = build_include(disk, amdevice, options, 0);
+
+	my_argv = alloc(sizeof(char *) * (17 + (nb_exclude*2)+(nb_include*2)));
+
 	cmd = vstralloc(libexecdir, "/", "runtar", versionsuffix(), NULL);
 	write_tapeheader();
 
-	start_index(createindex, dumpout, mesgf, indexf, indexcmd);
+	start_index(options->createindex, dumpout, mesgf, indexf, indexcmd);
 
-	dumppid = pipespawn(cmd, STDIN_PIPE,
-			    &dumpin, &dumpout, &mesgf,
-			    "gtar",
-			    "--create",
-			    "--file", "-",
-			    "--directory", dirname,
-			    "--one-file-system",
+	my_argv[i++] = "gtar";
+	my_argv[i++] = "--create";
+	my_argv[i++] = "--file";
+	my_argv[i++] = "-";
+	my_argv[i++] = "--directory";
+	my_argv[i++] = dirname;
+	my_argv[i++] = "--one-file-system";
 #ifdef GNUTAR_LISTED_INCREMENTAL_DIR
-			    "--listed-incremental", incrname,
+	my_argv[i++] = "--listed-incremental";
+	my_argv[i++] = incrname;
 #else
-			    "--incremental",
-			    "--newer", dumptimestr,
-#endif
+	my_argv[i++] = "--incremental";
+	my_argv[i++] = "--newer";
+	my_argv[i++] = dumptimestr;
+#endif 
 #ifdef ENABLE_GNUTAR_ATIME_PRESERVE
-			    /* --atime-preserve causes gnutar to call
-			     * utime() after reading files in order to
-			     * adjust their atime.  However, utime()
-			     * updates the file's ctime, so incremental
-			     * dumps will think the file has changed. */
-			    "--atime-preserve",
+	/* --atime-preserve causes gnutar to call
+	 * utime() after reading files in order to
+	 * adjust their atime.  However, utime()
+	 * updates the file's ctime, so incremental
+	 * dumps will think the file has changed. */
+	my_argv[i++] = "--atime-preserve";
 #endif
-			    "--sparse",
-			    "--ignore-failed-read",
-			    "--totals",
-			    efile ? "--exclude-from" : skip_argument,
-			    efile ? efile : skip_argument,
-			    estr ? "--exclude" : skip_argument,
-			    estr ? estr : skip_argument,
-			    ".",
-			    NULL);
+	my_argv[i++] = "--sparse";
+	my_argv[i++] = "--ignore-failed-read";
+	my_argv[i++] = "--totals";
+
+	if(file_exclude) {
+	    my_argv[i++] = "--exclude-from";
+	    my_argv[i++] = file_exclude;
+	}
+
+	if(file_include) {
+	    my_argv[i++] = "--files-from";
+	    my_argv[i++] = file_include;
+	}
+	else {
+	    my_argv[i++] = ".";
+	}
+	my_argv[i++] = NULL;
+	dumppid = pipespawnv(cmd, STDIN_PIPE,
+			     &dumpin, &dumpout, &mesgf, my_argv);
 	tarpid = dumppid;
+	amfree(file_exclude);
+	amfree(file_include);
+	amfree(my_argv);
     }
-    dbprintf(("%s-gnutar: %s: pid %ld\n", get_pname(), cmd, (long)dumppid));
+    dbprintf(("%s: %s: pid %ld\n",
+	      debug_prefix_time("-gnutar"),
+	      cmd,
+	      (long)dumppid));
 
     amfree(dirname);
     amfree(cmd);
     amfree(indexcmd);
+    amfree(error_pn);
 
     /* close the write ends of the pipes */
 
@@ -457,14 +543,14 @@ static void start_backup(host, disk, level, dumpdate, dataf, mesgf, indexf)
     aclose(dumpout);
     aclose(dataf);
     aclose(mesgf);
-    if (createindex)
+    if (options->createindex)
 	aclose(indexf);
 }
 
 static void end_backup(goterror)
 int goterror;
 {
-    if(!no_record && !goterror) {
+    if(!options->no_record && !goterror) {
 #ifdef GNUTAR_LISTED_INCREMENTAL_DIR
       if (incrname != NULL && strlen(incrname) > 4) {
         char *nodotnew;
