@@ -24,14 +24,16 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id: sendbackup.c,v 1.44.2.11 2001/03/15 02:17:34 jrjackson Exp $
+ * $Id: sendbackup.c,v 1.44.2.9.4.4.2.15 2003/12/16 22:36:45 martinea Exp $
  *
  * common code for the sendbackup-* programs.
  */
 
 #include "amanda.h"
 #include "sendbackup.h"
+#include "clock.h"
 #include "pipespawn.h"
+#include "amfeatures.h"
 #include "stream.h"
 #include "arglist.h"
 #include "getfsent.h"
@@ -50,12 +52,7 @@ int data_socket, data_port, dataf;
 int mesg_socket, mesg_port, mesgf;
 int index_socket, index_port, indexf;
 
-char *efile = NULL;
-char *estr = NULL;
-int compress, no_record, bsd_auth;
-int createindex;
-#define COMPR_FAST 1
-#define COMPR_BEST 2
+option_t *options;
 
 #ifdef KRB4_SECURITY
 #include "sendbackup-krb4.h"
@@ -67,10 +64,13 @@ long dump_size = -1;
 
 backup_program_t *program = NULL;
 
+static am_feature_t *our_features = NULL;
+static char *our_feature_string = NULL;
+static g_option_t *g_options = NULL;
+
 /* local functions */
 int main P((int argc, char **argv));
-void parse_options P((char *str, char *disk));
-char *optionstr P((void));
+char *optionstr P((option_t *options));
 char *childstr P((int pid));
 int check_status P((int pid, amwait_t w));
 
@@ -79,81 +79,8 @@ int pipefork P((void (*func) P((void)), char *fname, int *stdinfd,
 void parse_backup_messages P((int mesgin));
 static void process_dumpline P((char *str));
 
-void parse_options(str, disk)
-char *str;
-char *disk;
-{
-    char *e, *i, *j, *k;
-    int ch;
 
-    /* only a few options, no need to get fancy */
-
-    if(strstr(str, "compress") != NULL) {
-	if(strstr(str, "compress-best") != NULL)
-	    compress = COMPR_BEST;
-	else
-	    compress = COMPR_FAST;	/* the default */
-    }
-
-    if((i=strstr(str, "exclude")) != NULL) {
-	if((j = strchr(i, '=')) != NULL && (k = strchr(j, ';')) != NULL) {
-	    j++;				/* advance to after the '=' */
-
-	    ch = k[1];				/* character after ';' */
-	    k[1] = '\0';
-	    estr = newstralloc(estr, i);	/* save the whole option */
-	    k[1] = ch;
-
-	    k[0] = '\0';			/* zap the ';' */
-
-#define sc "exclude-list"
-	    if(strncmp(sc, estr, sizeof(sc)-1) == 0) {
-		e = "-from";
-	    } else {
-		e = "";
-	    }
-#undef sc
-/* BEGIN HPS */
-	    if (*e != '\0')
-		{
-		  char *file = j;
-		  if(*file != '/')
-		  {
-			char *dirname = amname_to_dirname(disk);
-			efile = vstralloc(dirname,"/",file, NULL);
-		  }
-		  else
-			efile = stralloc(file);
-		  
-		  if(access(efile, F_OK) != 0) {
-			/* if exclude list file does not exist, ignore it.
-			 * Should not test for R_OK, because the file may be
-			 * readable by root only! */
-			dbprintf(("%s: exclude list file \"%s\" does not exist, ignoring\n",
-				  get_pname(), efile));
-			amfree(efile);
-		  }
-		  estr = NULL;
-	    } else
-/* END HPS */
-	    {
-		  estr = stralloc(j);
-		  amfree(efile);
-	    }
-	    k[0] = ';';
-	}
-    }
-
-    no_record = strstr(str, "no-record") != NULL;
-    bsd_auth = strstr(str, "bsd-auth") != NULL;
-#ifdef KRB4_SECURITY
-    krb4_auth = strstr(str, "krb4-auth") != NULL;
-    kencrypt = strstr(str, "kencrypt") != NULL;
-#endif
-    createindex = strstr(str, "index") != NULL;
-}
-
-char *optionstr()
+char *optionstr(option_t *options)
 {
     static char *optstr = NULL;
     char *compress_opt = "";
@@ -162,43 +89,66 @@ char *optionstr()
     char *krb4_opt = "";
     char *kencrypt_opt = "";
     char *index_opt = "";
+    char *exclude_file_opt;
+    char *exclude_list_opt;
+    char *exc = NULL;
+    sle_t *excl;
 
-    if(compress == COMPR_BEST)
+    if(options->compress == COMPR_BEST)
 	compress_opt = "compress-best;";
-    else if(compress == COMPR_FAST)
+    else if(options->compress == COMPR_FAST)
 	compress_opt = "compress-fast;";
-    if(no_record) record_opt = "no-record;";
-    if(bsd_auth) bsd_opt = "bsd-auth;";
+    else if(options->compress == COMPR_SERVER_BEST)
+	compress_opt = "srvcomp-best;";
+    else if(options->compress == COMPR_SERVER_FAST)
+	compress_opt = "srvcomp-fast;";
+    if(options->no_record) record_opt = "no-record;";
+    if(options->bsd_auth) bsd_opt = "bsd-auth;";
 #ifdef KRB4_SECURITY
-    if(krb4_auth) krb4_opt = "krb4-auth;";
-    if(kencrypt) kencrypt_opt = "kencrypt;";
+    if(options->krb4_auth) krb4_opt = "krb4-auth;";
+    if(options->kencrypt) kencrypt_opt = "kencrypt;";
 #endif
-    if(createindex) index_opt = "index;";
+    if(options->createindex) index_opt = "index;";
 
+    exclude_file_opt = stralloc("");
+    if(options->exclude_file) {
+	for(excl = options->exclude_file->first; excl != NULL; excl=excl->next){
+	    exc = newvstralloc(exc, "exclude-file=", excl->name, ";", NULL);
+	    strappend(exclude_file_opt, exc);
+	}
+    }
+    exclude_list_opt = stralloc("");
+    if(options->exclude_list) {
+	for(excl = options->exclude_list->first; excl != NULL; excl=excl->next){
+	    exc = newvstralloc(exc, "exclude-list=", excl->name, ";", NULL);
+	    strappend(exclude_list_opt, exc);
+	}
+    }
     optstr = newvstralloc(optstr,
-			  ";",
 			  compress_opt,
 			  record_opt,
 			  bsd_opt,
 			  krb4_opt,
 			  kencrypt_opt,
 			  index_opt,
-			  estr ? estr : "",
+			  exclude_file_opt,
+			  exclude_list_opt,
 			  NULL);
     return optstr;
 }
 
+
 int main(argc, argv)
 int argc;
 char **argv;
 {
     int interactive = 0;
     int level, mesgpipe[2];
-    char *prog, *disk, *dumpdate, *options;
-    char *host;				/* my hostname from the server */
+    char *prog, *disk, *amdevice, *dumpdate, *stroptions;
     char *line = NULL;
     char *err_extra = NULL;
-    char *s, *fp;
+    char *s;
+    int i;
     int ch;
     unsigned long malloc_hist_1, malloc_size_1;
     unsigned long malloc_hist_2, malloc_size_2;
@@ -228,138 +178,160 @@ char **argv;
     interactive = (argc > 1 && strcmp(argv[1],"-t") == 0);
     erroutput_type = (ERR_INTERACTIVE|ERR_SYSLOG);
     dbopen();
+    startclock();
     dbprintf(("%s: version %s\n", argv[0], version()));
 
+    our_features = am_init_feature_set();
+    our_feature_string = am_feature_to_string(our_features);
+
     if(interactive) {
 	/*
 	 * In interactive (debug) mode, the backup data is sent to
-        * /dev/null and none of the network connections back to driver
-        * programs on the tape host are set up.
-	 * Index service is run and goes to stdout.
+	 * /dev/null and none of the network connections back to driver
+	 * programs on the tape host are set up.  The index service is
+	 * run and goes to stdout.
 	 */
 	fprintf(stderr, "%s: running in interactive test mode\n", get_pname());
 	fflush(stderr);
     }
 
-    host = alloc(MAX_HOSTNAME_LENGTH+1);
-    gethostname(host, MAX_HOSTNAME_LENGTH);
-    host[MAX_HOSTNAME_LENGTH] = '\0';
+    prog = NULL;
+    disk = NULL;
+    amdevice = NULL;
+    dumpdate = NULL;
+    stroptions = NULL;
 
     /* parse dump request */
 
-    if(interactive) {
-	fprintf(stderr, "%s> ", get_pname());
-	fflush(stderr);
-    }
-
-    if((line = agets(stdin)) == NULL) {
-	err_extra = "no input";
-	goto err;
-    }
-
-#define sc "OPTIONS"
-    if(strncmp(line, sc, sizeof(sc)-1) == 0) {
+    for(; (line = agets(stdin)) != NULL; free(line)) {
+	if(interactive) {
+	    fprintf(stderr, "%s> ", get_pname());
+	    fflush(stderr);
+	}
+#define sc "OPTIONS "
+	if(strncmp(line, sc, sizeof(sc)-1) == 0) {
 #undef sc
-#define sc "hostname="
-	s = strstr(line, sc);
-	if(s != NULL) {
-	    s += sizeof(sc)-1;
-	    ch = *s++;
-#undef sc
-	    fp = s-1;
-	    while(ch != '\0' && ch != ';') ch = *s++;
-	    s[-1] = '\0';
-	    host = newstralloc(host, fp);
+	    g_options = parse_g_options(line+8, 1);
+	    if(!g_options->hostname) {
+		g_options->hostname = alloc(MAX_HOSTNAME_LENGTH+1);
+		gethostname(g_options->hostname, MAX_HOSTNAME_LENGTH);
+		g_options->hostname[MAX_HOSTNAME_LENGTH] = '\0';
+	    }
+	    continue;
 	}
 
-	amfree(line);
-	if((line = agets(stdin)) == NULL) {
-	    err_extra = "no request after OPTIONS";
+	if (prog != NULL) {
+	    err_extra = "multiple requests";
 	    goto err;
 	}
-    }
 
-    dbprintf(("%s: got input request: %s\n", get_pname(), line));
+	s = line;
+	ch = *s++;
 
-    s = line;
-    ch = *s++;
+	skip_whitespace(s, ch);			/* find the program name */
+	if(ch == '\0') {
+	    err_extra = "no program name";
+	    goto err;				/* no program name */
+	}
+	prog = s - 1;
+	skip_non_whitespace(s, ch);
+	s[-1] = '\0';
+	prog = stralloc(prog);
 
-    skip_whitespace(s, ch);			/* find the program name */
-    if(ch == '\0') {
-	err_extra = "no program name";
-	goto err;
-    }
-    prog = s - 1;
-    skip_non_whitespace(s, ch);
-    s[-1] = '\0';
+	skip_whitespace(s, ch);			/* find the disk name */
+	if(ch == '\0') {
+	    err_extra = "no disk name";
+	    goto err;				/* no disk name */
+	}
+	amfree(disk);
+	disk = s - 1;
+	skip_non_whitespace(s, ch);
+	s[-1] = '\0';
+	disk = stralloc(disk);
 
-    skip_whitespace(s, ch);			/* find the disk name */
-    if(ch == '\0') {
-	err_extra = "no disk name";
-	goto err;
-    }
-    disk = s - 1;
-    skip_non_whitespace(s, ch);
-    s[-1] = '\0';
+	skip_whitespace(s, ch);			/* find the device or level */
+	if (ch == '\0') {
+	    err_extra = "bad level";
+	    goto err;
+	}
 
-    skip_whitespace(s, ch);			/* find the level number */
-    if(ch == '\0' || sscanf(s - 1, "%d", &level) != 1) {
-	err_extra = "bad level";
-	goto err;				/* bad level */
-    }
-    skip_integer(s, ch);
+	if(!isdigit((int)s[-1])) {
+	    amfree(amdevice);
+	    amdevice = s - 1;
+	    skip_non_whitespace(s, ch);
+	    s[-1] = '\0';
+	    amdevice = stralloc(amdevice);
+	    skip_whitespace(s, ch);		/* find level number */
+	}
+	else {
+	    amdevice = stralloc(disk);
+	}
 
-    skip_whitespace(s, ch);			/* find the dump date */
-    if(ch == '\0') {
-	err_extra = "no dump date";
-	goto err;				/* no dump date */
-    }
-    dumpdate = s - 1;
-    skip_non_whitespace(s, ch);
-    s[-1] = '\0';
+						/* find the level number */
+	if(ch == '\0' || sscanf(s - 1, "%d", &level) != 1) {
+	    err_extra = "bad level";
+	    goto err;				/* bad level */
+	}
+	skip_integer(s, ch);
 
-    skip_whitespace(s, ch);			/* find the options keyword */
-    if(ch == '\0') {
-	err_extra = "no options";
-	goto err;				/* no options */
-    }
-#define sc "OPTIONS"
-    if(strncmp(s - 1, sc, sizeof(sc)-1) != 0) {
-	err_extra = "no OPTIONS keyword";
-	goto err;				/* no options */
-    }
-    s += sizeof(sc)-1;
-    ch = s[-1];
+	skip_whitespace(s, ch);			/* find the dump date */
+	if(ch == '\0') {
+	    err_extra = "no dumpdate";
+	    goto err;				/* no dumpdate */
+	}
+	amfree(dumpdate);
+	dumpdate = s - 1;
+	skip_non_whitespace(s, ch);
+	s[-1] = '\0';
+	dumpdate = stralloc(dumpdate);
+
+	skip_whitespace(s, ch);			/* find the options keyword */
+	if(ch == '\0') {
+	    err_extra = "no options";
+	    goto err;				/* no options */
+	}
+#define sc "OPTIONS "
+	if(strncmp(s - 1, sc, sizeof(sc)-1) != 0) {
+	    err_extra = "no OPTIONS keyword";
+	    goto err;				/* no options */
+	}
+	s += sizeof(sc)-1;
+	ch = s[-1];
 #undef sc
-    skip_whitespace(s, ch);			/* find the options string */
-    if(ch == '\0') {
-	err_extra = "bad options string";
-	goto err;				/* no options */
+	skip_whitespace(s, ch);			/* find the options string */
+	if(ch == '\0') {
+	    err_extra = "bad options string";
+	    goto err;				/* no options */
+	}
+	amfree(stroptions);
+	stroptions = stralloc(s - 1);
     }
-    options = s - 1;
+    amfree(line);
 
     dbprintf(("  parsed request as: program `%s'\n", prog));
     dbprintf(("                     disk `%s'\n", disk));
-    dbprintf(("                     lev %d\n", level));
+    dbprintf(("                     device `%s'\n", amdevice));
+    dbprintf(("                     level %d\n", level));
     dbprintf(("                     since %s\n", dumpdate));
-    dbprintf(("                     opt `%s'\n", options));
+    dbprintf(("                     options `%s'\n", stroptions));
 
-    {
-      int i;
-      for(i = 0; programs[i]; ++i)
-	if (strcmp(programs[i]->name, prog) == 0)
-	  break;
-      if (programs[i]) {
-	program = programs[i];
-      } else {
-	error("ERROR [%s: unknown program %s]", get_pname(), prog);
-      }
+    for(i = 0; programs[i] != NULL; i++) {
+	if (strcmp(programs[i]->name, prog) == 0) {
+	    break;
+	}
     }
+    if (programs[i] == NULL) {
+	error("ERROR [%s: unknown program %s]", get_pname(), prog);
+    }
+    program = programs[i];
 
-    parse_options(options, disk);
+    options = parse_options(stroptions, disk, amdevice, g_options->features, 0);
 
 #ifdef KRB4_SECURITY
-    if(krb4_auth) {
+    /* modification by BIS@BBN 4/25/2003:
+     * with the option processing changes in amanda 2.4.4, must change
+     * the conditional from krb4_auth to options->krb4_auth */
+    if(options->krb4_auth) {
 	if(read(KEY_PIPE, session_key, sizeof session_key) 
 	   != sizeof session_key) {
 	  error("ERROR [%s: could not read session key]", get_pname());
@@ -368,19 +340,19 @@ char **argv;
 #endif
 
     if(!interactive) {
-      data_socket = stream_server(&data_port, DATABUF_SIZE*2, DEFAULT_SIZE);
+      data_socket = stream_server(&data_port, STREAM_BUFSIZE, -1);
       if(data_socket < 0) {
 	error("ERROR [%s: could not create data socket: %s]",
 	      get_pname(), strerror(errno));
       }
-      mesg_socket = stream_server(&mesg_port, DEFAULT_SIZE, DEFAULT_SIZE);
+      mesg_socket = stream_server(&mesg_port, -1, -1);
       if(mesg_socket < 0) {
 	error("ERROR [%s: could not create mesg socket: %s]",
 	      get_pname(), strerror(errno));
       }
     }
-    if (!interactive && createindex) {
-      index_socket = stream_server(&index_port, DEFAULT_SIZE, DEFAULT_SIZE);
+    if (!interactive && options->createindex) {
+      index_socket = stream_server(&index_port, -1, -1);
       if(index_socket < 0) {
 	error("ERROR [%s: could not create index socket: %s]",
 	      get_pname(), strerror(errno));
@@ -391,15 +363,26 @@ char **argv;
 
     printf("CONNECT DATA %d MESG %d INDEX %d\n",
 	   data_port, mesg_port, index_port);
-    printf("OPTIONS %s\n", optionstr());
-    freopen("/dev/null","w",stdout);
+    printf("OPTIONS ");
+    if(am_has_feature(g_options->features, fe_rep_options_features)) {
+	printf("features=%s;", our_feature_string);
+    }
+    if(am_has_feature(g_options->features, fe_rep_options_hostname)) {
+	printf("hostname=%s;", g_options->hostname);
+    }
+    if(am_has_feature(g_options->features, fe_rep_options_sendbackup_options)) {
+	printf("%s", optionstr(options));
+    }
+    printf("\n");
+    fflush(stdout);
+    freopen("/dev/null", "w", stdout);
 
-    if (createindex)
-      dbprintf(("  waiting for connect on %d, then %d, then %d\n",
-		data_port, mesg_port, index_port));
+    if (options->createindex)
+      dbprintf(("%s: waiting for connect on %d, then %d, then %d\n",
+		debug_prefix_time(NULL), data_port, mesg_port, index_port));
     else
-      dbprintf(("  waiting for connect on %d, then %d\n",
-		data_port, mesg_port));
+      dbprintf(("%s: waiting for connect on %d, then %d\n",
+		debug_prefix_time(NULL), data_port, mesg_port));
 
     if(interactive) {
       if((dataf = open("/dev/null", O_RDWR)) < 0) {
@@ -408,50 +391,58 @@ char **argv;
       }
       mesgf = 2;
     } else {
-      dataf = stream_accept(data_socket, TIMEOUT, DEFAULT_SIZE, DEFAULT_SIZE);
+      dataf = stream_accept(data_socket, TIMEOUT, -1, -1);
       if(dataf == -1) {
-        dbprintf(("%s: timeout on data port %d\n", get_pname(), data_port));
+	dbprintf(("%s: timeout on data port %d\n",
+		  debug_prefix_time(NULL), data_port));
       }
-      mesgf = stream_accept(mesg_socket, TIMEOUT, DEFAULT_SIZE, DEFAULT_SIZE);
+      mesgf = stream_accept(mesg_socket, TIMEOUT, -1, -1);
       if(mesgf == -1) {
-        dbprintf(("%s: timeout on mesg port %d\n", get_pname(), mesg_port));
+        dbprintf(("%s: timeout on mesg port %d\n",
+		  debug_prefix_time(NULL), mesg_port));
       }
     }
     if(interactive) {
       indexf = 1;
-    } else if (createindex) {
-      indexf = stream_accept(index_socket,
-			     TIMEOUT, DEFAULT_SIZE, DEFAULT_SIZE);
+    } else if (options->createindex) {
+      indexf = stream_accept(index_socket, TIMEOUT, -1, -1);
       if (indexf == -1) {
-	dbprintf(("%s: timeout on index port %d\n", get_pname(), index_port));
+	dbprintf(("%s: timeout on index port %d\n",
+		  debug_prefix_time(NULL), index_port));
       }
     }
 
     if(!interactive) {
-      if(dataf == -1 || mesgf == -1 || (createindex && indexf == -1)) {
+      if(dataf == -1 || mesgf == -1 || (options->createindex && indexf == -1)) {
         dbclose();
         exit(1);
       }
     }
 
-    dbprintf(("  got all connections\n"));
+    dbprintf(("%s: got all connections\n", debug_prefix_time(NULL)));
 
 #ifdef KRB4_SECURITY
     if(!interactive) {
-      if (krb4_auth) {
+      /* modification by BIS@BBN 4/25/2003:
+       * with the option processing changes in amanda 2.4.4, must change
+       * the conditional from krb4_auth to options->krb4_auth */
+      if (options->krb4_auth) {
         if(kerberos_handshake(dataf, session_key) == 0) {
-	    dbprintf(("kerberos_handshake on data socket failed\n"));
+	    dbprintf(("%s: kerberos_handshake on data socket failed\n",
+		      debug_prefix_time(NULL)));
 	    dbclose();
 	    exit(1);
         }
 
         if(kerberos_handshake(mesgf, session_key) == 0) {
-	    dbprintf(("kerberos_handshake on mesg socket failed\n"));
+	    dbprintf(("%s: kerberos_handshake on mesg socket failed\n",
+		      debug_prefix_time(NULL)));
 	    dbclose();
 	    exit(1);
         }
 
-        dbprintf(("%s: kerberos handshakes succeeded!\n", get_pname()));
+        dbprintf(("%s: kerberos handshakes succeeded!\n",
+		  debug_prefix_time(NULL)));
       }
     }
 #endif
@@ -459,7 +450,8 @@ char **argv;
     if(!interactive) {
       /* redirect stderr */
       if(dup2(mesgf, 2) == -1) {
-	  dbprintf(("error redirecting stderr: %s\n", strerror(errno)));
+	  dbprintf(("%s: error redirecting stderr: %s\n",
+		    debug_prefix(NULL), strerror(errno)));
 	  dbclose();
 	  exit(1);
       }
@@ -469,10 +461,24 @@ char **argv;
       error("error [opening mesg pipe: %s]", strerror(errno));
     }
 
-    program->start_backup(host, disk, level, dumpdate, dataf, mesgpipe[1],
-			  indexf);
+    program->start_backup(g_options->hostname, disk, amdevice, level, dumpdate,
+			  dataf, mesgpipe[1], indexf);
     parse_backup_messages(mesgpipe[0]);
 
+    amfree(prog);
+    amfree(disk);
+    amfree(amdevice);
+    amfree(dumpdate);
+    amfree(stroptions);
+    amfree(our_feature_string);
+    am_release_feature_set(our_features);
+    our_features = NULL;
+    am_release_feature_set(g_options->features);
+    g_options->features = NULL;
+    amfree(g_options->hostname);
+    amfree(g_options->str);
+    amfree(g_options);
+
     dbclose();
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
@@ -485,11 +491,10 @@ char **argv;
 
  err:
     printf("FORMAT ERROR IN REQUEST PACKET\n");
-    if(err_extra) {
-	dbprintf(("REQ packet is bogus: %s\n", err_extra));
-    } else {
-	dbprintf(("REQ packet is bogus\n"));
-    }
+    dbprintf(("%s: REQ packet is bogus%s%s\n",
+	      debug_prefix_time(NULL),
+	      err_extra ? ": " : "",
+	      err_extra ? err_extra : ""));
     dbclose();
     return 1;
 }
@@ -609,7 +614,7 @@ void write_tapeheader()
     fprintf(stderr, "%s: info BACKUP=%s\n", get_pname(), program->backup_name);
 
     fprintf(stderr, "%s: info RECOVER_CMD=", get_pname());
-    if (compress)
+    if (options->compress == COMPR_FAST || options->compress == COMPR_BEST)
 	fprintf(stderr, "%s %s |", UNCOMPRESS_PATH,
 #ifdef UNCOMPRESS_OPT
 		UNCOMPRESS_OPT
@@ -620,7 +625,7 @@ void write_tapeheader()
 
     fprintf(stderr, "%s -f... -\n", program->restore_name);
 
-    if (compress)
+    if (options->compress == COMPR_FAST || options->compress == COMPR_BEST)
 	fprintf(stderr, "%s: info COMPRESS_SUFFIX=%s\n",
 			get_pname(), COMPRESS_SUFFIX);
 
@@ -635,7 +640,8 @@ int stdoutfd, stderrfd;
 {
     int pid, inpipe[2];
 
-    dbprintf(("%s: forking function %s in pipeline\n", get_pname(), fname));
+    dbprintf(("%s: forking function %s in pipeline\n",
+	      debug_prefix_time(NULL), fname));
 
     if(pipe(inpipe) == -1) {
 	error("error [open pipe to %s: %s]", fname, strerror(errno));
@@ -704,7 +710,6 @@ int mesgin;
 
 
 double first_num P((char *str));
-dmpline_t parse_dumpline P((char *str));
 
 double first_num(str)
 char *str;
@@ -726,42 +731,52 @@ char *str;
     return d;
 }
 
-dmpline_t parse_dumpline(str)
-char *str;
-/*
- * Checks the dump output line in str against the regex table.
- */
-{
-    regex_t *rp;
-
-    /* check for error match */
-    for(rp = program->re_table; rp->regex != NULL; rp++) {
-	if(match(rp->regex, str))
-	    break;
-    }
-    if(rp->typ == DMP_SIZE) 
-	dump_size = (long)((first_num(str) * rp->scale + 1023.0)/1024.0);
-    return rp->typ;	
-}
-
-
 static void process_dumpline(str)
 char *str;
 {
+    regex_t *rp;
+    char *type;
     char startchr;
-    dmpline_t typ;
 
-    typ = parse_dumpline(str);
-    switch(typ) {
+    for(rp = program->re_table; rp->regex != NULL; rp++) {
+	if(match(rp->regex, str)) {
+	    break;
+	}
+    }
+    if(rp->typ == DMP_SIZE) {
+	dump_size = (long)((first_num(str) * rp->scale + 1023.0)/1024.0);
+    }
+    switch(rp->typ) {
     case DMP_NORMAL:
-    case DMP_SIZE:
+	type = "normal";
 	startchr = '|';
 	break;
-    default:
     case DMP_STRANGE:
+	type = "strange";
 	startchr = '?';
 	break;
+    case DMP_SIZE:
+	type = "size";
+	startchr = '|';
+	break;
+    case DMP_ERROR:
+	type = "error";
+	startchr = '?';
+	break;
+    default:
+	/*
+	 * Should never get here.
+	 */
+	type = "unknown";
+	startchr = '!';
+	break;
     }
+    dbprintf(("%s: %3d: %7s(%c): %s\n",
+	      debug_prefix_time(NULL),
+	      rp->srcline,
+	      type,
+	      startchr,
+	      str));
     fprintf(stderr, "%c %s\n", startchr, str);
 }
 
@@ -802,11 +817,13 @@ int *fd, min;
   while (*fd >= 0 && *fd < min) {
     int newfd = dup(*fd);
     if (newfd == -1)
-      dbprintf(("unable to save file descriptor [%s]\n", strerror(errno)));
+      dbprintf(("%s: unable to save file descriptor [%s]\n",
+		debug_prefix(NULL), strerror(errno)));
     *fd = newfd;
   }
   if (origfd != *fd)
-    dbprintf(("dupped file descriptor %i to %i\n", origfd, *fd));
+    dbprintf(("%s: dupped file descriptor %i to %i\n",
+	      debug_prefix(NULL), origfd, *fd));
 }
 
 void start_index(createindex, input, mesg, index, cmd)
@@ -851,9 +868,7 @@ char *cmd;
   dup2(mesg, 2);
   dup2(input, 3);
   for(index = 4; index < FD_SETSIZE; index++) {
-    extern int db_fd;
-
-    if (index != db_fd) {
+    if (index != dbfd()) {
       close(index);
     }
   }
@@ -872,7 +887,8 @@ char *cmd;
     error("couldn't start index creator [%s]", strerror(errno));
   }
 
-  dbprintf(("%s: started index creator: \"%s\"\n", get_pname(), cmd));
+  dbprintf(("%s: started index creator: \"%s\"\n",
+	    debug_prefix_time(NULL), cmd));
   while(1) {
     char buffer[BUFSIZ], *ptr;
     int bytes_read;
@@ -899,8 +915,8 @@ char *cmd;
 	  /* the signal handler may have assigned to index_finished
 	   * just as we waited for write() to complete. */
 	  if (!index_finished) {
-	      dbprintf(("index tee cannot write to index creator [%s]\n",
-			strerror(errno)));
+	      dbprintf(("%s: index tee cannot write to index creator [%s]\n",
+			debug_prefix_time(NULL), strerror(errno)));
 	      index_finished = 1;
 	}
       } else {
@@ -931,9 +947,10 @@ char *cmd;
   /* finished */
   /* check the exit code of the pipe and moan if not 0 */
   if ((exitcode = pclose(pipe_fp)) != 0) {
-    dbprintf(("%s: index pipe returned %d\n", get_pname(), exitcode));
+    dbprintf(("%s: index pipe returned %d\n",
+	      debug_prefix_time(NULL), exitcode));
   } else {
-    dbprintf(("%s: index created successfully\n", get_pname()));
+    dbprintf(("%s: index created successfully\n", debug_prefix_time(NULL)));
   }
   pipe_fp = NULL;
 
