@@ -24,12 +24,13 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /* 
- * $Id: sendbackup.h,v 1.11.4.1 2000/05/27 21:20:32 vectro Exp $
+ * $Id: sendbackup.h,v 1.11.4.1.6.3 2002/03/31 21:01:32 jrjackson Exp $
  *
  * a few common decls for the sendbackup-* sources
  */
 #include "amanda.h"
 #include "pipespawn.h"
+#include "client_util.h"
 
 void write_tapeheader P((void));
 
@@ -57,26 +58,28 @@ typedef enum {
 } dmpline_t;
 
 typedef struct regex_s {
-    dmpline_t typ;
     char *regex;
+    int srcline;
     int scale;                  /* only used for size lines */
+    dmpline_t typ;
 } regex_t;
 
-extern int compress, comppid, dumppid, tarpid, no_record;
-extern int indexpid, createindex;
+#define AM_NORMAL_RE(re)	{(re), __LINE__, 0, DMP_NORMAL}
+#define AM_STRANGE_RE(re)	{(re), __LINE__, 0, DMP_STRANGE}
+#define AM_SIZE_RE(re,s)	{(re), __LINE__, (s), DMP_SIZE}
+#define AM_ERROR_RE(re)		{(re), __LINE__, 0, DMP_ERROR}
+
+extern int  comppid, dumppid, tarpid;
+extern int indexpid;
+extern option_t *options;
 
 typedef struct backup_program_s {
     char *name, *backup_name, *restore_name;
     regex_t *re_table;
-    void (*start_backup) P((char *host, char *disk, int level, char *dumpdate, 
+    void (*start_backup) P((char *host, char *disk, char *amdevice, int level, char *dumpdate, 
 			    int dataf, int mesgf, int indexf));
     void (*end_backup) P((int goterror));
 } backup_program_t;
 
 extern backup_program_t *programs[], *program;
 
-#define COMPR_FAST 1
-#define COMPR_BEST 2
-
-extern char *efile;
-extern char *estr;
