@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amanda.h,v 1.66.2.7 2000/09/23 15:33:58 martinea Exp $
+ * $Id: amanda.h,v 1.66.2.7.4.5.2.11 2003/12/16 22:36:45 martinea Exp $
  *
  * the central header file included by all amanda sources
  */
@@ -128,6 +128,10 @@
 #  include <string.h>
 #endif
 
+#ifdef HAVE_STRINGS_H
+#  include <strings.h>
+#endif
+
 #ifdef HAVE_SYSLOG_H
 #  include <syslog.h>
 #endif
@@ -350,11 +354,13 @@ extern int errno;
  * print debug output, else compile to nothing.
  */
 
-#ifdef DEBUG_CODE
-#   define dbopen()    debug_open()
-#   define dbclose()    debug_close()
-#   define dbprintf(p)  (debug? (debug_printf p, 0) : 0)
-#   define dbfd()    debug_fd()
+#ifdef DEBUG_CODE							/* { */
+#   define dbopen()	debug_open()
+#   define dbclose()	debug_close()
+#   define dbprintf(p)	(debug? (debug_printf p, 0) : 0)
+#   define dbfd()	debug_fd()
+#   define dbfp()	debug_fp()
+#   define dbfn()	debug_fn()
 
 extern void debug_open P((void));
 extern void debug_close P((void));
@@ -362,12 +368,21 @@ extern void debug_printf P((char *format, ...))
     __attribute__ ((format (printf, 1, 2)));
 extern int  debug_fd P((void));
 extern FILE *  debug_fp P((void));
-#else
+extern char *  debug_fn P((void));
+extern void set_debug_prefix_pid P((pid_t));
+extern char *debug_prefix P((char *));
+extern char *debug_prefix_time P((char *));
+#else									/* }{ */
 #   define dbopen()
 #   define dbclose()
 #   define dbprintf(p)
-#   define dbfd()    (-1)
-#endif
+#   define dbfd()	(-1)
+#   define dbfp()	NULL
+#   define dbfn()	NULL
+#   define set_debug_prefix_pid(x)
+#   define debug_prefix(x) get_pname()
+#   define debug_prefix_time(x) get_pname()
+#endif									/* } */
 
 /* amanda #days calculation, with roundoff */
 
@@ -391,9 +406,17 @@ extern FILE *  debug_fp P((void));
 #define KAMANDA_SERVICE_DEFAULT	10081
 #endif
 
-/* Size of a tape block in kbytes.  Do not change lightly.  */
-#define TAPE_BLOCK_SIZE 32
-#define TAPE_BLOCK_BYTES (TAPE_BLOCK_SIZE*1024)
+#define am_round(v,u)	((((v) + (u) - 1) / (u)) * (u))
+#define am_floor(v,u)	(((v) / (u)) * (u))
+
+/* Holding disk block size.  Do not even think about changing this!  :-) */
+#define DISK_BLOCK_KB		32
+#define DISK_BLOCK_BYTES	(DISK_BLOCK_KB * 1024)
+
+/* Maximum size of a tape block */
+/* MAX_TAPE_BLOCK_KB is defined in config.h */
+/* by configure --with-maxtapeblocksize     */
+#define MAX_TAPE_BLOCK_BYTES (MAX_TAPE_BLOCK_KB*1024)
 
 /* Define miscellaneous amanda functions.  */
 #define ERR_INTERACTIVE	1
@@ -416,14 +439,16 @@ extern char  *get_pname P((void));
 extern int    erroutput_type;
 extern void   error     P((char *format, ...))
     __attribute__ ((format (printf, 1, 2)));
+extern void   errordump P((char *format, ...))
+    __attribute__ ((format (printf, 1, 2)));
 extern int    onerror         P((void (*errf)(void)));
 
-#if defined(USE_DBMALLOC)
-extern void  *debug_alloc           P((char *c, int l, int size));
-extern void  *debug_newalloc        P((char *c, int l, void *old, int size));
+extern void  *debug_alloc           P((char *c, int l, size_t size));
+extern void  *debug_newalloc        P((char *c, int l, void *old, size_t size));
 extern char  *debug_stralloc        P((char *c, int l, const char *str));
 extern char  *debug_newstralloc     P((char *c, int l, char *oldstr, const char *newstr));
-extern char  *dbmalloc_caller_loc   P((char *file, int line));
+extern char  *debug_caller_loc      P((char *file, int line));
+
 extern int   debug_alloc_push	    P((char *file, int line));
 extern void  debug_alloc_pop	    P((void));
 
@@ -468,22 +493,32 @@ extern void  debug_alloc_pop	    P((void));
 extern char  *debug_vstralloc       P((const char *str, ...));
 extern char  *debug_newvstralloc    P((char *oldstr, const char *newstr, ...));
 
-#else
-
-extern void  *alloc           P((int size));
-extern void  *newalloc        P((void *old, int size));
-extern char  *stralloc        P((const char *str));
-extern char  *newstralloc     P((char *oldstr, const char *newstr));
-extern char  *vstralloc       P((const char *str, ...));
-extern char  *newvstralloc    P((char *oldstr, const char *newstr, ...));
-
-#define debug_alloc_push(s,l)
-#define debug_alloc_pop()
-#endif
-
 #define	stralloc2(s1,s2)      vstralloc((s1),(s2),NULL)
 #define	newstralloc2(p,s1,s2) newvstralloc((p),(s1),(s2),NULL)
 
+extern char  *debug_agets     P((char *c, int l, FILE *file));
+extern char  *debug_areads    P((char *c, int l, int fd));
+#define agets(f)	      debug_agets(__FILE__,__LINE__,(f))
+#define areads(f)	      debug_areads(__FILE__,__LINE__,(f))
+
+extern int debug_amtable_alloc P((char *file,
+				  int line,
+				  void **table,
+				  int *current,
+				  size_t elsize,
+				  int count,
+				  int bump,
+				  void (*init_func)(void *)));
+#define amtable_alloc(t,c,s,n,b,f) debug_amtable_alloc(__FILE__,	\
+						       __LINE__,	\
+						       (t),		\
+						       (c),		\
+						       (s),		\
+						       (n),		\
+						       (b),		\
+						       (f))
+extern void amtable_free      P((void **table, int *current));
+
 extern void  *sbuf_man        P((void *bufs, void *ptr));
 extern uid_t  client_uid;
 extern gid_t  client_gid;
@@ -496,16 +531,10 @@ extern char  *clean_regex     P((char *regex));
 extern int    match           P((char *regex, char *str));
 extern int    match_glob      P((char *glob, char *str));
 extern char  *glob_to_regex   P((char *glob));
+extern int    match_host      P((char *glob, char *host));
+extern int    match_disk      P((char *glob, char *disk));
+extern int    match_datestamp P((char *dateexp, char *datestamp));
 extern time_t unctime         P((char *timestr));
-#if defined(USE_DBMALLOC)
-extern char  *dbmalloc_agets  P((char *c, int l, FILE *file));
-extern char  *dbmalloc_areads P((char *c, int l, int fd));
-#define agets(f)	      dbmalloc_agets(__FILE__,__LINE__,(f))
-#define areads(f)	      dbmalloc_areads(__FILE__,__LINE__,(f))
-#else
-extern char  *agets	      P((FILE *file));
-extern char  *areads	      P((int fd));
-#endif
 extern ssize_t  areads_dataready  P((int fd));
 extern void     areads_relbuf     P((int fd));
 
@@ -518,17 +547,17 @@ extern void     areads_relbuf     P((int fd));
 
 #define	amfree(ptr) do {						\
     if(ptr) {								\
-	int e = errno;							\
+	int e__errno = errno;						\
 	free(ptr);							\
 	(ptr) = NULL;							\
-	errno = e;							\
+	errno = e__errno;						\
     }									\
 } while(0)
 
 #define strappend(s1,s2) do {						\
-    char *t = (s1) ? stralloc2((s1),(s2)) : stralloc((s2));		\
+    char *t_t_t = (s1) ? stralloc2((s1),(s2)) : stralloc((s2));		\
     amfree((s1));							\
-    (s1) = t;								\
+    (s1) = t_t_t;							\
 } while(0)
 
 /*
@@ -574,6 +603,23 @@ extern void     areads_relbuf     P((int fd));
     (p) = NULL;								\
 } while(0)
 
+/*
+ * Return the number of elements in an array.
+ */
+#define am_countof(a)        (sizeof(a) / sizeof((a)[0]))
+
+/*
+ * min/max.  Don't do something like
+ *
+ *    x = min(y++, z);
+ *
+ * because the increment will be duplicated.
+ */
+#undef min
+#undef max
+#define min(a, b)       ((a) < (b) ? (a) : (b))
+#define max(a, b)       ((a) > (b) ? (a) : (b))
+
 /*
  * Utility string macros.  All assume a variable holds the current
  * character and the string pointer points to the next character to
@@ -856,7 +902,19 @@ extern int gettimeofday P((struct timeval *tp));
 # define initgroups(name,basegid) 0
 #else
 # ifndef HAVE_INITGROUPS_DECL
+/* modification by BIS@BBN 5/20/2003:
+ * In some Unix systems, basegid is defined as a gid_t; in others
+ * it is defined as an int.  On Mac OS X, there is a "pre-compiled"
+ * unistd.h which causes the gcc -E command in the ICE_CHECK_DECL
+ * autoconf macro to not succeed.  Thus, on Mac OS X, configure thinks
+ * we don't have this declaration when we actually do.  Since Mac OS X
+ * defines basegid as an int, this declaration causes a compilation
+ * failure.  The path of least resistance for fixing this problem
+ * is to just change the basegid declaration from gid_t to int, since
+ * other (but not all) UNIX flavors also defined basegid as an int.
 extern int initgroups P((const char *name, gid_t basegid));
+ */
+extern int initgroups P((const char *name, int basegid));
 # endif
 #endif
 
@@ -1125,4 +1183,13 @@ extern pid_t waitpid P((pid_t pid, amwait_t *stat_loc, int options));
 #define STDERR_FILENO 2
 #endif
 
+/* S_ISDIR is not defined on Nextstep */
+#ifndef S_ISDIR
+#if defined(_S_IFMT) && defined(_S_IFDIR)
+#define S_ISDIR(mode)   (((mode) & (_S_IFMT)) == (_S_IFDIR))
+#else
+error: Don t know how to define S_ISDIR
+#endif
+#endif
+
 #endif	/* !AMANDA_H */
