@@ -25,36 +25,47 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: debug.c,v 1.17.4.4 2001/03/20 00:21:06 jrjackson Exp $
+ * $Id: debug.c,v 1.17.4.3.4.3.2.9 2003/01/04 17:46:09 martinea Exp $
  *
  * debug log subroutines
  */
 
 #include "amanda.h"
+#include "clock.h"
 #include "util.h"
 #include "arglist.h"
 
-int debug = 1;
-
-#define	MIN_DB_FD			10
-
-#ifdef DEBUG_CODE
-int db_fd = 2;				/* default is stderr */
-#else
-int db_fd = -1;				/* default is to throw away */
-#endif
-static FILE *db_file = NULL;		/* stderr may not be a constant */
-
 #ifndef AMANDA_DBGDIR
 #  define AMANDA_DBGDIR		AMANDA_TMPDIR
 #endif
 
-arglist_function(void debug_printf, char *, format)
+#ifdef DEBUG_CODE
+
+int debug = 1;
+
+#define	MIN_DB_FD			10
+
+static int db_fd = 2;			/* default is stderr */
+static FILE *db_file = NULL;		/* stderr may not be a constant */
+static char *db_filename = NULL;
+
+static pid_t debug_prefix_pid = 0;
+
 /*
- * Formats and writes a debug message to the process's debug file.
+ * Format and write a debug message to the process debug file.
  */
+printf_arglist_function(void debug_printf, char *, format)
 {
     va_list argp;
+    int save_errno;
+
+    /*
+     * It is common in the code to call dbprintf to write out
+     * syserrno(errno) and then turn around and try to do something else
+     * with errno (e.g. printf() or log()), so we make sure errno goes
+     * back out with the same value it came in with.
+     */
+    save_errno = errno;
 
     if(db_file == NULL && db_fd == 2) {
 	db_file = stderr;
@@ -65,6 +76,8 @@ arglist_function(void debug_printf, char *, format)
 	fflush(db_file);
 	arglist_end(argp);
     }
+
+    errno = save_errno;
 }
 
 /*
@@ -105,12 +118,13 @@ void debug_open()
     DIR *d;
     struct dirent *entry;
     char *pname;
-    int pname_len;
+    size_t pname_len;
     int do_rename;
     char *test_name = NULL;
-    int test_name_len;
+    size_t test_name_len;
     int fd = -1;
     int i;
+    size_t d_name_len;
     int fd_close[MIN_DB_FD+1];
     struct passwd *pwent;
     struct stat sbuf;
@@ -121,6 +135,7 @@ void debug_open()
     if(client_uid == (uid_t) -1 && (pwent = getpwnam(CLIENT_LOGIN)) != NULL) {
 	client_uid = pwent->pw_uid;
 	client_gid = pwent->pw_gid;
+	endpwent();
     }
 
     /*
@@ -149,12 +164,15 @@ void debug_open()
 	if(is_dot_or_dotdot(entry->d_name)) {
 	    continue;
 	}
+	d_name_len = strlen(entry->d_name);
 	if(strncmp(entry->d_name, pname, pname_len) != 0
-	   || entry->d_name[pname_len] != '.') {
-	    continue;				/* not one of our files */
+	   || entry->d_name[pname_len] != '.'
+	   || d_name_len < 6
+	   || strcmp(entry->d_name + d_name_len - 6, ".debug") != 0) {
+	    continue;				/* not one of our debug files */
 	}
 	e = newvstralloc(e, dbgdir, entry->d_name, NULL);
-	if(strlen(entry->d_name) < test_name_len) {
+	if(d_name_len < test_name_len) {
 	    /*
 	     * Create a "pretend" name based on the last modification
 	     * time.  This name will be used to decide if the real name
@@ -179,7 +197,7 @@ void debug_open()
 	    i = 0;
 	    while(dbfilename != NULL
 		  && (s = newvstralloc(s, dbgdir, dbfilename, NULL)) != NULL
-		  && rename(e, s) != 0) {
+		  && rename(e, s) != 0 && errno != ENOENT) {
 		amfree(dbfilename);
 		dbfilename = get_debug_name((time_t)sbuf.st_mtime, ++i);
 	    }
@@ -189,6 +207,7 @@ void debug_open()
 	}
     }
     amfree(dbfilename);
+    amfree(e);
     amfree(s);
     amfree(test_name);
     closedir(d);
@@ -204,10 +223,12 @@ void debug_open()
     if(dbfilename == NULL) {
 	error("cannot create %s debug file", get_pname());
     }
-    (void) chown(s, client_uid, client_gid);
-    amfree(dbgdir);
     amfree(dbfilename);
-    amfree(s);
+    amfree(db_filename);
+    db_filename = s;
+    s = NULL;
+    (void) chown(db_filename, client_uid, client_gid);
+    amfree(dbgdir);
 
     /*
      * Move the file descriptor up high so it stays out of the way
@@ -227,7 +248,7 @@ void debug_open()
      * Make the first debug log file entry.
      */
     saved_debug = debug; debug = 1;
-    debug_printf("%s: debug %d pid %ld ruid %ld euid %ld start time %s",
+    debug_printf("%s: debug %d pid %ld ruid %ld euid %ld: start at %s",
 		 pname, saved_debug, (long)getpid(),
 		 (long)getuid(), (long)geteuid(),
 		 ctime(&curtime));
@@ -237,16 +258,30 @@ void debug_open()
 void debug_close()
 {
     time_t curtime;
+    int save_debug;
+    pid_t save_pid;
 
     time(&curtime);
+    save_debug = debug;
     debug = 1;
-    debug_printf("%s: pid %ld finish time %s", get_pname(), (long)getpid(),
+    save_pid = debug_prefix_pid;
+    debug_prefix_pid = 0;
+    debug_printf("%s: pid %ld finish time %s",
+		 debug_prefix_time(NULL),
+		 (long)getpid(),
 		 ctime(&curtime));
+    debug_prefix_pid = save_pid;
+    debug = save_debug;
 
-    if(fclose(db_file) == EOF)
-	error("close debug file: %s", strerror(errno));
+    if(db_file && fclose(db_file) == EOF) {
+	int save_errno = errno;
+
+	db_file = NULL;				/* prevent recursion */
+	error("close debug file: %s", strerror(save_errno));
+    }
     db_fd = -1;
     db_file = NULL;
+    amfree(db_filename);
 }
 
 int debug_fd()
@@ -258,3 +293,57 @@ FILE *debug_fp()
 {
     return db_file;
 }
+
+char *debug_fn()
+{
+    return db_filename;
+}
+
+/*
+ * Routines for returning a common debug file line prefix.  Always starts
+ * with the current program name, possibly with an optional suffix.
+ * May then be followed by a PID.  May then be followed by an elapsed
+ * time indicator.
+ */ 
+
+void set_debug_prefix_pid(p)
+    pid_t p;
+{
+    debug_prefix_pid = p;
+}
+
+char *debug_prefix(suffix)
+    char *suffix;
+{
+    static char *s = NULL;
+    char debug_pid[NUM_STR_SIZE];
+
+    s = newvstralloc(s, get_pname(), suffix, NULL);
+    if (debug_prefix_pid != (pid_t) 0) {
+	ap_snprintf(debug_pid, sizeof(debug_pid),
+		    "%ld",
+		    (long) debug_prefix_pid);
+	s = newvstralloc(s, s, "[", debug_pid, "]", NULL);
+    }
+    return s;
+}
+
+char *debug_prefix_time(suffix)
+    char *suffix;
+{
+    static char *s = NULL;
+    char *t1;
+    char *t2;
+
+    if (clock_is_running()) {
+	t1 = ": time ";
+	t2 = walltime_str(curclock());
+    } else {
+	t1 = t2 = NULL;
+    }
+
+    s = newvstralloc(s, debug_prefix(suffix), t1, t2, NULL);
+
+    return s;
+}
+#endif
