@@ -23,7 +23,7 @@
  * Author: AMANDA core development group.
  */
 /*
- * $Id: file.c,v 1.14.4.7 2001/03/20 00:21:06 jrjackson Exp $
+ * $Id: file.c,v 1.14.4.6.4.2.2.5 2003/01/01 23:28:52 martinea Exp $
  *
  * file and directory bashing routines
  */
@@ -205,6 +205,7 @@ safe_cd()
     if(client_uid == (uid_t) -1 && (pwent = getpwnam(CLIENT_LOGIN)) != NULL) {
 	client_uid = pwent->pw_uid;
 	client_gid = pwent->pw_gid;
+	endpwent();
     }
 
     (void) umask(0077);
@@ -212,11 +213,11 @@ safe_cd()
     if (client_uid != (uid_t) -1) {
 #if defined(AMANDA_DBGDIR)
 	d = stralloc2(AMANDA_DBGDIR, "/.");
-	(void) mkpdir(AMANDA_DBGDIR, 02700, client_uid, client_gid);
+	(void) mkpdir(d, 02700, client_uid, client_gid);
 	amfree(d);
 #endif
 	d = stralloc2(AMANDA_TMPDIR, "/.");
-	(void) mkpdir(AMANDA_TMPDIR, 02700, client_uid, client_gid);
+	(void) mkpdir(d, 02700, client_uid, client_gid);
 	amfree(d);
     }
 
@@ -358,18 +359,14 @@ char *inp;
  */
 
 char *
-#if defined(USE_DBMALLOC)
-dbmalloc_agets(s, l, file)
+debug_agets(s, l, file)
     char *s;
     int l;
     FILE *file;
-#else
-agets(file)
-    FILE *file;
-#endif
 {
     char *line = NULL, *line_ptr;
-    int line_size, line_free, size_save, line_len;
+    size_t line_size, size_save;
+    int line_free, line_len;
     char *cp;
     char *f;
 
@@ -378,7 +375,7 @@ agets(file)
 #define	AGETS_LINE_INCR	128
 
     line_size = AGETS_LINE_INCR;
-    line = alloc (line_size);
+    line = debug_alloc (s, l, line_size);
     line_free = line_size;
     line_ptr = line;
     line_len = 0;
@@ -395,13 +392,17 @@ agets(file)
 	}
 	line_len += line_free - 1;		/* bytes read minus '\0' */
 	size_save = line_size;
-	line_size += AGETS_LINE_INCR;		/* get more space */
-	cp = alloc (line_size);
+	if (line_size < 256 * AGETS_LINE_INCR) {
+	    line_size *= 2;
+	} else {
+	    line_size += 256 * AGETS_LINE_INCR;
+	}
+	cp = debug_alloc (s, l, line_size);	/* get more space */
 	memcpy (cp, line, size_save);		/* copy old to new */
 	free (line);				/* and release the old */
 	line = cp;
 	line_ptr = line + size_save - 1;	/* start at the null byte */
-	line_free = AGETS_LINE_INCR + 1;	/* and we get to use it */
+	line_free = line_size - line_len;	/* and we get to use it */
     }
     /*
      * Return what we got even if there was not a newline.  Only
@@ -423,9 +424,10 @@ agets(file)
  * Find/create a buffer for a particular file descriptor for use with
  * areads().
  *
- * void areads_getbuf (int fd)
+ * void areads_getbuf (char *file, int line, int fd)
  *
- * entry:	fd = file descriptor to look up
+ * entry:	file, line = caller source location
+ *		fd = file descriptor to look up
  * exit:	returns a pointer to the buffer, possibly new
  *=====================================================================
  */
@@ -439,7 +441,9 @@ static int areads_bufcount = 0;
 static ssize_t areads_bufsize = BUFSIZ;		/* for the test program */
 
 static void
-areads_getbuf(fd)
+areads_getbuf(s, l, fd)
+    char *s;
+    int l;
     int fd;
 {
     struct areads_buffer *new;
@@ -448,7 +452,7 @@ areads_getbuf(fd)
     assert(fd >= 0);
     if(fd >= areads_bufcount) {
 	size = (fd + 1) * sizeof(*areads_buffer);
-	new = (struct areads_buffer *) alloc(size);
+	new = (struct areads_buffer *) debug_alloc(s, l, size);
 	memset((char *)new, 0, size);
 	if(areads_buffer) {
 	    size = areads_bufcount * sizeof(*areads_buffer);
@@ -460,7 +464,8 @@ areads_getbuf(fd)
     }
     if(areads_buffer[fd].buffer == NULL) {
 	areads_buffer[fd].bufsize = areads_bufsize;
-	areads_buffer[fd].buffer = alloc(areads_buffer[fd].bufsize + 1);
+	areads_buffer[fd].buffer = debug_alloc(s, l,
+					       areads_buffer[fd].bufsize + 1);
 	areads_buffer[fd].buffer[0] = '\0';
 	areads_buffer[fd].endptr = areads_buffer[fd].buffer;
     }
@@ -484,7 +489,7 @@ areads_dataready(fd)
     ssize_t r = 0;
 
     if(fd >= 0 && fd < areads_bufcount && areads_buffer[fd].buffer != NULL) {
-	r = areads_buffer[fd].endptr - areads_buffer[fd].buffer;
+	r = (ssize_t) (areads_buffer[fd].endptr - areads_buffer[fd].buffer);
     }
     return r;
 }
@@ -527,15 +532,10 @@ areads_relbuf(fd)
  */
 
 char *
-#if defined(USE_DBMALLOC)
-dbmalloc_areads (s, l, fd)
+debug_areads (s, l, fd)
     char *s;
     int l;
     int fd;
-#else
-areads (fd)
-    int fd;
-#endif
 {
     char *nl;
     char *line;
@@ -552,7 +552,7 @@ areads (fd)
 	errno = EBADF;
 	return NULL;
     }
-    areads_getbuf(fd);
+    areads_getbuf(s, l, fd);
     buffer = areads_buffer[fd].buffer;
     endptr = areads_buffer[fd].endptr;
     buflen = areads_buffer[fd].bufsize - (endptr - buffer);
@@ -561,9 +561,15 @@ areads (fd)
 	 * No newline yet, so get more data.
 	 */
 	if (buflen == 0) {
-	    size = areads_buffer[fd].bufsize + areads_bufsize;
-	    newbuf = alloc(size + 1);
+	    if ((size = areads_buffer[fd].bufsize) < 256 * areads_bufsize) {
+		size *= 2;
+	    } else {
+		size += 256 * areads_bufsize;
+	    }
+	    newbuf = debug_alloc(s, l, size + 1);
 	    memcpy (newbuf, buffer, areads_buffer[fd].bufsize + 1);
+	    amfree(areads_buffer[fd].buffer);
+	    buffer = NULL;
 	    areads_buffer[fd].buffer = newbuf;
 	    areads_buffer[fd].endptr = newbuf + areads_buffer[fd].bufsize;
 	    areads_buffer[fd].bufsize = size;
