@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: protocol.h,v 1.8 1998/07/04 00:18:50 oliva Exp $
+ * $Id: protocol.h,v 1.8.10.3 2004/04/29 20:47:40 martinea Exp $
  *
  * interfaces for amanda protocol
  */
@@ -46,7 +46,7 @@ typedef enum { P_BOGUS, P_REQ, P_REP, P_ACK, P_NAK } pktype_t;
 typedef struct {			/* a predigested datagram */
     pktype_t type;
     struct sockaddr_in peer;
-    unsigned long cksum;
+    uint32_t cksum;
     int version_major, version_minor;
     int sequence;
     char *handle;
@@ -58,6 +58,7 @@ typedef struct {			/* a predigested datagram */
 
 typedef struct proto_s {
     pstate_t state;
+    pstate_t prevstate;
     struct sockaddr_in peer;
     time_t timeout;
     time_t repwait;
@@ -66,7 +67,7 @@ typedef struct proto_s {
     int origseq, curseq;
     int handleofs;
     char *security;
-    unsigned long auth_cksum;
+    uint32_t auth_cksum;
     char *req;					/* body of request msg */
     void (*continuation) P((struct proto_s *, pkt_t *));
     void *datap;
