@@ -25,13 +25,14 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: stream.c,v 1.10.2.7 2001/02/28 00:53:15 jrjackson Exp $
+ * $Id: stream.c,v 1.10.2.6.4.4.2.4 2002/11/12 19:19:03 martinea Exp $
  *
  * functions for managing stream sockets
  */
 #include "amanda.h"
 #include "dgram.h"
 #include "stream.h"
+#include "clock.h"
 #include "util.h"
 
 /* local functions */
@@ -41,20 +42,20 @@ int stream_server(portp, sendsize, recvsize)
 int *portp;
 int sendsize, recvsize;
 {
-    int server_socket, len;
+    int server_socket;
+    socklen_t len;
 #ifdef SO_KEEPALIVE
     int on = 1;
     int r;
 #endif
     struct sockaddr_in server;
-    struct in_addr in_addr;
     int save_errno;
 
     *portp = -1;				/* in case we error exit */
     if((server_socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
 	save_errno = errno;
 	dbprintf(("%s: stream_server: socket() failed: %s\n",
-		  get_pname(),
+		  debug_prefix(NULL),
 		  strerror(save_errno)));
 	errno = save_errno;
 	return -1;
@@ -64,7 +65,7 @@ int sendsize, recvsize;
 	errno = EMFILE;				/* out of range */
 	save_errno = errno;
 	dbprintf(("%s: stream_server: socket out of range: %d\n",
-		  get_pname(),
+		  debug_prefix(NULL),
 		  server_socket));
 	errno = save_errno;
 	return -1;
@@ -73,9 +74,9 @@ int sendsize, recvsize;
     server.sin_family = AF_INET;
     server.sin_addr.s_addr = INADDR_ANY;
 
-    if(sendsize != DEFAULT_SIZE) 
+    if(sendsize >= 0) 
         try_socksize(server_socket, SO_SNDBUF, sendsize);
-    if(recvsize != DEFAULT_SIZE) 
+    if(recvsize >= 0) 
         try_socksize(server_socket, SO_RCVBUF, recvsize);
 
     /*
@@ -87,8 +88,8 @@ int sendsize, recvsize;
      * to get the desired port, and to make sure we return a port that
      * is within the range it requires.
      */
-#ifdef PORTRANGE
-    if (bind_portrange(server_socket, &server, PORTRANGE) == 0)
+#ifdef TCPPORTRANGE
+    if (bind_portrange(server_socket, &server, TCPPORTRANGE) == 0)
 	goto out;
 #endif
 
@@ -96,10 +97,10 @@ int sendsize, recvsize;
 	goto out;
 
     server.sin_port = INADDR_ANY;
-    if (bind(server_socket, (struct sockaddr *)&server, sizeof(server)) == -1) {
+    if (bind(server_socket, (struct sockaddr *)&server, (socklen_t) sizeof(server)) == -1) {
 	save_errno = errno;
 	dbprintf(("%s: stream_server: bind(INADDR_ANY) failed: %s\n",
-		  get_pname(),
+		  debug_prefix(NULL),
 		  strerror(save_errno)));
 	aclose(server_socket);
 	errno = save_errno;
@@ -115,7 +116,7 @@ out:
     if(getsockname(server_socket, (struct sockaddr *)&server, &len) == -1) {
 	save_errno = errno;
 	dbprintf(("%s: stream_server: getsockname() failed: %s\n",
-		  get_pname(),
+		  debug_prefix(NULL),
 		  strerror(save_errno)));
 	aclose(server_socket);
 	errno = save_errno;
@@ -124,11 +125,11 @@ out:
 
 #ifdef SO_KEEPALIVE
     r = setsockopt(server_socket, SOL_SOCKET, SO_KEEPALIVE,
-	(void *)&on, sizeof(on));
+	(void *)&on, (socklen_t)sizeof(on));
     if(r == -1) {
 	save_errno = errno;
 	dbprintf(("%s: stream_server: setsockopt(SO_KEEPALIVE) failed: %s\n",
-		  get_pname(),
+		  debug_prefix(NULL),
 		  strerror(save_errno)));
         aclose(server_socket);
 	errno = save_errno;
@@ -137,17 +138,17 @@ out:
 #endif
 
     *portp = (int) ntohs(server.sin_port);
-    memcpy(&in_addr, &server.sin_addr, sizeof(server.sin_addr));
     dbprintf(("%s: stream_server: waiting for connection: %s.%d\n",
-	      get_pname(),
-	      inet_ntoa(in_addr),
+	      debug_prefix_time(NULL),
+	      inet_ntoa(server.sin_addr),
 	      *portp));
     return server_socket;
 }
 
-int stream_client(hostname, port, sendsize, recvsize, localport)
-char *hostname;
-int port, sendsize, recvsize, *localport;
+static int
+stream_client_internal(hostname, port, sendsize, recvsize, localport, priv)
+    char *hostname;
+    int port, sendsize, recvsize, *localport, priv;
 {
     int client_socket, len;
 #ifdef SO_KEEPALIVE
@@ -155,14 +156,17 @@ int port, sendsize, recvsize, *localport;
     int r;
 #endif
     struct sockaddr_in svaddr, claddr;
-    struct in_addr in_addr;
     struct hostent *hostp;
     int save_errno;
+    char *f;
+
+    f = priv ? "stream_client_privileged" : "stream_client";
 
     if((hostp = gethostbyname(hostname)) == NULL) {
 	save_errno = errno;
-	dbprintf(("%s: stream_client: gethostbyname(%s) failed\n",
-		  get_pname(),
+	dbprintf(("%s: %s: gethostbyname(%s) failed\n",
+		  debug_prefix(NULL),
+		  f,
 		  hostname));
 	errno = save_errno;
 	return -1;
@@ -175,8 +179,9 @@ int port, sendsize, recvsize, *localport;
 
     if((client_socket = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
 	save_errno = errno;
-	dbprintf(("%s: stream_client: socket() failed: %s\n",
-		  get_pname(),
+	dbprintf(("%s: %s: socket() failed: %s\n",
+		  debug_prefix(NULL),
+		  f,
 		  strerror(save_errno)));
 	errno = save_errno;
 	return -1;
@@ -192,8 +197,9 @@ int port, sendsize, recvsize, *localport;
 		   (void *)&on, sizeof(on));
     if(r == -1) {
 	save_errno = errno;
-	dbprintf(("%s: stream_client: setsockopt() failed: %s\n",
-		  get_pname(),
+	dbprintf(("%s: %s: setsockopt() failed: %s\n",
+		  debug_prefix(NULL),
+		  f,
 		  strerror(save_errno)));
         aclose(client_socket);
 	errno = save_errno;
@@ -206,27 +212,43 @@ int port, sendsize, recvsize, *localport;
     claddr.sin_addr.s_addr = INADDR_ANY;
 
     /*
-     * If a port range was specified, we try to get a port in that
-     * range first.  Next, we try to get a reserved port.  If that
-     * fails, we just go for any port.
+     * If a privileged port range was requested, we try to get a port in
+     * that range first and fail if it is not available.  Next, we try
+     * to get a port in the range built in when Amanda was configured.
+     * If that fails, we just go for any port.
      *
      * It is up to the caller to make sure we have the proper permissions
      * to get the desired port, and to make sure we return a port that
      * is within the range it requires.
      */
-#ifdef PORTRANGE
-    if (bind_portrange(client_socket, &claddr, PORTRANGE) == 0)
+    if (priv) {
+	int b;
+
+	b = bind_portrange(client_socket, &claddr, 512, IPPORT_RESERVED - 1);
+	if (b == 0) {
+	    goto out;				/* got what we wanted */
+	}
+	save_errno = errno;
+	dbprintf(("%s: %s: bind(IPPORT_RESERVED) failed: %s\n",
+		  debug_prefix(NULL),
+		  f,
+		  strerror(save_errno)));
+	aclose(client_socket);
+	errno = save_errno;
+	return -1;
+    }
+
+#ifdef TCPPORTRANGE
+    if (bind_portrange(client_socket, &claddr, TCPPORTRANGE) == 0)
 	goto out;
 #endif
 
-    if (bind_portrange(client_socket, &claddr, 512, IPPORT_RESERVED - 1) == 0)
-	goto out;
-
     claddr.sin_port = INADDR_ANY;
     if (bind(client_socket, (struct sockaddr *)&claddr, sizeof(claddr)) == -1) {
 	save_errno = errno;
-	dbprintf(("%s: stream_client: bind(INADDR_ANY) failed: %s\n",
-		  get_pname(),
+	dbprintf(("%s: %s: bind(INADDR_ANY) failed: %s\n",
+		  debug_prefix(NULL),
+		  f,
 		  strerror(save_errno)));
 	aclose(client_socket);
 	errno = save_errno;
@@ -240,8 +262,9 @@ out:
     len = sizeof(claddr);
     if(getsockname(client_socket, (struct sockaddr *)&claddr, &len) == -1) {
 	save_errno = errno;
-	dbprintf(("%s: stream_client: getsockname() failed: %s\n",
-		  get_pname(),
+	dbprintf(("%s: %s: getsockname() failed: %s\n",
+		  debug_prefix(NULL),
+		  f,
 		  strerror(save_errno)));
 	aclose(client_socket);
 	errno = save_errno;
@@ -251,29 +274,31 @@ out:
     if(connect(client_socket, (struct sockaddr *)&svaddr, sizeof(svaddr))
        == -1) {
 	save_errno = errno;
-	dbprintf(("%s: stream_client: connect(%d) failed: %s\n",
-		  get_pname(),
-		  port,
+	dbprintf(("%s: %s: connect to %s.%d failed: %s\n",
+		  debug_prefix_time(NULL),
+		  f,
+		  inet_ntoa(svaddr.sin_addr),
+		  ntohs(svaddr.sin_port),
 		  strerror(save_errno)));
 	aclose(client_socket);
 	errno = save_errno;
 	return -1;
     }
 
-    memcpy(&in_addr, &svaddr.sin_addr, sizeof(svaddr.sin_addr));
-    dbprintf(("%s: stream_client: connected to %s.%d\n",
-	      get_pname(),
-	      inet_ntoa(in_addr),
+    dbprintf(("%s: %s: connected to %s.%d\n",
+	      debug_prefix_time(NULL),
+	      f,
+	      inet_ntoa(svaddr.sin_addr),
 	      ntohs(svaddr.sin_port)));
-    memcpy(&in_addr, &claddr.sin_addr, sizeof(claddr.sin_addr));
-    dbprintf(("%s: stream_client: our side is %s.%d\n",
-	      get_pname(),
-	      inet_ntoa(in_addr),
+    dbprintf(("%s: %s: our side is %s.%d\n",
+	      debug_prefix(NULL),
+	      f,
+	      inet_ntoa(claddr.sin_addr),
 	      ntohs(claddr.sin_port)));
 
-    if(sendsize != DEFAULT_SIZE) 
+    if(sendsize >= 0) 
 	try_socksize(client_socket, SO_SNDBUF, sendsize);
-    if(recvsize != DEFAULT_SIZE) 
+    if(recvsize >= 0) 
 	try_socksize(client_socket, SO_RCVBUF, recvsize);
 
     if (localport != NULL)
@@ -282,6 +307,32 @@ out:
     return client_socket;
 }
 
+int
+stream_client_privileged(hostname, port, sendsize, recvsize, localport)
+    char *hostname;
+    int port, sendsize, recvsize, *localport;
+{
+    return stream_client_internal(hostname,
+				  port,
+				  sendsize,
+				  recvsize,
+				  localport,
+				  1);
+}
+
+int
+stream_client(hostname, port, sendsize, recvsize, localport)
+    char *hostname;
+    int port, sendsize, recvsize, *localport;
+{
+    return stream_client_internal(hostname,
+				  port,
+				  sendsize,
+				  recvsize,
+				  localport,
+				  0);
+}
+
 /* don't care about these values */
 static struct sockaddr_in addr;
 static int addrlen;
@@ -293,7 +344,6 @@ int server_socket, timeout, sendsize, recvsize;
     struct timeval tv;
     int nfound, connected_socket;
     int save_errno;
-    struct in_addr in_addr;
 
     assert(server_socket >= 0);
 
@@ -301,16 +351,20 @@ int server_socket, timeout, sendsize, recvsize;
     tv.tv_usec = 0;
     FD_ZERO(&readset);
     FD_SET(server_socket, &readset);
-    nfound = select(server_socket+1, (SELECT_ARG_TYPE *)&readset, NULL, NULL, &tv);
+    nfound = select(server_socket+1,
+		    (SELECT_ARG_TYPE *)&readset,
+		    NULL,
+		    NULL,
+		    &tv);
     if(nfound <= 0 || !FD_ISSET(server_socket, &readset)) {
 	save_errno = errno;
 	if(nfound < 0) {
 	    dbprintf(("%s: stream_accept: select() failed: %s\n",
-		      get_pname(),
+		      debug_prefix_time(NULL),
 		      strerror(save_errno)));
 	} else if(nfound == 0) {
 	    dbprintf(("%s: stream_accept: timeout after %d second%s\n",
-		      get_pname(),
+		      debug_prefix_time(NULL),
 		      timeout,
 		      (timeout == 1) ? "" : "s"));
 	    save_errno = ENOENT;			/* ??? */
@@ -320,7 +374,7 @@ int server_socket, timeout, sendsize, recvsize;
 	    for(i = 0; i < server_socket + 1; i++) {
 		if(FD_ISSET(i, &readset)) {
 		    dbprintf(("%s: stream_accept: got fd %d instead of %d\n",
-			      get_pname(),
+			      debug_prefix_time(NULL),
 			      i,
 			      server_socket));
 		}
@@ -339,15 +393,14 @@ int server_socket, timeout, sendsize, recvsize;
 	if(connected_socket < 0) {
 	    save_errno = errno;
 	    dbprintf(("%s: stream_accept: accept() failed: %s\n",
-		      get_pname(),
+		      debug_prefix_time(NULL),
 		      strerror(save_errno)));
 	    errno = save_errno;
 	    return -1;
 	}
-	memcpy(&in_addr, &addr.sin_addr, sizeof(addr.sin_addr));
 	dbprintf(("%s: stream_accept: connection from %s.%d\n",
-	          get_pname(),
-	          inet_ntoa(in_addr),
+	          debug_prefix_time(NULL),
+	          inet_ntoa(addr.sin_addr),
 	          ntohs(addr.sin_port)));
 	/*
 	 * Make certain we got an inet connection and that it is not
@@ -358,21 +411,21 @@ int server_socket, timeout, sendsize, recvsize;
 	}
 	if(addr.sin_family != AF_INET) {
 	    dbprintf(("%s: family is %d instead of %d(AF_INET): ignored\n",
-		      get_pname(),
+		      debug_prefix_time(NULL),
 		      addr.sin_family,
 		      AF_INET));
 	}
 	if(ntohs(addr.sin_port) == 20) {
 	    dbprintf(("%s: remote port is %d: ignored\n",
-		      get_pname(),
+		      debug_prefix_time(NULL),
 		      ntohs(addr.sin_port)));
 	}
 	aclose(connected_socket);
     }
 
-    if(sendsize != DEFAULT_SIZE) 
+    if(sendsize >= 0) 
 	try_socksize(connected_socket, SO_SNDBUF, sendsize);
-    if(recvsize != DEFAULT_SIZE) 
+    if(recvsize >= 0) 
 	try_socksize(connected_socket, SO_RCVBUF, recvsize);
 
     return connected_socket;
@@ -390,12 +443,12 @@ int sock, which, size;
 	size -= 1024;
     if(size > 1024) {
 	dbprintf(("%s: try_socksize: %s buffer size is %d\n",
-		  get_pname(),
+		  debug_prefix(NULL),
 		  (which == SO_SNDBUF) ? "send" : "receive",
 		  size));
     } else {
 	dbprintf(("%s: try_socksize: could not allocate %s buffer of %d\n",
-		  get_pname(),
+		  debug_prefix(NULL),
 		  (which == SO_SNDBUF) ? "send" : "receive",
 		  origsize));
     }
