@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amrecover.c,v 1.29.4.8 2001/02/28 02:11:23 jrjackson Exp $
+ * $Id: amrecover.c,v 1.29.4.7.4.6.2.7 2003/01/04 04:33:32 martinea Exp $
  *
  * an interactive program for recovering backed-up files
  */
@@ -39,6 +39,7 @@
 #include <netinet/ip.h>
 #endif
 #include "stream.h"
+#include "amfeatures.h"
 #include "amrecover.h"
 #include "getfsent.h"
 #include "dgram.h"
@@ -67,7 +68,7 @@
 #endif
 
 extern int process_line P((char *line));
-int guess_disk P((char *cwd, int cwd_len, char **dn_guess, char **mpt_guess));
+int guess_disk P((char *cwd, size_t cwd_len, char **dn_guess, char **mpt_guess));
 
 #define USAGE "Usage: amrecover [[-C] <config>] [-s <index-server>] [-t <tape-server>] [-d <tape-device>]\n"
 
@@ -85,6 +86,9 @@ int quit_prog;				/* set when time to exit parser */
 char *tape_server_name = NULL;
 int tape_server_socket;
 char *tape_device_name = NULL;
+am_feature_t *our_features = NULL;
+am_feature_t *their_features = NULL;
+
 
 #ifndef HAVE_LIBREADLINE
 /*
@@ -112,7 +116,7 @@ int get_line ()
 {
     char *line = NULL;
     char *part = NULL;
-    int len;
+    size_t len;
 
     while(1) {
 	if((part = areads(server_socket)) == NULL) {
@@ -127,8 +131,9 @@ int get_line ()
 				get_pname(),
 				strerror(save_errno));
 	    } else {
-		fprintf(stderr, "%s: Unexpected server end of file\n",
-			get_pname());
+		fprintf(stderr, "%s: Unexpected end of file, check amindexd*debug on server %s\n",
+			get_pname(),
+			server_name);
 	    }
 	    amfree(line);
 	    amfree(server_line);
@@ -207,7 +212,8 @@ int server_happy ()
 int send_command(cmd)
 char *cmd;
 {
-    int l, n, s;
+    size_t l, n;
+    ssize_t s;
     char *end;
 
     /*
@@ -275,7 +281,7 @@ int signum;
 void clean_pathname(s)
 char *s;
 {
-    int length;
+    size_t length;
     length = strlen(s);
 
     /* remove "/" at end of path */
@@ -299,11 +305,11 @@ char *s;
    current directory */
 int guess_disk (cwd, cwd_len, dn_guess, mpt_guess)
      char *cwd, **dn_guess, **mpt_guess;
-     int cwd_len;
+     size_t cwd_len;
 {
-    int longest_match = 0;
-    int current_length;
-    int cwd_length;
+    size_t longest_match = 0;
+    size_t current_length;
+    size_t cwd_length;
     int local_disk = 0;
     generic_fsent_t fsent;
     char *fsname = NULL;
@@ -314,13 +320,19 @@ int guess_disk (cwd, cwd_len, dn_guess, mpt_guess)
     if (getcwd(cwd, cwd_len) == NULL)
 	return -1;
     cwd_length = strlen(cwd);
+    dbprintf(("guess_disk: %d: \"%s\"\n", cwd_length, cwd));
 
     if (open_fstab() == 0)
 	return -1;
 
     while (get_fstab_nextentry(&fsent))
     {
-	current_length = fsent.mntdir ? strlen(fsent.mntdir) : 0;
+	current_length = fsent.mntdir ? strlen(fsent.mntdir) : (size_t)0;
+	dbprintf(("guess_disk: %d: %d: \"%s\": \"%s\"\n",
+		  longest_match,
+		  current_length,
+		  fsent.mntdir ? fsent.mntdir : "(mntdir null)",
+		  fsent.fsname ? fsent.fsname : "(fsname null)"));
 	if ((current_length > longest_match)
 	    && (current_length <= cwd_length)
 	    && (strncmp(fsent.mntdir, cwd, current_length) == 0))
@@ -337,6 +349,9 @@ int guess_disk (cwd, cwd_len, dn_guess, mpt_guess)
 	        fsname = newstralloc(fsname,fsent.fsname+strlen(DEV_PREFIX));
 	    }
 	    local_disk = is_local_fstype(&fsent);
+	    dbprintf(("guess_disk: local_disk = %d, fsname = \"%s\"\n",
+		      local_disk,
+		      fsname));
 	}
     }
     close_fstab();
@@ -356,6 +371,7 @@ int guess_disk (cwd, cwd_len, dn_guess, mpt_guess)
     /* have mount point now */
     /* disk name may be specified by mount point (logical name) or
        device name, have to determine */
+    printf("Trying disk %s ...\n", *mpt_guess);
     disk_try = stralloc2("DISK ", *mpt_guess);		/* try logical name */
     if (exchange(disk_try) == -1)
 	exit(1);
@@ -366,6 +382,7 @@ int guess_disk (cwd, cwd_len, dn_guess, mpt_guess)
 	amfree(fsname);
 	return 1;
     }
+    printf("Trying disk %s ...\n", fsname);
     disk_try = stralloc2("DISK ", fsname);		/* try device name */
     if (exchange(disk_try) == -1)
 	exit(1);
@@ -422,10 +439,12 @@ char **argv;
     set_pname("amrecover");
     dbopen();
 
+#ifndef IGNORE_UID_CHECK
     if (geteuid() != 0) {
 	erroutput_type |= ERR_SYSLOG;
 	error("amrecover must be run by root");
     }
+#endif
 
     localhost = alloc(MAX_HOSTNAME_LENGTH+1);
     if (gethostname(localhost, MAX_HOSTNAME_LENGTH) != 0) {
@@ -449,11 +468,7 @@ char **argv;
 	 */
 	char **new_argv;
 
-	new_argv = (char **) malloc ((argc + 1 + 1) * sizeof (*new_argv));
-	if (new_argv == NULL)
-	{
-	    error("no memory for argument list");
-	}
+	new_argv = (char **) alloc ((argc + 1 + 1) * sizeof (*new_argv));
 	new_argv[0] = argv[0];
 	new_argv[1] = "-C";
 	for (i = 1; i < argc; i++)
@@ -518,11 +533,12 @@ char **argv;
     {
 	error("%s/tcp unknown protocol", service_name);
     }
-    server_socket = stream_client(server_name,
-				  ntohs(sp->s_port),
-				  DEFAULT_SIZE,
-				  DEFAULT_SIZE,
-				  &my_port);
+    amfree(service_name);
+    server_socket = stream_client_privileged(server_name,
+					     ntohs(sp->s_port),
+					     -1,
+					     -1,
+					     &my_port);
     if (server_socket < 0)
     {
 	error("cannot connect to %s: %s", server_name, strerror(errno));
@@ -571,6 +587,26 @@ char **argv;
     memset(line, '\0', strlen(line));
     amfree(line);
 
+    /* try to get the features form the server */
+    {
+	char *our_feature_string = NULL;
+	char *their_feature_string = NULL;
+
+	our_features = am_init_feature_set();
+	our_feature_string = am_feature_to_string(our_features);
+	line = stralloc2("FEATURES ", our_feature_string);
+	if(exchange(line) == 0) {
+	    their_feature_string = stralloc(server_line+13);
+	    their_features = am_string_to_feature(their_feature_string);
+	}
+	else {
+	    their_features = am_set_default_feature_set();
+        }
+	amfree(our_feature_string);
+	amfree(their_feature_string);
+	amfree(line);
+    }
+
     /* set the date of extraction to be today */
     (void)time(&timer);
     strftime(dump_date, sizeof(dump_date), "%Y-%m-%d", localtime(&timer));
@@ -604,6 +640,7 @@ char **argv;
 		    set_directory(cwd);
 		    if (server_happy() && strcmp(cwd, mpt_guess) != 0)
 		        printf("WARNING: not on root of selected filesystem, check man-page!\n");
+		    amfree(dn_guess);
 		    amfree(mpt_guess);
 		    break;
 
@@ -618,7 +655,7 @@ char **argv;
 		case 2:
 		case -1:
 		default:
-		    printf("Can't determine disk and mount point from $CWD\n");
+		    printf("Can't determine disk and mount point from $CWD '%s'\n", cwd);
 		    /* fake an unhappy server */
 		    server_line[0] = '5';
 		    break;
@@ -629,11 +666,13 @@ char **argv;
     quit_prog = 0;
     do
     {
-	if ((lineread = readline("amrecover> ")) == NULL)
+	if ((lineread = readline("amrecover> ")) == NULL) {
+	    clearerr(stdin);
+	    putchar('\n');
 	    break;
+	}
 	if (lineread[0] != '\0') 
 	{
-
 	    add_history(lineread);
 	    process_line(lineread);	/* act on line's content */
 	}
