@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: extract_list.c,v 1.43.2.16 2001/03/21 20:55:21 jrjackson Exp $
+ * $Id: extract_list.c,v 1.43.2.13.4.6.2.19 2003/03/04 21:13:58 martinea Exp $
  *
  * implements the "extract" command in amrecover
  */
@@ -57,12 +57,16 @@ typedef struct EXTRACT_LIST
     char date[11];			/* date tape created */
     int  level;				/* level of dump */
     char tape[256];			/* tape label */
+    int fileno;				/* fileno on tape */
     EXTRACT_LIST_ITEM *files;		/* files to get off tape */
 
     struct EXTRACT_LIST *next;
 }
 EXTRACT_LIST;
 
+#define SKIP_TAPE 2
+#define RETRY_TAPE 3
+
 char *dump_device_name = NULL;
 
 extern char *localhost;
@@ -77,19 +81,21 @@ static EXTRACT_LIST *extract_list = NULL;
 unsigned short samba_extract_method = SAMBA_TAR;
 #endif /* SAMBA_CLIENT */
 
-#define READ_TIMEOUT	30*60
+#define READ_TIMEOUT	240*60
 
+static int okay_to_continue P((int, int,  int));
 
-int read_buffer(datafd, buffer, buflen)
+ssize_t read_buffer(datafd, buffer, buflen)
 int datafd;
 char *buffer;
-int buflen;
+size_t buflen;
 {
-    int maxfd, nfound, size = 0;
+    int maxfd, nfound = 0;
+    ssize_t size = 0;
     fd_set readset, selectset;
     struct timeval timeout;
     char *dataptr;
-    int spaceleft;
+    size_t spaceleft;
     int eof;
 
     if(datafd < 0 || datafd >= FD_SETSIZE) {
@@ -136,8 +142,9 @@ int buflen;
 	    case 0:
 		spaceleft -= size;
 		dataptr += size;
-		fprintf(stderr,"EOF, check amidxtaped.debug file on %s.\n",
-			       tape_server_name);
+		fprintf(stderr,
+			"EOF, check amidxtaped.<timestamp>.debug file on %s.\n",
+			tape_server_name);
 		break;
 	    default:
 		spaceleft -= size;
@@ -150,7 +157,7 @@ int buflen;
     if(size<0) {
 	return -1;
     }
-    return(buflen-spaceleft);
+    return (ssize_t)(buflen-spaceleft);
 }
 
 EXTRACT_LIST *first_tape_list P((void))
@@ -255,7 +262,7 @@ DIR_ITEM *ditem;
     for (this = extract_list; this != NULL; this = this->next)
     {
 	/* see if this is the list for the tape */	
-	if (strcmp(this->tape, ditem->tape) == 0)
+	if (this->level == ditem->level && strcmp(this->tape, ditem->tape) == 0)
 	{
 	    /* yes, so add to list */
 	    curr=this->files;
@@ -267,11 +274,7 @@ DIR_ITEM *ditem;
 		}
 		curr=curr->next;
 	    }
-	    if ((that = (EXTRACT_LIST_ITEM *)malloc(sizeof(EXTRACT_LIST_ITEM)))
-		== NULL) {
-		amfree(ditem_path);
-		return -1;
-	    }
+	    that = (EXTRACT_LIST_ITEM *)alloc(sizeof(EXTRACT_LIST_ITEM));
 	    strncpy(that->path, ditem_path, sizeof(that->path)-1);
 	    that->path[sizeof(that->path)-1] = '\0';
 	    that->next = this->files;
@@ -282,20 +285,14 @@ DIR_ITEM *ditem;
     }
 
     /* so this is the first time we have seen this tape */
-    if ((this = (EXTRACT_LIST *)malloc(sizeof(EXTRACT_LIST))) == NULL) {
-	amfree(ditem_path);
-	return -1;
-    }
+    this = (EXTRACT_LIST *)alloc(sizeof(EXTRACT_LIST));
     strncpy(this->tape, ditem->tape, sizeof(this->tape)-1);
     this->tape[sizeof(this->tape)-1] ='\0';
     this->level = ditem->level;
+    this->fileno = ditem->fileno;
     strncpy(this->date, ditem->date, sizeof(this->date)-1);
     this->date[sizeof(this->date)-1] = '\0';
-    if ((that = (EXTRACT_LIST_ITEM *)malloc(sizeof(EXTRACT_LIST_ITEM)))
-	== NULL) {
-	amfree(ditem_path);
-	return -1;
-    }
+    that = (EXTRACT_LIST_ITEM *)alloc(sizeof(EXTRACT_LIST_ITEM));
     strncpy(that->path, ditem_path, sizeof(that->path)-1);
     that->path[sizeof(that->path)-1] = '\0';
     that->next = NULL;
@@ -346,7 +343,7 @@ DIR_ITEM *ditem;
     for (this = extract_list; this != NULL; this = this->next)
     {
 	/* see if this is the list for the tape */	
-	if (strcmp(this->tape, ditem->tape) == 0)
+	if (this->level == ditem->level && strcmp(this->tape, ditem->tape) == 0)
 	{
 	    /* yes, so find file on list */
 	    that = this->files;
@@ -436,7 +433,8 @@ char *regex;
     char *path_on_disk_slash = NULL;
     char *cmd = NULL;
     char *err = NULL;
-    int  i;
+    int i;
+    int j;
     char *dir, *dir_undo, dir_undo_ch = '\0';
     char *ditem_path = NULL;
     char *l = NULL;
@@ -453,8 +451,8 @@ char *regex;
     dbprintf(("add_file: Looking for \"%s\"\n", regex));
 
     /* remove "/" at end of path */
-    i = strlen(regex)-1;
-    while(i >= 0 && regex[i] == '/') regex[i--] = '\0';
+    j = strlen(regex)-1;
+    while(j >= 0 && regex[j] == '/') regex[j--] = '\0';
 
     /* convert path (assumed in cwd) to one on disk */
     if (strcmp(disk_path, "/") == 0)
@@ -478,9 +476,9 @@ char *regex;
 	    || match(path_on_disk_slash, ditem->path))
 	{
 	    found_one = 1;
-	    i = strlen(ditem->path);
-	    if((i > 0 && ditem->path[i-1] == '/')
-	       || (i > 1 && ditem->path[i-2] == '/' && ditem->path[i-1] == '.'))
+	    j = strlen(ditem->path);
+	    if((j > 0 && ditem->path[j-1] == '/')
+	       || (j > 1 && ditem->path[j-2] == '/' && ditem->path[j-1] == '.'))
 	    {	/* It is a directory */
 
 		ditem_path = newstralloc(ditem_path, ditem->path);
@@ -576,6 +574,15 @@ char *regex;
 			continue;
 		    }
 
+		    if(am_has_feature(their_features, fe_amindexd_fileno_in_ORLD)) {
+			skip_whitespace(s, ch);
+			if(ch == '\0' || sscanf(s - 1, "%d", &lditem.fileno) != 1) {
+			    err = "bad reply: cannot parse fileno field";
+			    continue;
+			}
+			skip_integer(s, ch);
+		    }
+
 		    skip_whitespace(s, ch);
 		    if(ch == '\0') {
 			err = "bad reply: missing directory field";
@@ -700,11 +707,12 @@ char *regex;
     char *path_on_disk_slash = NULL;
     char *cmd = NULL;
     char *err = NULL;
-    int  i;
+    int i;
+    int j;
     char *date, *date_undo, date_undo_ch = '\0';
     char *tape, *tape_undo, tape_undo_ch = '\0';
     char *dir, *dir_undo, dir_undo_ch = '\0';
-    int  level;
+    int  level, fileno;
     char *ditem_path = NULL;
     char *l = NULL;
     int  deleted;
@@ -719,8 +727,8 @@ char *regex;
 
     dbprintf(("delete_file: Looking for \"%s\"\n", path));
     /* remove "/" at the end of the path */
-    i = strlen(regex)-1;
-    while(i >= 0 && regex[i] == '/') regex[i--] = '\0';
+    j = strlen(regex)-1;
+    while(j >= 0 && regex[j] == '/') regex[j--] = '\0';
 
     /* convert path (assumed in cwd) to one on disk */
     if (strcmp(disk_path, "/") == 0)
@@ -743,9 +751,9 @@ char *regex;
 	    || match(path_on_disk_slash, ditem->path))
 	{
 	    found_one = 1;
-	    i = strlen(ditem->path);
-	    if((i > 0 && ditem->path[i-1] == '/')
-	       || (i > 1 && ditem->path[i-2] == '/' && ditem->path[i-1] == '.'))
+	    j = strlen(ditem->path);
+	    if((j > 0 && ditem->path[j-1] == '/')
+	       || (j > 1 && ditem->path[j-2] == '/' && ditem->path[j-1] == '.'))
 	    {	/* It is a directory */
 		ditem_path = newstralloc(ditem_path, ditem->path);
 		clean_pathname(ditem_path);
@@ -841,6 +849,15 @@ char *regex;
 		    tape_undo_ch = *tape_undo;
 		    *tape_undo = '\0';
 
+		    if(am_has_feature(their_features, fe_amindexd_fileno_in_ORLD)) {
+			skip_whitespace(s, ch);
+			if(ch == '\0' || sscanf(s - 1, "%d", &fileno) != 1) {
+			    err = "bad reply: cannot parse fileno field";
+			    continue;
+			}
+			skip_integer(s, ch);
+		    }
+
 		    skip_whitespace(s, ch);
 		    if(ch == '\0') {
 			err = "bad reply: missing directory field";
@@ -984,20 +1001,78 @@ int is_extract_list_nonempty P((void))
 
 /* prints continue prompt and waits for response,
    returns 0 if don't, non-0 if do */
-static int okay_to_continue P((void))
+static int okay_to_continue(allow_tape, allow_skip, allow_retry)
+    int allow_tape;
+    int allow_skip;
+    int allow_retry;
 {
     int ch;
-    int ret;
+    int ret = -1;
+    char *line = NULL;
+    char *s;
+    char *prompt;
+    int get_tape;
 
-    printf("Continue? [Y/n]: ");
-    fflush(stdout); fflush(stderr);
-    while((ch = getchar()) != EOF && ch != '\n' && isspace(ch)) {}
-    if (ch == '\n' || ch == 'Y' || ch == 'y') {
-	ret = 1;
-    } else {
-	ret = 0;
+    get_tape = 0;
+    while (ret < 0) {
+	if (get_tape) {
+	    prompt = "New tape device [?]: ";
+	} else if (allow_tape && allow_skip) {
+	    prompt = "Continue [?/Y/n/s/t]? ";
+	} else if (allow_tape && !allow_skip) {
+	    prompt = "Continue [?/Y/n/t]? ";
+	} else if (allow_retry) {
+	    prompt = "Continue [?/Y/n/r]? ";
+	} else {
+	    prompt = "Continue [?/Y/n]? ";
+	}
+	fputs(prompt, stdout);
+	fflush(stdout); fflush(stderr);
+	amfree(line);
+	if ((line = agets(stdin)) == NULL) {
+	    putchar('\n');
+	    clearerr(stdin);
+	    if (get_tape) {
+		get_tape = 0;
+		continue;
+	    }
+	    ret = 0;
+	    break;
+	}
+	s = line;
+	while ((ch = *s++) != '\0' && isspace(ch)) {}
+	if (ch == '?') {
+	    if (get_tape) {
+		printf("Enter a new device ([host:]device) or \"default\"\n");
+	    } else {
+		printf("Enter \"y\"es to continue, \"n\"o to stop");
+		if(allow_skip) {
+		    printf(", \"s\"kip this tape");
+		}
+		if(allow_retry) {
+		    printf(" or \"r\"etry this tape");
+		}
+		if (allow_tape) {
+		    printf(" or \"t\"ape to change tape drives");
+		}
+		putchar('\n');
+	    }
+	} else if (get_tape) {
+	    set_tape(s - 1);
+	    get_tape = 0;
+	} else if (ch == '\0' || ch == 'Y' || ch == 'y') {
+	    ret = 1;
+	} else if (allow_tape && (ch == 'T' || ch == 't')) {
+	    get_tape = 1;
+	} else if (ch == 'N' || ch == 'n') {
+	    ret = 0;
+	} else if (allow_retry && (ch == 'R' || ch == 'r')) {
+	    ret = RETRY_TAPE;
+	} else if (allow_skip && (ch == 'S' || ch == 's')) {
+	    ret = SKIP_TAPE;
+	}
     }
-    while(ch != EOF && ch != '\n') ch = getchar();
+    amfree(line);
     return ret;
 }
 
@@ -1005,7 +1080,8 @@ static void send_to_tape_server(tss, cmd)
 int tss;
 char *cmd;
 {
-    int l, n, s;
+    size_t l, n;
+    ssize_t s;
     char *end;
 
     for (l = 0, n = strlen(cmd); l < n; l += s)
@@ -1027,12 +1103,15 @@ char *cmd;
 /* start up connection to tape server and set commands to initiate
    transfer of dump image.
    Return tape server socket on success, -1 on error. */
-static int extract_files_setup P((void))
+static int extract_files_setup(label, fsf)
+char *label;
+int fsf;
 {
     struct servent *sp;
     int my_port;
     int tape_server_socket;
     char *disk_regex = NULL;
+    char *host_regex = NULL;
     char *service_name = NULL;
     char *line = NULL;
     char *clean_datestamp, *ch, *ch1;
@@ -1049,11 +1128,11 @@ static int extract_files_setup P((void))
     amfree(service_name);
     seteuid(0);					/* it either works ... */
     setegid(0);
-    tape_server_socket = stream_client(tape_server_name,
-				       ntohs(sp->s_port),
-				       DEFAULT_SIZE,
-				       DATABUF_SIZE*2,
-				       &my_port);
+    tape_server_socket = stream_client_privileged(tape_server_name,
+						  ntohs(sp->s_port),
+						  -1,
+						  STREAM_BUFSIZE,
+						  &my_port);
     if (tape_server_socket < 0)
     {
 	printf("cannot connect to %s: %s\n", tape_server_name, strerror(errno));
@@ -1107,6 +1186,30 @@ static int extract_files_setup P((void))
 
     *ch1 = '\0';
 
+    host_regex = alloc(strlen(dump_hostname) * 2 + 3);
+
+    ch = dump_hostname;
+    ch1 = host_regex;
+
+    /* we want to force amrestore to only match dump_hostname exactly */
+    *(ch1++) = '^';
+
+    /* We need to escape some characters first... NT compatibilty crap */
+    for (; *ch != 0; ch++, ch1++) {
+	switch (*ch) {     /* done this way in case there are more */
+	case '$':
+	    *(ch1++) = '\\';
+	    /* no break; we do want to fall through... */
+	default:
+	    *ch1 = *ch;
+	}
+    }
+
+    /* we want to force amrestore to only match dump_hostname exactly */
+    *(ch1++) = '$';
+
+    *ch1 = '\0';
+
     clean_datestamp = stralloc(dump_datestamp);
     for(ch=ch1=clean_datestamp;*ch1 != '\0';ch1++) {
 	if(*ch1 != '-') {
@@ -1115,59 +1218,102 @@ static int extract_files_setup P((void))
 	}
     }
     *ch = '\0';
-    /* send to the tape server what tape file we want */
-    /* 6 args:
-     *   "-h"
-     *   "-p"
-     *   "tape device"
-     *   "hostname"
-     *   "diskname"
-     *   "datestamp"
-     */
-    send_to_tape_server(tape_server_socket, "6");
-    send_to_tape_server(tape_server_socket, "-h");
-    send_to_tape_server(tape_server_socket, "-p");
-    send_to_tape_server(tape_server_socket, dump_device_name);
-    send_to_tape_server(tape_server_socket, dump_hostname);
-    send_to_tape_server(tape_server_socket, disk_regex);
-    send_to_tape_server(tape_server_socket, clean_datestamp);
 
-    dbprintf(("Started amidxtaped with arguments \"6 -h -p %s %s %s %s\"\n",
-	      dump_device_name, dump_hostname, disk_regex, clean_datestamp));
+    if(am_has_feature(their_features, fe_amidxtaped_header) &&
+       am_has_feature(their_features, fe_amidxtaped_device) &&
+       am_has_feature(their_features, fe_amidxtaped_host) &&
+       am_has_feature(their_features, fe_amidxtaped_disk) &&
+       am_has_feature(their_features, fe_amidxtaped_datestamp)) {
+
+	char *tt = NULL;
+
+	if(am_has_feature(their_features, fe_amidxtaped_config)) {
+	    tt = newstralloc2(tt, "CONFIG=", config);
+	    send_to_tape_server(tape_server_socket, tt);
+	}
+	if(am_has_feature(their_features, fe_amidxtaped_label) &&
+	   label && label[0] != '/') {
+	    tt = newstralloc2(tt,"LABEL=",label);
+	    send_to_tape_server(tape_server_socket, tt);
+	}
+	if(am_has_feature(their_features, fe_amidxtaped_fsf)) {
+	    char v_fsf[100];
+	    ap_snprintf(v_fsf, 99, "%d", fsf);
+	    tt = newstralloc2(tt, "FSF=",v_fsf);
+	    send_to_tape_server(tape_server_socket, tt);
+	}
+	send_to_tape_server(tape_server_socket, "HEADER");
+	tt = newstralloc2(tt, "DEVICE=", dump_device_name);
+	send_to_tape_server(tape_server_socket, tt);
+	tt = newstralloc2(tt, "HOST=", host_regex);
+	send_to_tape_server(tape_server_socket, tt);
+	tt = newstralloc2(tt, "DISK=", disk_regex);
+	send_to_tape_server(tape_server_socket, tt);
+	tt = newstralloc2(tt, "DATESTAMP=", clean_datestamp);
+	send_to_tape_server(tape_server_socket, tt);
+	send_to_tape_server(tape_server_socket, "END");
+	amfree(tt);
+    }
+    else if(1 /* am_has_feature(their_features, fe_amidxtaped_nargs) */) {
+	/* 2.4.3 doesn't set fe_amidxtaped_nargs but support it */
+	/* must be supported without test until 2005 */
+
+	/* send to the tape server what tape file we want */
+	/* 6 args:
+	 *   "-h"
+	 *   "-p"
+	 *   "tape device"
+	 *   "hostname"
+	 *   "diskname"
+	 *   "datestamp"
+	 */
+	send_to_tape_server(tape_server_socket, "6");
+	send_to_tape_server(tape_server_socket, "-h");
+	send_to_tape_server(tape_server_socket, "-p");
+	send_to_tape_server(tape_server_socket, dump_device_name);
+	send_to_tape_server(tape_server_socket, host_regex);
+	send_to_tape_server(tape_server_socket, disk_regex);
+	send_to_tape_server(tape_server_socket, clean_datestamp);
+
+	dbprintf(("Started amidxtaped with arguments \"6 -h -p %s %s %s %s\"\n",
+		  dump_device_name, host_regex, disk_regex, clean_datestamp));
+    }
 
     amfree(disk_regex);
+    amfree(host_regex);
+    amfree(clean_datestamp);
 
     return tape_server_socket;
 }
 
 
-int read_file_header(buffer, file, buflen, tapedev)
+size_t read_file_header(buffer, file, buflen, tapedev)
 char *buffer;
 dumpfile_t *file;
-int buflen;
+size_t buflen;
 int tapedev;
 /*
  * Reads the first block of a tape file.
  */
 {
-    int bytes_read;
+    ssize_t bytes_read;
     bytes_read=read_buffer(tapedev,buffer,buflen);
     if(bytes_read < 0) {
 	error("error reading tape: %s", strerror(errno));
     }
-    else if(bytes_read < buflen) {
+    else if((size_t)bytes_read < buflen) {
 	fprintf(stderr, "%s: short block %d byte%s\n",
-		get_pname(), bytes_read, (bytes_read == 1) ? "" : "s");
+		get_pname(), (int)bytes_read, (bytes_read == 1) ? "" : "s");
 	print_header(stdout, file);
 	error("Can't read file header");
     }
     else { /* bytes_read == buflen */
 	parse_file_header(buffer, file, bytes_read);
     }
-    return(bytes_read);
+    return((size_t)bytes_read);
 }
 
-enum dumptypes {IS_UNKNOWN, IS_DUMP, IS_GNUTAR, IS_TAR, IS_SAMBA};
+enum dumptypes {IS_UNKNOWN, IS_DUMP, IS_GNUTAR, IS_TAR, IS_SAMBA, IS_SAMBA_TAR};
 
 /* exec restore to do the actual restoration */
 static void extract_files_child(in_fd, elist)
@@ -1181,10 +1327,10 @@ static void extract_files_child(in_fd, elist)
     int files_off_tape;
     EXTRACT_LIST_ITEM *fn;
     enum dumptypes dumptype = IS_UNKNOWN;
-    char buffer[TAPE_BLOCK_BYTES];
+    char buffer[DISK_BLOCK_BYTES];
     dumpfile_t file;
-    int buflen;
-    int len_program;
+    size_t buflen;
+    size_t len_program;
     char *cmd = NULL;
     int passwd_field = -1;
 #ifdef SAMBA_CLIENT
@@ -1226,7 +1372,7 @@ static void extract_files_child(in_fd, elist)
 #ifdef SAMBA_CLIENT
 	if (dumptype == IS_UNKNOWN && strcmp(file.program, SAMBA_CLIENT) ==0) {
 	    if (samba_extract_method == SAMBA_TAR)
-	      dumptype = IS_TAR;
+	      dumptype = IS_SAMBA_TAR;
 	    else
 	      dumptype = IS_SAMBA;
 	}
@@ -1243,6 +1389,7 @@ static void extract_files_child(in_fd, elist)
 #endif
     case IS_TAR:
     case IS_GNUTAR:
+    case IS_SAMBA_TAR:
         extra_params = 3;
         break;
     case IS_UNKNOWN:
@@ -1262,14 +1409,8 @@ static void extract_files_child(in_fd, elist)
     	break;
     }
 
-    if ((restore_args
-	 = (char **)malloc((extra_params + files_off_tape + 1)
-			   * sizeof(char *)))
-	== NULL) 
-    {
-  	perror("Couldn't malloc restore_args");
-  	exit(3);
-    }
+    restore_args = (char **)alloc((extra_params + files_off_tape + 1)
+				  * sizeof(char *));
     switch(dumptype) {
     case IS_SAMBA:
 #ifdef SAMBA_CLIENT
@@ -1298,6 +1439,11 @@ static void extract_files_child(in_fd, elist)
 	restore_args[j++] = stralloc("-xpGvf");
 	restore_args[j++] = stralloc("-");	/* data on stdin */
 	break;
+    case IS_SAMBA_TAR:
+    	restore_args[j++] = stralloc("tar");
+	restore_args[j++] = stralloc("-xpvf");
+	restore_args[j++] = stralloc("-");	/* data on stdin */
+	break;
     case IS_UNKNOWN:
     case IS_DUMP:
         restore_args[j++] = stralloc("restore");
@@ -1309,6 +1455,12 @@ static void extract_files_child(in_fd, elist)
             restore_args[j++] = stralloc("-v");
             restore_args[j++] = stralloc("silent");
 	} else
+#endif
+#if defined(VDUMP)
+	if (strcmp(file.program, VDUMP) == 0) {
+            restore_args[j++] = stralloc("xf");
+            restore_args[j++] = stralloc("-");	/* data on stdin */
+	} else
 #endif
 	{
         restore_args[j++] = stralloc("xbf");
@@ -1323,6 +1475,7 @@ static void extract_files_child(in_fd, elist)
 	switch (dumptype) {
     	case IS_TAR:
     	case IS_GNUTAR:
+    	case IS_SAMBA_TAR:
     	case IS_SAMBA:
 	    restore_args[j++] = stralloc2(".", fn->path);
 	    break;
@@ -1361,6 +1514,7 @@ static void extract_files_child(in_fd, elist)
 #endif
     case IS_TAR:
     case IS_GNUTAR:
+    case IS_SAMBA_TAR:
 #ifndef GNUTAR
 	fprintf(stderr, "warning: GNUTAR program not available.\n");
 	cmd = stralloc("tar");
@@ -1446,6 +1600,7 @@ void extract_files P((void))
     char *l;
     int tape_server_socket;
     int first;
+    int otc;
 
     if (!is_extract_list_nonempty())
     {
@@ -1511,7 +1666,7 @@ void extract_files P((void))
     if (samba_extract_method == SAMBA_SMBCLIENT)
       printf("(unless it is a Samba backup, that will go through to the SMB server)\n");
 #endif
-    if (!okay_to_continue())
+    if (!okay_to_continue(0,0,0))
 	return;
     printf("\n");
 
@@ -1522,15 +1677,22 @@ void extract_files P((void))
 	    printf("Extracting from file %s\n",dump_device_name);
 	}
 	else {
-	    dump_device_name = newstralloc(dump_device_name, tape_device_name);
+	    printf("Extracting files using tape drive %s on host %s.\n",
+		   tape_device_name, tape_server_name);
 	    printf("Load tape %s now\n", elist->tape);
-	    if (!okay_to_continue())
+	    otc = okay_to_continue(1,1,0);
+	    if (otc == 0)
 	        return;
+	    else if (otc == SKIP_TAPE) {
+		delete_tape_list(elist); /* skip this tape */
+		continue;
+	    }
+	    dump_device_name = newstralloc(dump_device_name, tape_device_name);
 	}
 	dump_datestamp = newstralloc(dump_datestamp, elist->date);
 
 	/* connect to the tape handler daemon on the tape drive server */
-	if ((tape_server_socket = extract_files_setup()) == -1)
+	if ((tape_server_socket = extract_files_setup(elist->tape, elist->fileno)) == -1)
 	{
 	    fprintf(stderr, "amrecover - can't talk to tape server\n");
 	    return;
@@ -1576,11 +1738,17 @@ void extract_files P((void))
 	    fprintf(stderr,
 		    "extract_list - child returned non-zero status: %d\n",
 		    WEXITSTATUS(child_stat));
-	    if (!okay_to_continue())
+	    otc = okay_to_continue(0,0,1);
+	    if(otc == 0)
 		return;
+	    else if(otc == 1) {
+		delete_tape_list(elist); /* tape failed so delete from list */
+	    }
+	    else { /* RETRY_TAPE */
+	    }
+	}
+	else {
+	    delete_tape_list(elist);	/* tape done so delete from list */
 	}
-
-	/* finish up */
-	delete_tape_list(elist);	/* tape done so delete from list */
     }
 }
