@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: help.c,v 1.4.4.2 2000/01/21 05:08:38 oliva Exp $
+ * $Id: help.c,v 1.4.4.2.4.1.2.1 2002/03/06 19:23:32 martinea Exp $
  *
  * implements the "help" command in amrecover
  */
@@ -38,7 +38,8 @@ void help_list P((void))
 
     printf("add path1 ...     - add to extraction list (shell wildcards)\n");
     printf("addx path1 ...    - add to extraction list (regular expressions)\n");
-    printf("cd directory      - change cwd on virtual file system\n");
+    printf("cd directory      - change cwd on virtual file system (shell wildcards)\n");
+    printf("cdx directory     - change cwd on virtual file system (regular expressions)\n");
     printf("clear             - clear extraction list\n");
     printf("delete path1 ...  - delete from extraction list (shell wildcards)\n");
     printf("deletex path1 ... - delete from extraction list (regular expressions)\n");
@@ -53,6 +54,7 @@ void help_list P((void))
     printf("mode              - show the method used to extract SMB shares\n");
     printf("pwd               - show cwd on virtual file system\n");
     printf("quit\n");
+    printf("listdisk [diskdevice] - list disks\n");
     printf("setdate {YYYY-MM-DD|--MM-DD|---DD} - set date of look\n");
     printf("setdisk diskname [mountpoint] - select disk on dump host\n");
     printf("sethost host      - select dump host\n");
