@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: set_commands.c,v 1.11.2.3 2000/01/21 05:08:41 oliva Exp $
+ * $Id: set_commands.c,v 1.11.2.3.4.2.2.7 2004/02/11 13:15:29 martinea Exp $
  *
  * implements the "set" commands in amrecover
  */
@@ -55,7 +55,6 @@ char *date;
 	cmd = newstralloc2(cmd, "OISD ", disk_path);
 	if (exchange(cmd) == -1)
 	    exit(1);
-	amfree(cmd);
 	if (server_happy())
 	{
 	    suck_dir_list_from_server();
@@ -103,7 +102,7 @@ char *host;
 	 */
 	if ((hp = gethostbyname(host)) != NULL) {
 	    host = hp->h_name;
-	    printf("Trying %s ...\n", host);
+	    printf("Trying host %s ...\n", host);
 	    cmd = newstralloc2(cmd, "HOST ", host);
 	    if (converse(cmd) == -1)
 		exit(1);
@@ -115,7 +114,7 @@ char *host;
 	    {
 	        for (hostp = hp->h_aliases; (host = *hostp) != NULL; hostp++)
 	        {
-		    printf("Trying %s ...\n", host);
+		    printf("Trying host %s ...\n", host);
 		    cmd = newstralloc2(cmd, "HOST ", host);
 		    if (converse(cmd) == -1)
 		        exit(1);
@@ -196,8 +195,6 @@ char *mtpt;
        a more informative message. */
     if (exchange("OISD /") == -1)
 	exit(1);
-    if (exchange("OISD /") == -1)
-	exit(1);
     if (server_happy())
     {
 	disk_path = newstralloc(disk_path, "/");
@@ -212,6 +209,160 @@ char *mtpt;
     }
 }
 
+void list_disk(amdevice)
+char *amdevice;
+{
+    char *cmd = NULL;
+
+    if(amdevice) {
+	cmd = stralloc2("LISTDISK ", amdevice);
+	if (converse(cmd) == -1)
+	    exit(1);
+	amfree(cmd);
+    }
+    else {
+	cmd = stralloc("LISTDISK");
+	if (converse(cmd) == -1)
+	    exit(1);
+	amfree(cmd);
+    }
+}
+
+void cd_glob(glob)
+char *glob;
+{
+    char *regex;
+    char *regex_path;
+    char *s;
+
+    char *path_on_disk = NULL;
+
+    if (disk_name == NULL) {
+	printf("Must select disk before changing directory\n");
+	return;
+    }
+
+    regex = glob_to_regex(glob);
+    dbprintf(("cd_glob (%s) -> %s\n", glob, regex));
+    if ((s = validate_regexp(regex)) != NULL) {
+        printf("\"%s\" is not a valid shell wildcard pattern: ", glob);
+        puts(s);
+	amfree(regex);
+        return;
+    }
+    /*
+     * glob_to_regex() anchors the beginning of the pattern with ^,
+     * but we will be tacking it onto the end of the current directory
+     * in add_file, so strip that off.  Also, it anchors the end with
+     * $, but we need to match a trailing /, add it if it is not there
+     */
+    regex_path = stralloc(regex + 1);
+    amfree(regex);
+    if(regex_path[strlen(regex_path) - 2] != '/' ) {
+	regex_path[strlen(regex_path) - 1] = '\0';
+	strappend(regex_path, "/$");
+    }
+
+    /* convert path (assumed in cwd) to one on disk */
+    if (strcmp(disk_path, "/") == 0)
+        path_on_disk = stralloc2("/", regex_path);
+    else {
+        char *clean_disk_path = clean_regex(disk_path);
+        path_on_disk = vstralloc(clean_disk_path, "/", regex_path, NULL);
+        amfree(clean_disk_path);
+    }
+
+    cd_dir(path_on_disk, glob);
+
+    amfree(regex_path);
+    amfree(path_on_disk);
+}
+
+void cd_regex(regex)
+char *regex;
+{
+    char *s;
+
+    char *path_on_disk = NULL;
+
+    if (disk_name == NULL) {
+	printf("Must select disk before changing directory\n");
+	return;
+    }
+
+    if ((s = validate_regexp(regex)) != NULL) {
+	printf("\"%s\" is not a valid regular expression: ", regex);
+	puts(s);
+	return;
+    }
+
+    /* convert path (assumed in cwd) to one on disk */
+    if (strcmp(disk_path, "/") == 0)
+        path_on_disk = stralloc2("/", regex);
+    else {
+        char *clean_disk_path = clean_regex(disk_path);
+        path_on_disk = vstralloc(clean_disk_path, "/", regex, NULL);
+        amfree(clean_disk_path);
+    }
+
+    cd_dir(path_on_disk, regex);
+
+    amfree(path_on_disk);
+}
+
+void cd_dir(path_on_disk, default_dir)
+char *path_on_disk;
+char *default_dir;
+{
+    char *path_on_disk_slash = NULL;
+    char *dir = NULL;
+
+    int nb_found;
+    size_t i;
+
+    DIR_ITEM *ditem;
+
+    path_on_disk_slash = stralloc2(path_on_disk, "/");
+
+    nb_found = 0;
+
+    for (ditem=get_dir_list(); ditem!=NULL && nb_found <= 1; 
+			       ditem=get_next_dir_item(ditem))
+    {
+	if (match(path_on_disk, ditem->path)
+	    || match(path_on_disk_slash, ditem->path))
+	{
+	    i = strlen(ditem->path);
+	    if((i > 0 && ditem->path[i-1] == '/')
+               || (i > 1 && ditem->path[i-2] == '/' && ditem->path[i-1] == '.'))
+            {   /* It is a directory */
+		char *dir1, *dir2;
+		nb_found++;
+		dir = newstralloc(dir,ditem->path);
+		if(dir[strlen(dir)-1] == '/')
+		    dir[strlen(dir)-1] = '\0'; /* remove last / */
+		/* remove everything before the last / */
+		dir1 = rindex(dir,'/');
+		dir1++;
+		dir2 = stralloc(dir1);
+		amfree(dir);
+		dir = dir2;
+	    }
+	}
+    }
+    amfree(path_on_disk_slash);
+
+    if(nb_found==0) {
+	set_directory(default_dir);
+    }
+    else if(nb_found==1) {
+	set_directory(dir);
+    }
+    else {
+	printf("Too many directory\n");
+    }
+    amfree(dir);
+}
 
 void set_directory(dir)
 char *dir;
@@ -284,6 +435,7 @@ char *dir;
 		/* at top of disk */
 		printf("Invalid directory - Can't cd outside mount point \"%s\"\n",
 		       mount_point);
+		amfree(new_dir);
 		amfree(ldir);
 		return;
 	    }
@@ -348,11 +500,22 @@ void set_tape (tape)
     if (tapedev)
     {
 	if (tapedev != tape) {
-	    *tapedev = '\0';
-	    tape_server_name = newstralloc(tape_server_name, tape);
-	} else
+	    if((strchr(tapedev+1, ':') == NULL) &&
+	       (strncmp(tape, "null:", 5) == 0 ||
+		strncmp(tape, "rait:", 5) == 0 ||
+		strncmp(tape, "file:", 5) == 0 ||
+		strncmp(tape, "tape:", 5) == 0)) {
+		tapedev = tape;
+	    }
+	    else {
+		*tapedev = '\0';
+		tape_server_name = newstralloc(tape_server_name, tape);
+		++tapedev;
+	    }
+	} else { /* reset server_name if start with : */
 	    amfree(tape_server_name);
-	++tapedev;
+	    ++tapedev;
+	}
     } else
 	tapedev = tape;
     
@@ -365,7 +528,7 @@ void set_tape (tape)
     }
 
     if (tape_device_name)
-	printf ("Using tape %s", tape_device_name);
+	printf ("Using tape \"%s\"", tape_device_name);
     else
 	printf ("Using default tape");
 
