@@ -24,40 +24,37 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amrestore.c,v 1.28.2.4 2000/12/03 20:56:20 jrjackson Exp $
+ * $Id: amrestore.c,v 1.28.2.4.4.3.2.8 2003/02/09 04:33:13 jrjackson Exp $
  *
  * retrieves files from an amanda tape
  */
 /*
- * usage: amrestore [-r|-c|-C] [-p] [-h] tape-device|holdingfile [hostname [diskname [datestamp [hostname [diskname [datestamp ... ]]]]]]
- *
  * Pulls all files from the tape that match the hostname, diskname and
  * datestamp regular expressions.
  *
- * For example, specifying "rz1" as the diskname matches "rz1a",
- * "rz1g" etc on the tape.
- *
- * Command line options:
- *	-p   put output on stdout
- *	-c   write compressed with COMPRESS_FAST_OPT
- *	-C   write compressed with COMPRESS_BEST_OPT
- *	-r   raw, write file as is on tape (with header, possibly compressed)
- *	-h   write the header too
+ * If the header is output, only up to DISK_BLOCK_BYTES worth of it is
+ * sent, regardless of the tape blocksize.  This makes the disk image
+ * look like a holding disk image, and also makes it easier to remove
+ * the header (e.g. in amrecover) since it has a fixed size.
  */
 
 #include "amanda.h"
 #include "tapeio.h"
 #include "fileheader.h"
+#include "util.h"
 
 #define CREAT_MODE	0640
 
-char buffer[TAPE_BLOCK_BYTES];
-
+char *buffer = NULL;
 
 int compflag, rawflag, pipeflag, headerflag;
-int buflen, got_sigpipe, file_number;
+int got_sigpipe, file_number;
 pid_t compress_pid = -1;
 char *compress_type = COMPRESS_FAST_OPT;
+int tapedev;
+int bytes_read;
+long blocksize = -1;
+long filefsf = -1;
 
 /* local functions */
 
@@ -66,10 +63,9 @@ void handle_sigpipe P((int sig));
 int disk_match P((dumpfile_t *file, char *datestamp, 
 		  char *hostname, char *diskname));
 char *make_filename P((dumpfile_t *file));
-int read_file_header P((char *buffer, dumpfile_t *file,
-			int buflen, int tapedev));
-void restore P((dumpfile_t *file, char *filename,
-		int tapedev, int isafile));
+void read_file_header P((dumpfile_t *file, int isafile));
+static int get_block P((int isafile));
+void restore P((dumpfile_t *file, char *filename, int isafile));
 void usage P((void));
 int main P((int argc, char **argv));
 
@@ -106,9 +102,9 @@ char *datestamp, *hostname, *diskname;
 {
     if(file->type != F_DUMPFILE) return 0;
 
-    if((*hostname == '\0' || match(hostname, file->name)) &&
-       (*diskname == '\0' || match(diskname, file->disk)) &&
-       (*datestamp == '\0' || match(datestamp, file->datestamp)))
+    if((*hostname == '\0' || match_host(hostname, file->name)) &&
+       (*diskname == '\0' || match_disk(diskname, file->disk)) &&
+       (*datestamp== '\0' || match_datestamp(datestamp, file->datestamp)))
 	return 1;
     else
 	return 0;
@@ -137,40 +133,80 @@ dumpfile_t *file;
 }
 
 
-int read_file_header(buffer, file, buflen, tapedev)
-char *buffer;
+static int get_block(isafile)
+int isafile;
+{
+    static int test_blocksize = 1;
+    int buflen;
+
+    /*
+     * If this is the first call, set the blocksize if it was not on
+     * the command line.  Allocate the I/O buffer in any case.
+     *
+     * For files, the blocksize is always DISK_BLOCK_BYTES.  For tapes,
+     * we allocate a large buffer and set the size to the length of the
+     * first (successful) record.
+     */
+    buflen = blocksize;
+    if(test_blocksize) {
+	if(blocksize < 0) {
+	    if(isafile) {
+		blocksize = buflen = DISK_BLOCK_BYTES;
+	    } else {
+		buflen = MAX_TAPE_BLOCK_BYTES;
+	    }
+	}
+	buffer = newalloc(buffer, buflen);
+    }
+    if(isafile) {
+	bytes_read = fullread(tapedev, buffer, buflen);
+    } else {
+	bytes_read = tapefd_read(tapedev, buffer, buflen);
+	if(blocksize < 0 && bytes_read > 0 && bytes_read < buflen) {
+	    char *new_buffer;
+
+	    blocksize = bytes_read;
+	    new_buffer = alloc(blocksize);
+	    memcpy(new_buffer, buffer, bytes_read);
+	    amfree(buffer);
+	    buffer = new_buffer;
+	}
+    }
+    if(blocksize > 0) {
+	test_blocksize = 0;
+    }
+    return bytes_read;
+}
+
+
+void read_file_header(file, isafile)
 dumpfile_t *file;
-int buflen;
-int tapedev;
+int isafile;
 /*
  * Reads the first block of a tape file.
  */
 {
-    int bytes_read;
-    bytes_read = tapefd_read(tapedev, buffer, buflen);
+    bytes_read = get_block(isafile);
     if(bytes_read < 0) {
 	error("error reading file header: %s", strerror(errno));
-    }
-    else if(bytes_read < buflen) {
+    } else if(bytes_read < blocksize) {
 	if(bytes_read == 0) {
 	    fprintf(stderr, "%s: missing file header block\n", get_pname());
 	} else {
 	    fprintf(stderr, "%s: short file header block: %d byte%s\n",
 		    get_pname(), bytes_read, (bytes_read == 1) ? "" : "s");
 	}
-	file->type = F_TAPEEND;
-    }
-    else {
+	file->type = F_UNKNOWN;
+    } else {
 	parse_file_header(buffer, file, bytes_read);
     }
-    return(bytes_read);
+    return;
 }
 
 
-void restore(file, filename, tapedev, isafile)
+void restore(file, filename, isafile)
 dumpfile_t *file;
 char *filename;
-int tapedev;
 int isafile;
 /*
  * Restore the current file from tape.  Depending on the settings of
@@ -220,9 +256,15 @@ int isafile;
 
     out = dest;
 
-    /* if -r or -h, write the header before compress or uncompress pipe */
+    /*
+     * If -r or -h, write the header before compress or uncompress pipe.
+     * Only write DISK_BLOCK_BYTES, regardless of how much was read.
+     * This makes the output look like a holding disk image, and also
+     * makes it easier to remove the header (e.g. in amrecover) since
+     * it has a fixed size.
+     */
     if(rawflag || headerflag) {
-	int l, s;
+	int w;
 	char *cont_filename;
 
 	if(compflag && !file_is_compressed) {
@@ -244,11 +286,14 @@ int isafile;
 	/* remove CONT_FILENAME from header */
 	cont_filename = stralloc(file->cont_filename);
 	memset(file->cont_filename,'\0',sizeof(file->cont_filename));
-	write_header(buffer,file,buflen);
+	file->blocksize = DISK_BLOCK_BYTES;
+	build_header(buffer, file, bytes_read);
 
-	for(l = 0; l < buflen; l += s) {
-	    if((s = write(out, buffer + l, buflen - l)) < 0) {
+	if((w = fullwrite(out, buffer, DISK_BLOCK_BYTES)) != DISK_BLOCK_BYTES) {
+	    if(w < 0) {
 		error("write error: %s", strerror(errno));
+	    } else {
+		error("write error: %d instead of %d", w, DISK_BLOCK_BYTES);
 	    }
 	}
 	/* add CONT_FILENAME to header */
@@ -300,7 +345,7 @@ int isafile;
 	out = outpipe[1];
 	switch(compress_pid = fork()) {
 	case -1: 
-	    error("%s: could not fork for %s: %s", get_pname(), 
+	    error("could not fork for %s: %s",
 		  UNCOMPRESS_PATH, strerror(errno));
 	default:
 	    aclose(outpipe[0]);
@@ -332,58 +377,62 @@ int isafile;
     got_sigpipe = 0;
     wc = 0;
     do {
-	buflen = fill_buffer(tapedev, buffer, sizeof(buffer));
-	if(buflen == 0 && isafile) { /* switch to next file */
-	    int save_tapedev;
-
-	    save_tapedev = tapedev;
-	    close(tapedev);
-	    if(file->cont_filename[0] == '\0') break; /* no more file */
-	    if((tapedev = open(file->cont_filename,O_RDONLY)) == -1) {
-		error("can't open %s: %s",file->cont_filename,strerror(errno));
-	    }
-	    if(tapedev != save_tapedev) {
-		if(dup2(tapedev, save_tapedev) == -1) {
-		    error("can't dup2: %s",strerror(errno));
-		}
-		close(tapedev);
-		tapedev = save_tapedev;
-	    }
-	    buflen=read_file_header(buffer, file, sizeof(buffer), tapedev);
-/* should be validated */
-	    
-	    buflen = fill_buffer(tapedev, buffer, sizeof(buffer));
+	bytes_read = get_block(isafile);
+	if(bytes_read < 0) {
+	    error("read error: %s", strerror(errno));
 	}
-	if(buflen == 0 && !isafile) break; /* EOF */
-
-	if(buflen < 0) break;
-
-	for(l = 0; l < buflen; l += s) {
-	    if((s = write(out, buffer + l, buflen - l)) < 0) {
+	if(bytes_read == 0 && isafile) {
+	    /*
+	     * See if we need to switch to the next file.
+	     */
+	    if(file->cont_filename[0] == '\0') {
+		break;				/* no more files */
+	    }
+	    close(tapedev);
+	    if((tapedev = open(file->cont_filename, O_RDONLY)) == -1) {
+		char *cont_filename = strrchr(file->cont_filename,'/');
+		if(cont_filename) {
+		    cont_filename++;
+		    if((tapedev = open(cont_filename,O_RDONLY)) == -1) {
+			error("can't open %s: %s", file->cont_filename,
+			      strerror(errno));
+		    }
+		    else {
+			fprintf(stderr, "cannot open %s: %s\n",
+				file->cont_filename, strerror(errno));
+			fprintf(stderr, "using %s\n",
+				cont_filename);
+		    }
+		}
+		else {
+		    error("can't open %s: %s", file->cont_filename,
+			  strerror(errno));
+		}
+	    }
+	    read_file_header(file, isafile);
+	    if(file->type != F_DUMPFILE && file->type != F_CONT_DUMPFILE) {
+		fprintf(stderr, "unexpected header type: ");
+		print_header(stderr, file);
+		exit(2);
+	    }
+	    continue;
+	}
+	for(l = 0; l < bytes_read; l += s) {
+	    if((s = write(out, buffer + l, bytes_read - l)) < 0) {
 		if(got_sigpipe) {
 		    fprintf(stderr,"Error %d (%s) offset %d+%d, wrote %d\n",
-				   errno, strerror(errno), wc, buflen, rc);
+				   errno, strerror(errno), wc, bytes_read, rc);
 		    fprintf(stderr,  
 			    "%s: pipe reader has quit in middle of file.\n",
 			    get_pname());
-		    fprintf(stderr,
-			    "%s: skipping ahead to start of next file, please wait...\n",
-			    get_pname());
-		    if(!isafile) {
-			if(tapefd_fsf(tapedev, 1) == -1) {
-			    error("fast-forward: %s", strerror(errno));
-			}
-		    }
 		} else {
 		    perror("amrestore: write error");
 		}
 		exit(2);
 	    }
 	}
-	wc += buflen;
-    } while (buflen > 0);
-    if(buflen < 0)
-	error("read error: %s", strerror(errno));
+	wc += bytes_read;
+    } while (bytes_read > 0);
     if(pipeflag) {
 	if(out != dest) {
 	    aclose(out);
@@ -399,7 +448,7 @@ void usage()
  * Print usage message and terminate.
  */
 {
-    error("Usage: amrestore [-r|-c] [-p] [-h] tape-device|holdingfile [hostname [diskname [datestamp [hostname [diskname [datestamp ... ]]]]]]");
+    error("Usage: amrestore [-b blocksize] [-r|-c] [-p] [-h] [-f fileno] [-l label] tape-device|holdingfile [hostname [diskname [datestamp [hostname [diskname [datestamp ... ]]]]]]");
 }
 
 
@@ -428,8 +477,11 @@ char **argv;
     int found_match;
     int arg_state;
     amwait_t compress_status;
-    int tapedev;
     int fd;
+    int r = 0;
+    char *e;
+    char *err;
+    char *label = NULL;
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -449,13 +501,35 @@ char **argv;
     signal(SIGPIPE, handle_sigpipe);
 
     /* handle options */
-    while( (opt = getopt(argc, argv, "cCd:rpkh")) != -1) {
+    while( (opt = getopt(argc, argv, "b:cCd:rpkhf:l:")) != -1) {
 	switch(opt) {
+	case 'b':
+	    blocksize = strtol(optarg, &e, 10);
+	    if(*e == 'k' || *e == 'K') {
+		blocksize *= 1024;
+	    } else if(*e == 'm' || *e == 'M') {
+		blocksize *= 1024 * 1024;
+	    } else if(*e != '\0') {
+		error("invalid blocksize value \"%s\"", optarg);
+	    }
+	    if(blocksize < DISK_BLOCK_BYTES) {
+		error("minimum block size is %dk", DISK_BLOCK_BYTES / 1024);
+	    }
+	    break;
 	case 'c': compflag = 1; break;
 	case 'C': compflag = 1; compress_type = COMPRESS_BEST_OPT; break;
 	case 'r': rawflag = 1; break;
 	case 'p': pipeflag = 1; break;
 	case 'h': headerflag = 1; break;
+	case 'f':
+	    filefsf = strtol(optarg, &e, 10);
+	    if(*e != '\0') {
+		error("invalid fileno value \"%s\"", optarg);
+	    }
+	    break;
+	case 'l':
+	    label = stralloc(optarg);
+	    break;
 	default:
 	    usage();
 	}
@@ -468,13 +542,12 @@ char **argv;
     }
 
     if(optind >= argc) {
-	fprintf(stderr, "%s: Must specify tape-device or holdingfile\n", get_pname());
+	fprintf(stderr, "%s: Must specify tape-device or holdingfile\n",
+			get_pname());
 	usage();
     }
-    else tapename = argv[optind++];
 
-    if((tapedev = tape_open(tapename, 0)) < 0)
-	error("could not open tape %s: %s", tapename, strerror(errno));
+    tapename = argv[optind++];
 
 #define ARG_GET_HOST 0
 #define ARG_GET_DISK 1
@@ -531,16 +604,68 @@ char **argv;
 	match_list->next = NULL;
     }
 
-    if(stat(tapename,&stat_tape)!=0)
-	error("could not stat %s",tapename);
+    if(tape_stat(tapename,&stat_tape)!=0) {
+	error("could not stat %s: %s", tapename, strerror(errno));
+    }
     isafile=S_ISREG((stat_tape.st_mode));
-    file_number = 0;
-    buflen=read_file_header(buffer, &file, sizeof(buffer), tapedev);
 
-    if(file.type != F_TAPESTART && !isafile)
-	fprintf(stderr,
-    "%s: WARNING: not at start of tape, file numbers will be offset\n",
-    get_pname());
+    if(label) {
+	if(isafile) {
+	    fprintf(stderr,"%s: ignoring -l flag when restoring from a file.\n",
+		    get_pname());
+	}
+	else {
+	    if((err = tape_rewind(tapename)) != NULL) {
+		error("Could not rewind device '%s': %s", tapename, err);
+	    }
+	    tapedev = tape_open(tapename, 0);
+	    read_file_header(&file, isafile);
+	    if(file.type != F_TAPESTART) {
+		fprintf(stderr,"Not an amanda tape\n");
+		exit (1);
+	    }
+	    if(strcmp(label, file.name) != 0) {
+		fprintf(stderr,"Wrong label: '%s'\n", file.name);
+		exit (1);
+	    }
+	    tapefd_close(tapedev);
+	    if((err = tape_rewind(tapename)) != NULL) {
+		error("Could not rewind device '%s': %s", tapename, err);
+	    }
+	}
+    }
+    file_number = 0;
+    if(filefsf != -1) {
+	if(isafile) {
+	    fprintf(stderr,"%s: ignoring -f flag when restoring from a file.\n",
+		    get_pname());
+	}
+	else {
+	    if((err = tape_rewind(tapename)) != NULL) {
+		error("Could not rewind device '%s': %s", tapename, err);
+	    }
+	    if((err = tape_fsf(tapename,filefsf)) != NULL) {
+		error("Could not fsf device '%s': %s", tapename, err);
+	    }
+	    file_number = filefsf;
+	}
+    }
+
+    if(isafile) {
+	tapedev = open(tapename, 0);
+    } else {
+	tapedev = tape_open(tapename, 0);
+    }
+    if(tapedev < 0) {
+	error("could not open %s: %s", tapename, strerror(errno));
+    }
+
+    read_file_header(&file, isafile);
+
+    if(file.type != F_TAPESTART && !isafile && filefsf == -1) {
+	fprintf(stderr, "%s: WARNING: not at start of tape, file numbers will be offset\n",
+			get_pname());
+    }
 
     while(file.type == F_TAPESTART || file.type == F_DUMPFILE) {
 	amfree(filename);
@@ -552,44 +677,86 @@ char **argv;
 		break;
 	    }
 	}
+	fprintf(stderr, "%s: %3d: %s ",
+			get_pname(),
+			file_number,
+			found_match ? "restoring" : "skipping");
+	if(file.type != F_DUMPFILE) {
+	    print_header(stderr, &file);
+	} else {
+	    fprintf(stderr, "%s\n", filename);
+	}
 	if(found_match) {
-	    fprintf(stderr, "%s: %3d: restoring %s\n", 
-		    get_pname(), file_number, filename);
-	    restore(&file, filename, tapedev, isafile);
-	    if(pipeflag) break;
-	      /* GH: close and reopen the tape device, so that the
-		 read_file_header() below reads the next file on the
-		 tape and does not report: short block... */
-	    tapefd_close(tapedev);
-	    /* DB: wait for (un)compress, otherwise
-                   reopening the tape might fail */
-	    if (compress_pid > 0) {
-	      waitpid(compress_pid, &compress_status, 0);
-	      compress_pid = -1;
+	    restore(&file, filename, isafile);
+	    if(compress_pid > 0) {
+		waitpid(compress_pid, &compress_status, 0);
+		compress_pid = -1;
+	    }
+	    if(pipeflag) {
+		file_number++;			/* for the last message */
+		break;
 	    }
-	    if((tapedev = tape_open(tapename, 0)) < 0)
-		error("could not open tape %s: %s", tapename, strerror(errno));
 	}
-	else {
-	    fprintf(stderr, "%s: %3d: skipping ", get_pname(), file_number);
-	    if(file.type != F_DUMPFILE) print_header(stderr,&file);
-	    else fprintf(stderr, "%s\n", filename);
-	    if(tapefd_fsf(tapedev, 1) == -1)
-		error("fast-forward: %s", strerror(errno));
+	if(isafile) {
+	    break;
 	}
-	file_number += 1;
-	if(isafile)
-	    file.type = F_TAPEEND;
-	else {
-	    buflen=read_file_header(buffer, &file, sizeof(buffer), tapedev);
+	/*
+	 * Note that at this point we know we are working with a tape,
+	 * not a holding disk file, so we can call the tape functions
+	 * without checking.
+	 */
+	if(bytes_read == 0) {
+	    /*
+	     * If the last read got EOF, how to get to the next
+	     * file depends on how the tape device driver is acting.
+	     * If it is BSD-like, we do not really need to do anything.
+	     * If it is Sys-V-like, we need to either fsf or close/open.
+	     * The good news is, a close/open works in either case,
+	     * so that's what we do.
+	     */
+	    tapefd_close(tapedev);
+	    if((tapedev = tape_open(tapename, 0)) < 0) {
+		error("could not open %s: %s", tapename, strerror(errno));
+	    }
+	} else {
+	    /*
+	     * If the last read got something (even an error), we can
+	     * do an fsf to get to the next file.
+	     */
+	    if(tapefd_fsf(tapedev, 1) < 0) {
+		error("could not fsf %s: %s", tapename, strerror(errno));
+	    }
 	}
+	file_number++;
+	read_file_header(&file, isafile);
+    }
+    if(isafile) {
+	close(tapedev);
+    } else {
+	/*
+	 * See the notes above about advancing to the next file.
+	 */
+	if(bytes_read == 0) {
+	    tapefd_close(tapedev);
+	    if((tapedev = tape_open(tapename, 0)) < 0) {
+		error("could not open %s: %s", tapename, strerror(errno));
+	    }
+	} else {
+	    if(tapefd_fsf(tapedev, 1) < 0) {
+		error("could not fsf %s: %s", tapename, strerror(errno));
+	    }
+	}
+	tapefd_close(tapedev);
     }
-    tapefd_close(tapedev);
 
-    if(file.type == F_TAPEEND && !isafile) {
+    if((bytes_read <= 0 || file.type == F_TAPEEND) && !isafile) {
 	fprintf(stderr, "%s: %3d: reached ", get_pname(), file_number);
-	print_header(stderr,&file);
-	return 1;
+	if(bytes_read <= 0) {
+	    fprintf(stderr, "end of information\n");
+	} else {
+	    print_header(stderr,&file);
+	}
+	r = 1;
     }
-    return 0;
+    return r;
 }
