@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: amadmin.c,v 1.49.2.14 2001/02/28 14:21:54 martinea Exp $
+ * $Id: amadmin.c,v 1.49.2.13.2.3.2.15 2003/11/18 16:44:34 martinea Exp $
  *
  * controlling process for the Amanda backup system
  */
@@ -62,7 +62,7 @@ void due_one P((disk_t *dp));
 void find P((int argc, char **argv));
 void delete P((int argc, char **argv));
 void delete_one P((disk_t *dp));
-void balance P((void));
+void balance P((int argc, char **argv));
 void tape P((void));
 void bumpsize P((void));
 void diskloop P((int argc, char **argv, char *cmdname,
@@ -75,10 +75,6 @@ void import_db P((int argc, char **argv));
 void disklist P((int argc, char **argv));
 void disklist_one P((disk_t *dp));
 
-char *config_name = NULL;
-char *config_dir = NULL;
-
-static char *conffile = NULL;
 static char *conf_tapelist = NULL;
 
 int main(argc, argv)
@@ -90,6 +86,7 @@ char **argv;
     unsigned long malloc_hist_2, malloc_size_2;
     char *conf_diskfile;
     char *conf_infofile;
+    char *conffile;
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -119,7 +116,7 @@ char **argv;
     config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
     conffile = stralloc2(config_dir, CONFFILE_NAME);
     if(read_conffile(conffile)) {
-	error("could not find config file \"%s\"", conffile);
+	error("errors processing config file \"%s\"", conffile);
     }
     amfree(conffile);
     conf_diskfile = getconf_str(CNF_DISKFILE);
@@ -163,7 +160,7 @@ char **argv;
     else if(strcmp(argv[2],"due") == 0) due(argc, argv);
     else if(strcmp(argv[2],"find") == 0) find(argc, argv);
     else if(strcmp(argv[2],"delete") == 0) delete(argc, argv);
-    else if(strcmp(argv[2],"balance") == 0) balance();
+    else if(strcmp(argv[2],"balance") == 0) balance(argc, argv);
     else if(strcmp(argv[2],"tape") == 0) tape();
     else if(strcmp(argv[2],"bumpsize") == 0) bumpsize();
     else if(strcmp(argv[2],"import") == 0) import_db(argc, argv);
@@ -195,39 +192,39 @@ void usage P((void))
     fprintf(stderr, "    Valid <command>s are:\n");
     fprintf(stderr,"\tversion\t\t\t\t# Show version info.\n");
     fprintf(stderr,
-	    "\tforce <hostname> <disks> ...\t# Force level 0 at next run.\n");
+	    "\tforce [<hostname> [<disks>]* ]+\t# Force level 0 at next run.\n");
     fprintf(stderr,
-	    "\tunforce <hostname> <disks> ...\t# Clear force command.\n");
+	    "\tunforce [<hostname> [<disks>]* ]+\t# Clear force command.\n");
     fprintf(stderr,
-	    "\tforce-bump <hostname> <disks> ...\t# Force bump at next run.\n");
+	    "\tforce-bump [<hostname> [<disks>]* ]+\t# Force bump at next run.\n");
     fprintf(stderr,
-	    "\tforce-no-bump <hostname> <disks> ...\t# Force no-bump at next run.\n");
+	    "\tforce-no-bump [<hostname> [<disks>]* ]+\t# Force no-bump at next run.\n");
     fprintf(stderr,
-	    "\tunforce-bump <hostname> <disks> ...\t# Clear bump command.\n");
+	    "\tunforce-bump [<hostname> [<disks>]* ]+\t# Clear bump command.\n");
     fprintf(stderr,
 	    "\treuse <tapelabel> ...\t\t# re-use this tape.\n");
     fprintf(stderr,
 	    "\tno-reuse <tapelabel> ...\t# never re-use this tape.\n");
     fprintf(stderr,
-	    "\tfind <hostname> <disks> ...\t# Show which tapes these dumps are on.\n");
+	    "\tfind [<hostname> [<disks>]* ]*\t# Show which tapes these dumps are on.\n");
     fprintf(stderr,
-	    "\tdelete <hostname> <disks> ...\t# Delete from database.\n");
+	    "\tdelete [<hostname> [<disks>]* ]*\t# Delete from database.\n");
     fprintf(stderr,
-	    "\tinfo <hostname> <disks> ...\t# Show current info records.\n");
+	    "\tinfo [<hostname> [<disks>]* ]*\t# Show current info records.\n");
     fprintf(stderr,
-	    "\tdue <hostname> <disks> ...\t# Show due date.\n");
+	    "\tdue [<hostname> [<disks>]* ]*\t# Show due date.\n");
     fprintf(stderr,
-	    "\tbalance\t\t\t\t# Show nightly dump size balance.\n");
+	    "\tbalance [-days <num>]\t\t# Show nightly dump size balance.\n");
     fprintf(stderr,
 	    "\ttape\t\t\t\t# Show which tape is due next.\n");
     fprintf(stderr,
 	    "\tbumpsize\t\t\t# Show current bump thresholds.\n");
     fprintf(stderr,
-	    "\texport [<hostname> [<disks>]]\t# Export curinfo database to stdout.\n");
+	    "\texport [<hostname> [<disks>]* ]*\t# Export curinfo database to stdout.\n");
     fprintf(stderr,
 	    "\timport\t\t\t\t# Import curinfo database from stdin.\n");
     fprintf(stderr,
-  	    "\tdisklist [<hostname> [<disks> ...]]\t# Show disklist entries.\n");
+  	    "\tdisklist [<hostname> [<disks>]* ]*\t# Show disklist entries.\n");
 
     exit(1);
 }
@@ -237,7 +234,6 @@ void usage P((void))
 
 #define SECS_PER_DAY (24*60*60)
 time_t today;
-int runtapes, dumpcycle;
 
 char *seqdatestr(seq)
 int seq;
@@ -254,6 +250,8 @@ int seq;
     return str;
 }
 
+#undef days_diff
+#define days_diff(a, b)        (((b) - (a) + SECS_PER_DAY) / SECS_PER_DAY)
 
 /* when is next level 0 due? 0 = tonight, 1 = tommorrow, etc*/
 static int next_level0(dp, info)
@@ -262,7 +260,7 @@ info_t *info;
 {
     if(dp->strategy == DS_NOFULL)
 	return 1;	/* fake it */
-    else if(info->inf[0].date < (time_t)0)
+    if(info->inf[0].date < (time_t)0)
 	return 0;	/* new disk */
     else
 	return dp->dumpcycle - days_diff(info->inf[0].date, today);
@@ -293,9 +291,8 @@ static void check_dumpuser()
 	/* NOTREACHED */
     }
     if (uid_me != uid_dumpuser) {
-	fprintf(stderr, "WARNING: running as user \"%s\" instead of \"%s\".\n",
-	        pw->pw_name, dumpuser);
-	fprintf(stderr, "WARNING: run \"amcheck -st %s\"\n", config_name);
+	error("ERROR: running as user \"%s\" instead of \"%s\"",
+	      pw->pw_name, dumpuser);
     }
     been_here = 1;
     return;
@@ -309,41 +306,26 @@ char **argv;
 char *cmdname;
 void (*func) P((disk_t *dp));
 {
-    host_t *hp;
     disk_t *dp;
-    char *diskname;
-    int count;
+    int count = 0;
 
     if(argc < 4) {
-	fprintf(stderr,"%s: expecting \"%s <hostname> {<disks> ...}\"\n",
-		argv[0], cmdname);
+	fprintf(stderr,"%s: expecting \"%s [<hostname> [<disks>]* ]+\"\n",
+		get_pname(), cmdname);
 	usage();
     }
 
-    if((hp = lookup_host(argv[3])) == NULL) {
-	fprintf(stderr, "%s: host %s not in current disklist database.\n",
-		argv[0], argv[3]);
-	exit(1);
-    }
-    if(argc < 5) {
-	for(dp = hp->disks; dp != NULL; dp = dp->hostnext)
+    match_disklist(diskqp, argc-3, argv+3);
+
+    for(dp = diskqp->head; dp != NULL; dp = dp->next) {
+	if(dp->todo) {
+	    count++;
 	    func(dp);
-    }
-    else {
-	for(argc -= 4, argv += 4; argc; argc--, argv++) {
-	    count = 0;
-	    diskname = *argv;
-	    for(dp = hp->disks; dp != NULL; dp = dp->hostnext) {
-		if(match(diskname, dp->name)) {
-		    count++;
-		    func(dp);
-		}
-	    }
-	    if(count == 0)
-		fprintf(stderr, "%s: host %s has no disks that match \"%s\"\n",
-			argv[0], hp->hostname, diskname);
 	}
     }
+    if(count==0) {
+	fprintf(stderr,"%s: no disk matched\n",get_pname());
+    }
 }
 
 /* ----------------------------------------------- */
@@ -557,7 +539,7 @@ char **argv;
 
     if(argc < 4) {
 	fprintf(stderr,"%s: expecting \"reuse <tapelabel> ...\"\n",
-		argv[0]);
+		get_pname());
 	usage();
     }
 
@@ -593,7 +575,7 @@ char **argv;
 
     if(argc < 4) {
 	fprintf(stderr,"%s: expecting \"no-reuse <tapelabel> ...\"\n",
-		argv[0]);
+		get_pname());
 	usage();
     }
 
@@ -666,7 +648,7 @@ void info_one(dp)
 disk_t *dp;
 {
     info_t info;
-    int lev, lev0date;
+    int lev;
     struct tm *tm;
     stats_t *sp;
 
@@ -689,7 +671,6 @@ disk_t *dp;
 	   info.incr.comp[0]*100, info.incr.comp[1]*100, info.incr.comp[2]*100);
 
     printf("  Dumps: lev datestmp  tape             file   origK   compK secs\n");
-    lev0date = info.inf[0].date;
     for(lev = 0, sp = &info.inf[0]; lev < 9; lev++, sp++) {
 	if(sp->date < (time_t)0 && sp->label[0] == '\0') continue;
 	tm = localtime(&sp->date);
@@ -762,8 +743,8 @@ char **argv;
 
 void tape()
 {
-    tape_t *tp;
-    int runtapes,i;
+    tape_t *tp, *lasttp;
+    int runtapes, i;
 
     runtapes = getconf_int(CNF_RUNTAPES);
     tp = lookup_last_reusable_tape(0);
@@ -777,48 +758,85 @@ void tape()
 	
 	tp = lookup_last_reusable_tape(i + 1);
     }
+    lasttp = lookup_tapepos(lookup_nb_tape());
+    i = runtapes;
+    if(lasttp && i > 0 && lasttp->datestamp == 0) {
+	int c = 0;
+	while(lasttp && i > 0 && lasttp->datestamp == 0) {
+	    c++;
+	    lasttp = lasttp->prev;
+	    i--;
+	}
+	lasttp = lookup_tapepos(lookup_nb_tape());
+	i = runtapes;
+	if(c == 1) {
+	    printf("The next new tape already labelled is: %s.\n",
+		   lasttp->label);
+	}
+	else {
+	    printf("The next %d new tapes already labelled are: %s", c,
+		   lasttp->label);
+	    lasttp = lasttp->prev;
+	    c--;
+	    while(lasttp && c > 0 && lasttp->datestamp == 0) {
+		printf(", %s", lasttp->label);
+		lasttp = lasttp->prev;
+		c--;
+	    }
+	    printf(".\n");
+	}
+    }
 }
 
 /* ----------------------------------------------- */
 
-void balance()
+void balance(argc, argv)
+int argc;
+char **argv;
 {
     disk_t *dp;
     struct balance_stats {
 	int disks;
 	long origsize, outsize;
     } *sp;
-    int seq, max_seq, total, balanced, runs_per_cycle, overdue, max_overdue;
-    int runspercycle;
+    int conf_runspercycle, conf_dumpcycle;
+    int seq, runs_per_cycle, overdue, max_overdue;
+    int later, total, balance, distinct;
+    float fseq, disk_dumpcycle;
     info_t info;
+    long int total_balanced, balanced;
 
-    total = getconf_int(CNF_TAPECYCLE);
-    max_seq = getconf_int(CNF_DUMPCYCLE)-1;	/* print at least this many */
     time(&today);
-    runtapes = getconf_int(CNF_RUNTAPES);
-    dumpcycle = getconf_int(CNF_DUMPCYCLE);
-    runspercycle = getconf_int(CNF_RUNSPERCYCLE);
+    conf_dumpcycle = getconf_int(CNF_DUMPCYCLE);
+    conf_runspercycle = getconf_int(CNF_RUNSPERCYCLE);
+    later = conf_dumpcycle;
     overdue = 0;
     max_overdue = 0;
 
-    if(runspercycle == 0) {
-	runs_per_cycle = dumpcycle;
-    } else if(runspercycle == -1 ) {
+    if(argc > 4 && strcmp(argv[3],"--days") == 0) {
+	later = atoi(argv[4]);
+	if(later < 0) later = conf_dumpcycle;
+    }
+
+    if(conf_runspercycle == 0) {
+	runs_per_cycle = conf_dumpcycle;
+    } else if(conf_runspercycle == -1 ) {
 	runs_per_cycle = guess_runs_from_tapelist();
     } else
-	runs_per_cycle = runspercycle;
+	runs_per_cycle = conf_runspercycle;
 
     if (runs_per_cycle <= 0) {
 	runs_per_cycle = 1;
     }
 
-    if(dumpcycle > total)
-	total = dumpcycle;
+    total = later + 1;
+    balance = later + 2;
+    distinct = later + 3;
 
     sp = (struct balance_stats *)
-	alloc(sizeof(struct balance_stats) * (total+1));
+	alloc(sizeof(struct balance_stats) * (distinct+1));
 
-    for(seq=0; seq <= total; seq++)
+    for(seq=0; seq <= distinct; seq++)
 	sp[seq].disks = sp[seq].origsize = sp[seq].outsize = 0;
 
     for(dp = diskqp->head; dp != NULL; dp = dp->next) {
@@ -826,29 +844,57 @@ void balance()
 	    printf("new disk %s:%s ignored.\n", dp->host->hostname, dp->name);
 	    continue;
 	}
+	if (dp->strategy == DS_NOFULL) {
+	    continue;
+	}
+	sp[distinct].disks++;
+	sp[distinct].origsize += info.inf[0].size;
+	sp[distinct].outsize += info.inf[0].csize;
+
+	sp[balance].disks++;
+	if(dp->dumpcycle == 0) {
+	    sp[balance].origsize += info.inf[0].size * runs_per_cycle;
+	    sp[balance].outsize += info.inf[0].csize * runs_per_cycle;
+	}
+	else {
+	    sp[balance].origsize += info.inf[0].size *
+				    (conf_dumpcycle / dp->dumpcycle);
+	    sp[balance].outsize += info.inf[0].csize *
+				   (conf_dumpcycle / dp->dumpcycle);
+	}
+
+	disk_dumpcycle = dp->dumpcycle;
+	if(dp->dumpcycle <= 0)
+	    disk_dumpcycle = ((float)conf_dumpcycle) / ((float)runs_per_cycle);
+
 	seq = next_level0(dp, &info);
-	if(seq < 0) {
-	    overdue++;
-	    if (-seq > max_overdue)
-		max_overdue = -seq;
-	    seq = 0;
-	}
-	if(seq >= total) {
-	    if(seq > total) /* next_level0() can return dumpcycle if run */
-			    /* immediately  after amdump */
-		printf("bogus seq number %d for %s:%s\n", seq,
-			dp->host->hostname, dp->name);
-	    seq = total-1;
-	}
-	if(seq > max_seq) max_seq = seq;
+	fseq = seq + 0.0001;
+	do {
+	    if(seq < 0) {
+		overdue++;
+		if (-seq > max_overdue)
+		    max_overdue = -seq;
+		seq = 0;
+		fseq = seq + 0.0001;
+	    }
+	    if(seq > later) {
+	       	seq = later;
+	    }
+	    
+	    sp[seq].disks++;
+	    sp[seq].origsize += info.inf[0].size;
+	    sp[seq].outsize += info.inf[0].csize;
 
-	sp[seq].disks++;
-	sp[seq].origsize += info.inf[0].size;
-	sp[seq].outsize += info.inf[0].csize;
-
-	sp[total].disks++;
-	sp[total].origsize += info.inf[0].size;
-	sp[total].outsize += info.inf[0].csize;
+	    if(seq < later) {
+		sp[total].disks++;
+		sp[total].origsize += info.inf[0].size;
+		sp[total].outsize += info.inf[0].csize;
+	    }
+	    
+	    /* See, if there's another run in this dumpcycle */
+	    fseq += disk_dumpcycle;
+	    seq = fseq;
+	} while (seq < later);
     }
 
     if(sp[total].outsize == 0) {
@@ -857,21 +903,46 @@ void balance()
 	return;
     }
 
-    balanced = sp[total].outsize / runs_per_cycle;
+    balanced = sp[balance].outsize / runs_per_cycle;
+    if(conf_dumpcycle == later) {
+	total_balanced = sp[total].outsize / runs_per_cycle;
+    }
+    else {
+	total_balanced = 1024*(((sp[total].outsize/1024) * conf_dumpcycle)
+			    / (runs_per_cycle * later));
+    }
 
-    printf("\n due-date  #fs   orig KB    out KB  balance\n");
-    printf("-------------------------------------------\n");
-    for(seq = 0; seq <= max_seq; seq++) {
-	printf("%-9.9s  %3d %9ld %9ld ",
+    printf("\n due-date  #fs    orig KB     out KB   balance\n");
+    printf("----------------------------------------------\n");
+    for(seq = 0; seq < later; seq++) {
+	printf("%-9.9s  %3d %10ld %10ld ",
 	       seqdatestr(seq), sp[seq].disks,
 	       sp[seq].origsize, sp[seq].outsize);
-	if(!sp[seq].outsize) printf("    --- \n");
-	else printf("%+7.1f%%\n",
+	if(!sp[seq].outsize) printf("     --- \n");
+	else printf("%+8.1f%%\n",
 		    (sp[seq].outsize-balanced)*100.0/(double)balanced);
     }
-    printf("-------------------------------------------\n");
-    printf("TOTAL      %3d %9ld %9ld %8d", sp[total].disks,
-	   sp[total].origsize, sp[total].outsize, balanced);
+    if(sp[later].disks != 0) {
+	printf("later      %3d %10ld %10ld ",
+	       sp[later].disks,
+	       sp[later].origsize, sp[later].outsize);
+	if(!sp[later].outsize) printf("     --- \n");
+	else printf("%+8.1f%%\n",
+		    (sp[later].outsize-balanced)*100.0/(double)balanced);
+    }
+    printf("----------------------------------------------\n");
+    printf("TOTAL      %3d %10ld %10ld %9ld\n", sp[total].disks,
+	   sp[total].origsize, sp[total].outsize, total_balanced);
+    if (sp[balance].origsize != sp[total].origsize ||
+        sp[balance].outsize != sp[total].outsize ||
+	balanced != total_balanced) {
+	printf("BALANCED       %10ld %10ld %9ld\n",
+	       sp[balance].origsize, sp[balance].outsize, balanced);
+    }
+    if (sp[distinct].disks != sp[total].disks) {
+	printf("DISTINCT   %3d %10ld %10ld\n", sp[distinct].disks,
+	       sp[distinct].origsize, sp[distinct].outsize);
+    }
     printf("  (estimated %d run%s per dumpcycle)\n",
 	   runs_per_cycle, (runs_per_cycle == 1) ? "" : "s");
     if (overdue) {
@@ -889,18 +960,13 @@ void find(argc, argv)
 int argc;
 char **argv;
 {
-    host_t *hp;
     int start_argc;
     char *sort_order = NULL;
     find_result_t *output_find;
-    char *find_hostname;
-    char **find_diskstrs;
-    int  find_ndisks;
-    int  find_nhosts;
 
     if(argc < 3) {
 	fprintf(stderr,
-		"%s: expecting \"find [--sort <hkdlb>] [hostname [<disk> ...]]\"\n",
+		"%s: expecting \"find [--sort <hkdlb>] [hostname [<disk>]]*\"\n",
 		get_pname());
 	usage();
     }
@@ -936,23 +1002,15 @@ char **argv;
     } else {
 	start_argc=4;
     }
-    if(argc < start_argc) {
-	find_nhosts = 0;
-	find_hostname = NULL;
-	find_ndisks = 0;
-	find_diskstrs = NULL;
+    match_disklist(diskqp, argc-(start_argc-1), argv+(start_argc-1));
+    output_find = find_dump(1, diskqp);
+
+    if(argc-(start_argc-1) > 0) {
+	free_find_result(&output_find);
+	match_disklist(diskqp, argc-(start_argc-1), argv+(start_argc-1));
+	output_find = find_dump(0, NULL);
     }
-    else {
-	find_nhosts = 1;
-        find_hostname = argv[start_argc-1];
-        if((hp = lookup_host(find_hostname)) == NULL)
-	    printf("Warning: host %s not in disklist.\n", find_hostname);
-        else
-	    find_hostname = hp->hostname;
-        find_ndisks = argc - start_argc;
-        find_diskstrs = &argv[start_argc];
-    }
-    output_find = find_dump(find_hostname, find_ndisks, find_diskstrs);
+
     sort_find_result(sort_order, &output_find);
     print_find_result(output_find);
     free_find_result(&output_find);
@@ -1078,7 +1136,7 @@ int argc;
 char **argv;
 {
     int vers_maj, vers_min, vers_patch, newer;
-    char *org, *vers_comment;
+    char *org;
     char *line = NULL;
     char *hdr;
     char *s;
@@ -1124,7 +1182,6 @@ char **argv;
     if(ch == '\0') {
 	goto bad_header;
     }
-    vers_comment = s - 1;			/* note: right next to patch */
     skip_non_whitespace(s, ch);
     s[-1] = '\0';
 
@@ -1420,6 +1477,7 @@ disk_t *dp;
 {
     host_t *hp;
     interface_t *ip;
+    sle_t *excl;
 
     hp = dp->host;
     ip = hp->netif;
@@ -1431,13 +1489,43 @@ disk_t *dp;
 	   ip->name[0] ? ip->name : "default");
 
     printf("    disk %s:\n", dp->name);
+    if(dp->device) printf("        device %s\n", dp->device);
 
     printf("        program \"%s\"\n", dp->program);
-    if(dp->exclude != (char *)0)
-	printf("        exclude %s\"%s\"\n", dp->exclude_list? "list ":"", dp->exclude);
+    if(dp->exclude_file != NULL && dp->exclude_file->nb_element > 0) {
+	printf("        exclude file");
+	for(excl = dp->exclude_file->first; excl != NULL; excl = excl->next) {
+	    printf(" \"%s\"", excl->name);
+	}
+	printf("\n");
+    }
+    if(dp->exclude_list != NULL && dp->exclude_list->nb_element > 0) {
+	printf("        exclude list");
+	if(dp->exclude_optional) printf(" optional");
+	for(excl = dp->exclude_list->first; excl != NULL; excl = excl->next) {
+	    printf(" \"%s\"", excl->name);
+	}
+	printf("\n");
+    }
+    if(dp->include_file != NULL && dp->include_file->nb_element > 0) {
+	printf("        include file");
+	for(excl = dp->include_file->first; excl != NULL; excl = excl->next) {
+	    printf(" \"%s\"", excl->name);
+	}
+	printf("\n");
+    }
+    if(dp->include_list != NULL && dp->include_list->nb_element > 0) {
+	printf("        include list");
+	if(dp->include_optional) printf(" optional");
+	for(excl = dp->include_list->first; excl != NULL; excl = excl->next) {
+	    printf(" \"%s\"", excl->name);
+	}
+	printf("\n");
+    }
     printf("        priority %ld\n", dp->priority);
     printf("        dumpcycle %ld\n", dp->dumpcycle);
     printf("        maxdumps %d\n", dp->maxdumps);
+    printf("        maxpromoteday %d\n", dp->maxpromoteday);
 
     printf("        strategy ");
     switch(dp->strategy) {
