@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amcheck.c,v 1.50.2.20 2001/03/15 02:34:57 jrjackson Exp $
+ * $Id: amcheck.c,v 1.50.2.19.2.7.2.20 2003/11/25 12:21:07 martinea Exp $
  *
  * checks for common problems in server and clients
  */
@@ -39,6 +39,10 @@
 #include "clock.h"
 #include "version.h"
 #include "amindex.h"
+#include "token.h"
+#include "util.h"
+#include "pipespawn.h"
+#include "amfeatures.h"
 
 /*
  * If we don't have the new-style wait access functions, use our own,
@@ -59,11 +63,11 @@
 static int conf_ctimeout;
 static int overwrite;
 dgram_t *msg = NULL;
-char *config_name = NULL;
-char *config_dir = NULL;
 
 static disklist_t *origqp;
 
+static uid_t uid_dumpuser;
+
 /* local functions */
 
 void usage P((void));
@@ -78,17 +82,20 @@ int test_server_pgm P((FILE *outf, char *dir, char *pgm,
 
 void usage()
 {
-    error("Usage: amcheck%s [-M <username>] [-mwsclt] <conf>", versionsuffix());
+    error("Usage: amcheck%s [-M <username>] [-mawsclt] <conf> [host [disk]* ]*", versionsuffix());
 }
 
 static unsigned long malloc_hist_1, malloc_size_1;
 static unsigned long malloc_hist_2, malloc_size_2;
 
+static am_feature_t *our_features = NULL;
+static char *our_feature_string = NULL;
+
 int main(argc, argv)
 int argc;
 char **argv;
 {
-    char buffer[BUFFER_SIZE], *cmd = NULL;
+    char buffer[BUFFER_SIZE];
     char *version_string;
     char *mainfname = NULL;
     char pid_str[NUM_STR_SIZE];
@@ -104,10 +111,14 @@ char **argv;
     char *mailto = NULL;
     extern char *optarg;
     int mailout;
+    int alwaysmail;
     char *tempfname = NULL;
     char *conffile;
     char *conf_diskfile;
-    
+    char *dumpuser;
+    struct passwd *pw;
+    uid_t uid_me;
+
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
 	 * Make sure nobody spoofs us with a lot of extra open files
@@ -129,6 +140,9 @@ char **argv;
 
     erroutput_type = ERR_INTERACTIVE;
 
+    our_features = am_init_feature_set();
+    our_feature_string = am_feature_to_string(our_features);
+
     /* set up dgram port first thing */
 
     msg = dgram_alloc();
@@ -141,8 +155,9 @@ char **argv;
 	setgid(getgid());
 	setuid(getuid());
     }
+    uid_me = getuid();
 
-    mailout = overwrite = 0;
+    alwaysmail = mailout = overwrite = 0;
     do_localchk = do_tapechk = do_clientchk = 0;
     chk_flag = 0;
     server_probs = client_probs = 0;
@@ -150,9 +165,9 @@ char **argv;
 
     /* process arguments */
 
-    while((opt = getopt(argc, argv, "M:mwsclt")) != EOF) {
+    while((opt = getopt(argc, argv, "M:mawsclt")) != EOF) {
 	switch(opt) {
-	case 'M':	mailto=optarg;
+	case 'M':	mailto=stralloc(optarg);
 	case 'm':	
 #ifdef MAILER
 			mailout = 1;
@@ -160,6 +175,16 @@ char **argv;
 			printf("You can't use -%c because configure didn't find a mailer.\n",
 				opt);
 			exit(1);
+#endif
+			break;
+	case 'a':	
+#ifdef MAILER
+			mailout = 1;
+			alwaysmail = 1;
+#else
+			printf("You can't use -%c because configure didn't find a mailer.\n",
+				opt);
+			exit(1);
 #endif
 			break;
 	case 's':	do_localchk = 1; do_tapechk = 1;
@@ -187,14 +212,14 @@ char **argv;
 	do_localchk = do_clientchk = do_tapechk = 1;
     }
 
-    if(argc != 1) usage();
+    if(argc < 1) usage();
 
     config_name = stralloc(*argv);
 
     config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
     conffile = stralloc2(config_dir, CONFFILE_NAME);
     if(read_conffile(conffile)) {
-	error("could not find config file \"%s\"", conffile);
+	error("errors processing config file \"%s\"", conffile);
     }
     amfree(conffile);
     conf_ctimeout = getconf_int(CNF_CTIMEOUT);
@@ -207,8 +232,26 @@ char **argv;
     if((origqp = read_diskfile(conf_diskfile)) == NULL) {
 	error("could not load disklist %s", conf_diskfile);
     }
+    match_disklist(origqp, argc-1, argv+1);
     amfree(conf_diskfile);
 
+    /*
+     * Make sure we are running as the dump user.
+     */
+    dumpuser = getconf_str(CNF_DUMPUSER);
+    if ((pw = getpwnam(dumpuser)) == NULL) {
+	error("cannot look up dump user \"%s\"", dumpuser);
+    }
+    uid_dumpuser = pw->pw_uid;
+    if ((pw = getpwuid(uid_me)) == NULL) {
+	error("cannot look up my own uid (%ld)", (long)uid_me);
+    }
+    if (uid_me != uid_dumpuser) {
+	error("running as user \"%s\" instead of \"%s\"",
+	      pw->pw_name,
+	      dumpuser);
+    }
+
     /*
      * If both server and client side checks are being done, the server
      * check output goes to the main output, while the client check output
@@ -220,17 +263,19 @@ char **argv;
     if(do_clientchk && (do_localchk || do_tapechk)) {
 	/* we need the temp file */
 	tempfname = vstralloc(AMANDA_TMPDIR, "/amcheck.temp.", pid_str, NULL);
-	malloc_mark(tempfname);
 	if((tempfd = open(tempfname, O_RDWR|O_CREAT|O_TRUNC, 0600)) == -1)
 	    error("could not open %s: %s", tempfname, strerror(errno));
+	unlink(tempfname);			/* so it goes away on close */
+	amfree(tempfname);
     }
 
     if(mailout) {
 	/* the main fd is a file too */
 	mainfname = vstralloc(AMANDA_TMPDIR, "/amcheck.main.", pid_str, NULL);
-	malloc_mark(mainfname);
 	if((mainfd = open(mainfname, O_RDWR|O_CREAT|O_TRUNC, 0600)) == -1)
 	    error("could not open %s: %s", mainfname, strerror(errno));
+	unlink(mainfname);			/* so it goes away on close */
+	amfree(mainfname);
     }
     else
 	/* just use stdout */
@@ -297,7 +342,6 @@ char **argv;
 	if(size < 0)
 	    error("read temp file: %s", strerror(errno));
 	aclose(tempfd);
-	unlink(tempfname);
     }
 
     version_string = vstralloc("\n",
@@ -311,6 +355,9 @@ char **argv;
     amfree(version_string);
     amfree(config_dir);
     amfree(config_name);
+    amfree(our_feature_string);
+    am_release_feature_set(our_features);
+    our_features = NULL;
 
     malloc_size_2 = malloc_inuse(&malloc_hist_2);
 
@@ -321,25 +368,121 @@ char **argv;
     /* send mail if requested, but only if there were problems */
 #ifdef MAILER
 
-    if((server_probs || client_probs) && mailout) {
-	fflush(stdout);
-	if(close(mainfd) == -1)
-	    error("close main file: %s", strerror(errno));
-	mainfd = -1;
+#define	MAILTO_LIMIT	10
 
-	cmd = vstralloc(MAILER,
-			" -s ", "\"",
-			  getconf_str(CNF_ORG),
-			  " AMANDA PROBLEM: FIX BEFORE RUN, IF POSSIBLE\"",
-			" ", (mailto ? mailto : getconf_str(CNF_MAILTO)),
-			" < ", mainfname,
-			NULL);
-	if(system(cmd) != 0)
-	    error("mail command failed: %s", cmd);
-	amfree(cmd);
-    }
-    if(mailout) {
-	unlink(mainfname);
+    if((server_probs || client_probs || alwaysmail) && mailout) {
+	int mailfd;
+	int nullfd;
+	int errfd;
+	FILE *ferr;
+	char *subject;
+	char **a;
+	amwait_t retstat;
+	pid_t mailpid;
+	pid_t wpid;
+	int r;
+	int w;
+	char *err = NULL;
+	char *extra_info = NULL;
+	char *line = NULL;
+	int ret;
+	int rc;
+	int sig;
+	char number[NUM_STR_SIZE];
+
+	fflush(stdout);
+	if(lseek(mainfd, (off_t)0, SEEK_SET) == -1) {
+	    error("lseek main file: %s", strerror(errno));
+	}
+	if(alwaysmail && !(server_probs || client_probs)) {
+	    subject = stralloc2(getconf_str(CNF_ORG),
+			    " AMCHECK REPORT: NO PROBLEMS FOUND");
+	} else {
+	    subject = stralloc2(getconf_str(CNF_ORG),
+			    " AMANDA PROBLEM: FIX BEFORE RUN, IF POSSIBLE");
+	}
+	/*
+	 * Variable arg lists are hard to deal with when we do not know
+	 * ourself how many args are involved.  Split the address list
+	 * and hope there are not more than 9 entries.
+	 *
+	 * Remember that split() returns the original input string in
+	 * argv[0], so we have to skip over that.
+	 */
+	a = (char **) alloc((MAILTO_LIMIT + 1) * sizeof(char *));
+	memset(a, 0, (MAILTO_LIMIT + 1) * sizeof(char *));
+	if(mailto) {
+	    a[1] = mailto;
+	    a[2] = NULL;
+	} else {
+	    n = split(getconf_str(CNF_MAILTO), a, MAILTO_LIMIT, " ");
+	    a[n + 1] = NULL;
+	}
+	if((nullfd = open("/dev/null", O_RDWR)) < 0) {
+	    error("nullfd: /dev/null: %s", strerror(errno));
+	}
+	mailpid = pipespawn(MAILER, STDIN_PIPE | STDERR_PIPE,
+			    &mailfd, &nullfd, &errfd,
+			    MAILER,
+			    "-s", subject,
+			          a[1], a[2], a[3], a[4],
+			    a[5], a[6], a[7], a[8], a[9],
+			    NULL);
+	amfree(subject);
+	/*
+	 * There is the potential for a deadlock here since we are writing
+	 * to the process and then reading stderr, but in the normal case,
+	 * nothing should be coming back to us, and hopefully in error
+	 * cases, the pipe will break and we will exit out of the loop.
+	 */
+	signal(SIGPIPE, SIG_IGN);
+	while((r = fullread(mainfd, buffer, sizeof(buffer))) > 0) {
+	    if((w = fullwrite(mailfd, buffer, r)) != r) {
+		if(w < 0 && errno == EPIPE) {
+		    strappend(extra_info, "EPIPE writing to mail process\n");
+		    break;
+		} else if(w < 0) {
+		    error("mailfd write: %s", strerror(errno));
+		} else {
+		    error("mailfd write: wrote %d instead of %d", w, r);
+		}
+	    }
+	}
+	aclose(mailfd);
+	ferr = fdopen(errfd, "r");
+	for(; (line = agets(ferr)) != NULL; free(line)) {
+	    strappend(extra_info, line);
+	    strappend(extra_info, "\n");
+	}
+	afclose(ferr);
+	errfd = -1;
+	rc = 0;
+	while ((wpid = wait(&retstat)) != -1) {
+	    if (WIFSIGNALED(retstat)) {
+		    ret = 0;
+		    rc = sig = WTERMSIG(retstat);
+	    } else {
+		    sig = 0;
+		    rc = ret = WEXITSTATUS(retstat);
+	    }
+	    if (rc != 0) {
+		    if (ret == 0) {
+			strappend(err, "got signal ");
+			ret = sig;
+		    } else {
+			strappend(err, "returned ");
+		    }
+		    ap_snprintf(number, sizeof(number), "%d", ret);
+		    strappend(err, number);
+	    }
+	}
+	if (rc != 0) {
+	    if(extra_info) {
+		fputs(extra_info, stderr);
+		amfree(extra_info);
+	    }
+	    error("error running mailer %s: %s", MAILER, err);
+	}
     }
 #endif
     dbclose();
@@ -392,7 +535,9 @@ char *device;
 	    /* got an amanda tape */
 	    fprintf(errf, "%s: slot %s: date %-8s label %s",
 		    get_pname(), slotstr, datestamp, label);
-	    if(searchlabel != NULL && strcmp(label, searchlabel) == 0) {
+	    if(searchlabel != NULL
+	       && (strcmp(label, FAKE_LABEL) == 0
+		   || strcmp(label, searchlabel) == 0)) {
 		/* it's the one we are looking for, stop here */
 		fprintf(errf, " (exact label match)\n");
 		found_device = newstralloc(found_device, device);
@@ -405,8 +550,19 @@ char *device;
 		/* not an exact label match, but a labelstr match */
 		/* check against tape list */
 		tp = lookup_tapelabel(label);
-		if(tp != NULL && !reusable_tape(tp))
+		if(tp == NULL)
+		    fprintf(errf, " (Not in tapelist)\n");
+		else if(!reusable_tape(tp))
 		    fprintf(errf, " (active tape)\n");
+		else if(got_match == 0 && tp->datestamp == 0) {
+		    got_match = 1;
+		    first_match = newstralloc(first_match, slotstr);
+		    first_match_label = newstralloc(first_match_label, label);
+		    fprintf(errf, " (new tape)\n");
+		    found = 3;
+		    found_device = newstralloc(found_device, device);
+		    return 1;
+		}
 		else if(got_match)
 		    fprintf(errf, " (labelstr match)\n");
 		else {
@@ -440,7 +596,7 @@ char *taper_scan()
 
     changer_find(scan_init, taperscan_slot, searchlabel);
 
-    if(found == 2)
+    if(found == 2 || found == 3)
 	searchlabel = first_match_label;
     else if(!found && got_match) {
 	searchlabel = first_match_label;
@@ -506,11 +662,9 @@ int start_server_check(fd, do_localchk, do_tapechk)
     FILE *outf;
     holdingdisk_t *hdp;
     int pid;
-    int tapebad = 0, disklow = 0, logbad = 0;
+    int confbad = 0, tapebad = 0, disklow = 0, logbad = 0;
     int userbad = 0, infobad = 0, indexbad = 0, pgmbad = 0;
     int testtape = do_tapechk;
-    uid_t uid_me = getuid();
-    uid_t uid_dumpuser = uid_me;
 
     switch(pid = fork()) {
     case -1: error("could not fork server check: %s", strerror(errno));
@@ -536,27 +690,39 @@ int start_server_check(fd, do_localchk, do_tapechk)
     fprintf(outf, "-----------------------------\n");
 
     /*
-     * Make sure we are running as the dump user.
+     * Check various server side config file settings.
      */
-    {
-	char *dumpuser = getconf_str(CNF_DUMPUSER);
-	struct passwd *pw;
+    if(do_localchk) {
+	char *ColumnSpec;
+	char *errstr = NULL;
+	tapetype_t *tp;
+	char *lbl_templ;
 
-	if ((pw = getpwnam(dumpuser)) == NULL) {
-	    fprintf(outf, "ERROR: cannot look up dump user \"%s\"\n", dumpuser);
-	    userbad = 1;
-	} else {
-	    uid_dumpuser = pw->pw_uid;
+	ColumnSpec = getconf_str(CNF_COLUMNSPEC);
+	if(SetColumDataFromString(ColumnData, ColumnSpec, &errstr) < 0) {
+	    fprintf(outf, "ERROR: %s\n", errstr);
+	    amfree(errstr);
+	    confbad = 1;
 	}
-	if ((pw = getpwuid(uid_me)) == NULL) {
-	    fprintf(outf, "ERROR: cannot look up my own uid (%ld)\n",
-		    (long)uid_me);
-	    userbad = 1;
-	}
-	if (uid_me != uid_dumpuser) {
-	    fprintf(outf, "ERROR: running as user \"%s\" instead of \"%s\"\n",
-		    pw->pw_name, dumpuser);
-	    userbad = 1;
+	tp = lookup_tapetype(getconf_str(CNF_TAPETYPE));
+	lbl_templ = tp->lbl_templ;
+	if(strcmp(lbl_templ, "") != 0) {
+	    if(strchr(lbl_templ, '/') == NULL) {
+		lbl_templ = stralloc2(config_dir, lbl_templ);
+	    } else {
+		lbl_templ = stralloc(lbl_templ);
+	    }
+	    if(access(lbl_templ, R_OK) == -1) {
+		fprintf(outf,
+			"ERROR: cannot access lbl_templ file %s: %s\n",
+			lbl_templ,
+			strerror(errno));
+		confbad = 1;
+	    }
+#if !defined(LPRCMD)
+	    fprintf(outf, "ERROR: lbl_templ set but no LPRCMD defined, you should reconfigure amanda\n       and make sure it find a lpr or lp command.\n");
+	    confbad = 1;
+#endif
 	}
     }
 
@@ -606,13 +772,17 @@ int start_server_check(fd, do_localchk, do_tapechk)
 		     || test_server_pgm(outf, sbindir, "amreport",
 					0, uid_dumpuser);
 	}
+	if(access(COMPRESS_PATH, X_OK) == -1) {
+	    fprintf(outf, "WARNING: %s is not executable, server-compression and indexing will not work\n",
+	            COMPRESS_PATH);
+	}
     }
 
     /*
      * Check that the directory for the tapelist file is writable, as well
      * as the tapelist file itself (if it already exists).  Also, check for
      * a "hold" file (just because it is convenient to do it here) and warn
-     * if tapedev is set to "/dev/null".
+     * if tapedev is set to the null device.
      */
 
     if(do_localchk || do_tapechk) {
@@ -656,7 +826,7 @@ int start_server_check(fd, do_localchk, do_tapechk)
 	amfree(tape_dir);
 	amfree(holdfile);
 	tapename = getconf_str(CNF_TAPEDEV);
-	if (strcmp(tapename, "/dev/null") == 0) {
+	if (strncmp(tapename, "null:", 5) == 0) {
 	    fprintf(outf,
 		    "WARNING: tapedev is %s, dumps will be thrown away\n",
 		    tapename);
@@ -763,6 +933,7 @@ int start_server_check(fd, do_localchk, do_tapechk)
 	}
 
 	amfree(logfile);
+	amfree(conf_logdir);
     }
 
     if (testtape) {
@@ -780,13 +951,13 @@ int start_server_check(fd, do_localchk, do_tapechk)
 	if(changer_init() && (tapename = taper_scan()) == NULL) {
 	    fprintf(outf, "ERROR: %s\n", changer_resultstr);
 	    tapebad = 1;
-	} else if(access(tapename,F_OK|R_OK|W_OK) == -1) {
+	} else if(tape_access(tapename,F_OK|R_OK|W_OK) == -1) {
 	    fprintf(outf, "ERROR: %s: %s\n", tapename, strerror(errno));
 	    tapebad = 1;
 	} else if((errstr = tape_rdlabel(tapename, &datestamp, &label)) != NULL) {
 	    fprintf(outf, "ERROR: %s: %s\n", tapename, errstr);
 	    tapebad = 1;
-	} else {
+	} else if(strcmp(label, FAKE_LABEL) != 0) {
 	    tp = lookup_tapelabel(label);
 	    if(tp != NULL && !reusable_tape(tp)) {
 		fprintf(outf, "ERROR: cannot overwrite active tape %s\n", label);
@@ -846,18 +1017,27 @@ int start_server_check(fd, do_localchk, do_tapechk)
 	int hostindexdir_checked = 0;
 	char *host;
 	char *disk;
+	int conf_tapecycle, conf_runspercycle;
+
+	conf_tapecycle = getconf_int(CNF_TAPECYCLE);
+	conf_runspercycle = getconf_int(CNF_RUNSPERCYCLE);
+
+	if(conf_tapecycle <= conf_runspercycle) {
+		fprintf(outf, "WARNING: tapecycle (%d) <= runspercycle (%d).\n",
+			conf_tapecycle, conf_runspercycle);
+	}
 
 	conf_infofile = stralloc(getconf_str(CNF_INFOFILE));
-	if (*conf_infofile == '/') {
-	    conf_infofile = stralloc(conf_infofile);
-	} else {
-	    conf_infofile = stralloc2(config_dir, conf_infofile);
+	if (*conf_infofile != '/') {
+	    char *ci = stralloc2(config_dir, conf_infofile);
+	    amfree(conf_infofile);
+	    conf_infofile = ci;
 	}
 	conf_indexdir = stralloc(getconf_str(CNF_INDEXDIR));
-	if (*conf_indexdir == '/') {
-	    conf_indexdir = stralloc(conf_indexdir);
-	} else {
-	    conf_indexdir = stralloc2(config_dir, conf_indexdir);
+	if (*conf_indexdir != '/') {
+	    char *ci = stralloc2(config_dir, conf_indexdir);
+	    amfree(conf_indexdir);
+	    conf_indexdir = ci;
 	}
 #if TEXTDB
 	if(stat(conf_infofile, &statbuf) == -1) {
@@ -1006,13 +1186,6 @@ int start_server_check(fd, do_localchk, do_tapechk)
 	amfree(conf_indexdir);
     }
 
-    if(do_localchk) {
-	if(access(COMPRESS_PATH, X_OK) == -1) {
-	    fprintf(outf, "WARNING: %s is not executable, server-compression and indexing will not work\n",
-	            COMPRESS_PATH);
-	}
-    }
-
     amfree(datestamp);
     amfree(label);
     amfree(config_dir);
@@ -1029,6 +1202,7 @@ int start_server_check(fd, do_localchk, do_tapechk)
     }
 
     exit(userbad \
+	 || confbad \
 	 || tapebad \
 	 || disklow \
 	 || logbad \
@@ -1043,24 +1217,223 @@ int start_server_check(fd, do_localchk, do_tapechk)
 
 int remote_errors;
 FILE *outf;
+int amanda_port;
+
+#ifdef KRB4_SECURITY
+int kamanda_port;
+#endif
 
 static void handle_response P((proto_t *p, pkt_t *pkt));
 
+#define HOST_READY				((void *)0)	/* must be 0 */
+#define HOST_ACTIVE				((void *)1)
+#define HOST_DONE				((void *)2)
+
+#define DISK_READY				((void *)0)	/* must be 0 */
+#define DISK_ACTIVE				((void *)1)
+#define DISK_DONE				((void *)2)
+
+int start_host(hostp)
+    host_t *hostp;
+{
+    disk_t *dp;
+    char *req = NULL;
+    int req_len = 0;
+    int rc;
+    int disk_count;
+    char number[NUM_STR_SIZE];
+
+    if(hostp->up != HOST_READY) {
+	return 0;
+    }
+
+    /*
+     * The first time through here we send a "noop" request.  This will
+     * return the feature list from the client if it supports that.
+     * If it does not, handle_result() will set the feature list to an
+     * empty structure.  In either case, we do the disks on the second
+     * (and subsequent) pass(es).
+     */
+    disk_count = 0;
+    if(hostp->features != NULL) { /* selfcheck service */
+	int has_features = am_has_feature(hostp->features,
+					  fe_req_options_features);
+	int has_hostname = am_has_feature(hostp->features,
+					  fe_req_options_hostname);
+	int has_maxdumps = am_has_feature(hostp->features,
+					  fe_req_options_maxdumps);
+
+	if(!am_has_feature(hostp->features, fe_selfcheck_req) &&
+	   !am_has_feature(hostp->features, fe_selfcheck_req_device)) {
+	    fprintf(outf,
+		    "ERROR: Client %s does not support selfcheck REQ packet.\n",
+		    hostp->hostname);
+	}
+	if(!am_has_feature(hostp->features, fe_selfcheck_rep)) {
+	    fprintf(outf,
+		    "ERROR: Client %s does not support selfcheck REP packet.\n",
+		    hostp->hostname);
+	}
+	if(!am_has_feature(hostp->features, fe_sendsize_req_options) &&
+	   !am_has_feature(hostp->features, fe_sendsize_req_no_options) &&
+	   !am_has_feature(hostp->features, fe_sendsize_req_device)) {
+	    fprintf(outf,
+		    "ERROR: Client %s does not support sendsize REQ packet.\n",
+		    hostp->hostname);
+	}
+	if(!am_has_feature(hostp->features, fe_sendsize_rep)) {
+	    fprintf(outf,
+		    "ERROR: Client %s does not support sendsize REP packet.\n",
+		    hostp->hostname);
+	}
+	if(!am_has_feature(hostp->features, fe_sendbackup_req) &&
+	   !am_has_feature(hostp->features, fe_sendbackup_req_device)) {
+	    fprintf(outf,
+		   "ERROR: Client %s does not support sendbackup REQ packet.\n",
+		   hostp->hostname);
+	}
+	if(!am_has_feature(hostp->features, fe_sendbackup_rep)) {
+	    fprintf(outf,
+		   "ERROR: Client %s does not support sendbackup REP packet.\n",
+		   hostp->hostname);
+	}
+
+	ap_snprintf(number, sizeof(number), "%d", hostp->maxdumps);
+	req = vstralloc("SERVICE ", "selfcheck", "\n",
+			"OPTIONS ",
+			has_features ? "features=" : "",
+			has_features ? our_feature_string : "",
+			has_features ? ";" : "",
+			has_maxdumps ? "maxdumps=" : "",
+			has_maxdumps ? number : "",
+			has_maxdumps ? ";" : "",
+			has_hostname ? "hostname=" : "",
+			has_hostname ? hostp->hostname : "",
+			has_hostname ? ";" : "",
+			"\n",
+			NULL);
+
+	req_len = strlen(req);
+	req_len += 128;				/* room for SECURITY ... */
+	req_len += 256;				/* room for non-disk answers */
+	for(dp = hostp->disks; dp != NULL; dp = dp->hostnext) {
+	    char *l;
+	    int l_len;
+	    char *o;
+
+	    if(dp->todo == 0) continue;
+
+	    if(dp->up != DISK_READY) {
+		continue;
+	    }
+	    o = optionstr(dp, hostp->features, outf);
+
+	    if(dp->device) {
+		if(!am_has_feature(hostp->features, fe_selfcheck_req_device)) {
+		    fprintf(outf,
+		     "ERROR: %s:%s (%s): selfcheck does not support device.\n",
+		     hostp->hostname, dp->name, dp->device);
+		}
+		if(!am_has_feature(hostp->features, fe_sendsize_req_device)) {
+		    fprintf(outf,
+		     "ERROR: %s:%s (%s): sendsize does not support device.\n",
+		     hostp->hostname, dp->name, dp->device);
+		}
+		if(!am_has_feature(hostp->features, fe_sendbackup_req_device)) {
+		    fprintf(outf,
+		     "ERROR: %s:%s (%s): sendbackup does not support device.\n",
+		     hostp->hostname, dp->name, dp->device);
+		}
+	    }
+	    if(strcmp(dp->program, "DUMP") == 0 &&
+	       !am_has_feature(hostp->features, fe_program_dump)) {
+		fprintf(outf, "ERROR: %s:%s does not support DUMP.\n",
+			hostp->hostname, dp->name);
+	    }
+	    if(strcmp(dp->program, "GNUTAR") == 0 &&
+	       !am_has_feature(hostp->features, fe_program_gnutar)) {
+		fprintf(outf, "ERROR: %s:%s does not support GNUTAR.\n",
+			hostp->hostname, dp->name);
+	    }
+	    l = vstralloc(dp->program, 
+			  " ",
+			  dp->name,
+			  " ",
+			  dp->device ? dp->device : "",
+			  " 0 OPTIONS |",
+			  o,
+			  "\n",
+			  NULL);
+	    l_len = strlen(l);
+	    amfree(o);
+	    /*
+	     * Allow 2X for error response in return packet.
+	     */
+	    if(req_len + l_len > MAX_DGRAM / 2) {
+		amfree(l);
+		break;
+	    }
+	    strappend(req, l);
+	    req_len += l_len;
+	    amfree(l);
+	    dp->up = DISK_ACTIVE;
+	    disk_count++;
+	}
+
+    }
+    else { /* noop service */
+	req = vstralloc("SERVICE ", "noop", "\n",
+			"OPTIONS ",
+			"features=", our_feature_string, ";",
+			"\n",
+			NULL);
+	for(dp = hostp->disks; dp != NULL; dp = dp->hostnext) {
+	    if(dp->todo == 0) continue;
+
+	    if(dp->up != DISK_READY) {
+		continue;
+	    }
+	    disk_count++;
+	}
+    }
+    if(disk_count == 0) {
+	amfree(req);
+	hostp->up = HOST_DONE;
+	return 0;
+    }
+
+#ifdef KRB4_SECURITY
+    if(hostp->disks->auth == AUTH_KRB4)
+	rc = make_krb_request(hostp->hostname, kamanda_port, req,
+			      hostp, conf_ctimeout, handle_response);
+    else
+#endif
+        rc = make_request(hostp->hostname, amanda_port, req,
+			  hostp, conf_ctimeout, handle_response);
+
+    req = NULL;				/* do not own this any more */
+
+    if(rc) {
+	/* couldn't resolve hostname */
+	fprintf(outf,
+		"ERROR: %s: could not resolve hostname\n", hostp->hostname);
+	remote_errors++;
+	hostp->up = HOST_DONE;
+    } else {
+	hostp->up = HOST_ACTIVE;
+    }
+    return 1;
+}
+
 int start_client_checks(fd)
 int fd;
 {
-    disk_t *dp;
     host_t *hostp;
-    char *req = NULL;
-    int hostcount, rc, pid;
-    int amanda_port;
+    disk_t *dp;
+    int hostcount, pid;
     struct servent *amandad;
     int userbad = 0;
 
-#ifdef KRB4_SECURITY
-    int kamanda_port;
-#endif
-
     switch(pid = fork()) {
     case -1: error("could not fork client check: %s", strerror(errno));
     case 0: break;
@@ -1082,33 +1455,6 @@ int fd;
     fprintf(outf, "\nAmanda Backup Client Hosts Check\n");
     fprintf(outf,   "--------------------------------\n");
 
-    /*
-     * Make sure we are running as the dump user.
-     */
-    {
-	uid_t uid_me = getuid();
-	uid_t uid_dumpuser = uid_me;
-	char *dumpuser = getconf_str(CNF_DUMPUSER);
-	struct passwd *pw;
-
-	if ((pw = getpwnam(dumpuser)) == NULL) {
-	    fprintf(outf, "ERROR: cannot look up dump user \"%s\"\n", dumpuser);
-	    userbad = 1;
-	} else {
-	    uid_dumpuser = pw->pw_uid;
-	}
-	if ((pw = getpwuid(uid_me)) == NULL) {
-	    fprintf(outf, "ERROR: cannot look up my own uid %ld\n",
-		    (long)uid_me);
-	    userbad = 1;
-	}
-	if (uid_me != uid_dumpuser) {
-	    fprintf(outf, "ERROR: running as user \"%s\" instead of \"%s\"\n",
-		    pw->pw_name, dumpuser);
-	    userbad = 1;
-	}
-    }
-
 #ifdef KRB4_SECURITY
     kerberos_service_init();
 #endif
@@ -1130,50 +1476,16 @@ int fd;
 
     hostcount = remote_errors = 0;
 
-    while(!empty(*origqp)) {
-	req = vstralloc("SERVICE selfcheck\n",
-			"OPTIONS ;\n",
-			NULL);
-	hostp = origqp->head->host;
-	for(dp = hostp->disks; dp != NULL; dp = dp->hostnext) {
-	    char *t;
-	    char *o;
-
-	    remove_disk(origqp, dp);
-	    o = optionstr(dp);
-	    t = vstralloc(req,
-			  dp->program, 
-			  " ",
-			  dp->name,
-			  " 0 OPTIONS |",
-			  o,
-			  "\n",
-			  NULL);
-	    amfree(req);
-	    amfree(o);
-	    req = t;
+    for(dp = origqp->head; dp != NULL; dp = dp->next) {
+	hostp = dp->host;
+	if(hostp->up == HOST_READY) {
+	    if(start_host(hostp) == 1) {
+		hostcount++;
+		check_protocol();
+	    }
 	}
-	hostcount++;
-
-#ifdef KRB4_SECURITY
-	if(hostp->disks->auth == AUTH_KRB4)
-	    rc = make_krb_request(hostp->hostname, kamanda_port, req,
-				  hostp, conf_ctimeout, handle_response);
-	else
-#endif
-	    rc = make_request(hostp->hostname, amanda_port, req,
-			      hostp, conf_ctimeout, handle_response);
-
-	req = NULL;				/* do not own this any more */
-
-	if(rc) {
-	    /* couldn't resolve hostname */
-	    fprintf(outf,
-		    "ERROR: %s: could not resolve hostname\n", hostp->hostname);
-	    remote_errors++;
-	}
-	check_protocol();
     }
+
     run_protocol();
     amfree(msg);
 
@@ -1203,31 +1515,29 @@ proto_t *p;
 pkt_t *pkt;
 {
     host_t *hostp;
-    char *errstr;
+    disk_t *dp;
+    char *line;
     char *s;
+    char *t;
     int ch;
+    int tch;
 
     hostp = (host_t *) p->datap;
+    hostp->up = HOST_READY;
 
-    if(p->state == S_FAILED) {
-	if(pkt == NULL) {
+    if(p->state == S_FAILED && pkt == NULL) {
+	if(p->prevstate == S_REPWAIT) {
+	    fprintf(outf,
+		    "WARNING: %s: selfcheck reply timed out.\n",
+		    hostp->hostname);
+	}
+	else {
 	    fprintf(outf,
 		    "WARNING: %s: selfcheck request timed out.  Host down?\n",
 		    hostp->hostname);
-#define sc "ERROR"
-	} else if(strncmp(pkt->body, sc, sizeof(sc)-1) == 0) {
-	    s = pkt->body + sizeof(sc)-1;
-	    ch = *s++;
-#undef sc
-	    skip_whitespace(s, ch);
-	    errstr = s - 1;
-
-	    fprintf(outf, "ERROR: %s NAK: %s\n", hostp->hostname, errstr);
-	} else {
-	    fprintf(outf, "ERROR: %s NAK: [NAK parse failed]\n",
-		    hostp->hostname);
 	}
 	remote_errors++;
+	hostp->up = HOST_DONE;
 	return;
     }
 
@@ -1237,41 +1547,95 @@ pkt_t *pkt;
 	fprintf(outf, "ERROR: %s [mutual-authentication failed]\n",
 		hostp->hostname);
 	remote_errors++;
+	hostp->up = HOST_DONE;
 	return;
     }
 #endif
 
+#if 0
+    fprintf(errf, "got %sresponse from %s:\n----\n%s----\n\n",
+	    (p->state == S_FAILED) ? "NAK " : "", hostp->hostname, pkt->body);
+#endif
+
     s = pkt->body;
     ch = *s++;
-/*
-    fprintf(errf, "got response from %s:\n----\n%s----\n\n",
-	    hostp->hostname, resp);
-*/
-
-#define sc "OPTIONS"
-    if(strncmp(s - 1, sc, sizeof(sc)-1) == 0) {
-	/* no options yet */
-	skip_line(s, ch);
-    }
-#undef sc
-
     while(ch) {
-#define sc "ERROR"
-	if(strncmp(s - 1, sc, sizeof(sc)-1) == 0) {
-	    s += sizeof(sc)-1;
-	    ch = s[-1];
+	line = s - 1;
+	skip_line(s, ch);
+	if (s[-2] == '\n') {
+	    s[-2] = '\0';
+	}
+
+#define sc "OPTIONS "
+	if(strncmp(line, sc, sizeof(sc)-1) == 0) {
 #undef sc
 
-	    skip_whitespace(s, ch);
-	    errstr = s - 1;
-	    skip_line(s, ch);
-	    /* overwrite '\n'; s-1 points to the beginning of the next line */
-	    s[-2] = '\0';
+#define sc "features="
+	    t = strstr(line, sc);
+	    if(t != NULL && (isspace((int)t[-1]) || t[-1] == ';')) {
+		t += sizeof(sc)-1;
+#undef sc
+		am_release_feature_set(hostp->features);
+		if((hostp->features = am_string_to_feature(t)) == NULL) {
+		    fprintf(outf, "ERROR: %s: bad features value: %s\n",
+			    hostp->hostname, line);
+		}
+	    }
 
-	    fprintf(outf, "ERROR: %s: %s\n", hostp->hostname, errstr);
-	    remote_errors++;
-	} else {
-	    skip_line(s, ch);
+	    continue;
+	}
+
+#define sc "OK "
+	if(strncmp(line, sc, sizeof(sc)-1) == 0) {
+#undef sc
+	    continue;
+	}
+
+#define sc "ERROR "
+	if(strncmp(line, sc, sizeof(sc)-1) == 0) {
+	    t = line + sizeof(sc)-1;
+	    tch = t[-1];
+#undef sc
+
+	    skip_whitespace(t, tch);
+	    /*
+	     * If the "error" is that the "noop" service is unknown, it
+	     * just means the client is "old" (does not support the service).
+	     * We can ignore this.
+	     */
+	    if(hostp->features == NULL
+	       && p->state == S_FAILED
+	       && (strcmp(t - 1, "unknown service: noop") == 0
+	           || strcmp(t - 1, "noop: invalid service") == 0)) {
+	    } else {
+		fprintf(outf, "ERROR: %s%s: %s\n",
+		        (p->state == S_FAILED) ? "NAK " : "",
+		        hostp->hostname,
+		        t - 1);
+		remote_errors++;
+		hostp->up = HOST_DONE;
+	    }
+	    continue;
+	}
+
+	fprintf(outf, "ERROR: %s: unknown response: %s\n",
+		hostp->hostname, line);
+	remote_errors++;
+	hostp->up = HOST_DONE;
+    }
+    if(hostp->up == HOST_READY && hostp->features == NULL) {
+	/*
+	 * The client does not support the features list, so give it an
+	 * empty one.
+	 */
+	dbprintf(("%s: no feature set from host %s\n",
+		  debug_prefix_time(NULL), hostp->hostname));
+	hostp->features = am_set_default_feature_set();
+    }
+    for(dp = hostp->disks; dp != NULL; dp = dp->hostnext) {
+	if(dp->up == DISK_ACTIVE) {
+	    dp->up = DISK_DONE;
 	}
     }
+    start_host(hostp);
 }
