@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amindexd.c,v 1.39.2.12 2001/02/25 21:10:26 martinea Exp $
+ * $Id: amindexd.c,v 1.39.2.11.4.4.2.13 2003/10/27 18:33:03 martinea Exp $
  *
  * This is the server daemon part of the index client/server system.
  * It is assumed that this is launched from inetd instead of being
@@ -44,6 +44,7 @@
 #include "conffile.h"
 #include "diskfile.h"
 #include "arglist.h"
+#include "clock.h"
 #include "dgram.h"
 #include "version.h"
 #include "protocol.h"
@@ -76,8 +77,6 @@ char local_hostname[MAX_HOSTNAME_LENGTH+1];	/* me! */
 char *remote_hostname = NULL;			/* the client */
 char *dump_hostname = NULL;			/* machine we are restoring */
 char *disk_name;				/* disk we are restoring */
-char *config_name = NULL;			/* config we are restoring */
-char *config_dir = NULL;			/* config directory */
 char *target_date = NULL;
 disklist_t *disk_list;				/* all disks in cur config */
 find_result_t *output_find = NULL;
@@ -87,13 +86,24 @@ static int amindexd_debug = 0;
 static REMOVE_ITEM *uncompress_remove = NULL;
 					/* uncompressed files to remove */
 
+static am_feature_t *our_features = NULL;
+static am_feature_t *their_features = NULL;
+
+static void reply P((int n, char * fmt, ...))
+    __attribute__ ((format (printf, 2, 3)));
+static void lreply P((int n, char * fmt, ...))
+    __attribute__ ((format (printf, 2, 3)));
+static void fast_lreply P((int n, char * fmt, ...))
+    __attribute__ ((format (printf, 2, 3)));
+
 REMOVE_ITEM *remove_files(remove)
 REMOVE_ITEM *remove;
 {
     REMOVE_ITEM *prev;
 
     while(remove) {
-	dbprintf(("Removing index file: %s\n", remove->filename));
+	dbprintf(("%s: removing index file: %s\n",
+		  debug_prefix_time(NULL), remove->filename));
 	unlink(remove->filename);
 	amfree(remove->filename);
 	prev = remove;
@@ -134,7 +144,8 @@ char **emsg;
 			" | sort",
 			" > ", "\'", filename, "\'",
 			NULL);
-	dbprintf(("Uncompress command: %s\n",cmd));
+	dbprintf(("%s: uncompress command: %s\n",
+		  debug_prefix_time(NULL), cmd));
 	if (system(cmd)!=0) {
 	    amfree(*emsg);
 	    *emsg = vstralloc("\"", cmd, "\" failed", NULL);
@@ -219,6 +230,7 @@ char **emsg;
 	    }
 	    if(old_line == NULL || strcmp(line, old_line) != 0) {
 		add_dir_list_item(dump_item, line);
+		amfree(old_line);
 		old_line = line;
 		line = NULL;
 	    }
@@ -233,7 +245,7 @@ char **emsg;
 }
 
 /* send a 1 line reply to the client */
-arglist_function1(void reply, int, n, char *, fmt)
+printf_arglist_function1(static void reply, int, n, char *, fmt)
 {
     va_list args;
     char buf[STR_SIZE];
@@ -245,21 +257,23 @@ arglist_function1(void reply, int, n, char *, fmt)
 
     if (printf("%s\r\n", buf) < 0)
     {
-	dbprintf(("! error %d (%s) in printf\n", errno, strerror(errno)));
+	dbprintf(("%s: ! error %d (%s) in printf\n",
+		  debug_prefix_time(NULL), errno, strerror(errno)));
 	uncompress_remove = remove_files(uncompress_remove);
 	exit(1);
     }
     if (fflush(stdout) != 0)
     {
-	dbprintf(("! error %d (%s) in fflush\n", errno, strerror(errno)));
+	dbprintf(("%s: ! error %d (%s) in fflush\n",
+		  debug_prefix_time(NULL), errno, strerror(errno)));
 	uncompress_remove = remove_files(uncompress_remove);
 	exit(1);
     }
-    dbprintf(("< %s\n", buf));
+    dbprintf(("%s: < %s\n", debug_prefix_time(NULL), buf));
 }
 
 /* send one line of a multi-line response */
-arglist_function1(void lreply, int, n, char *, fmt)
+printf_arglist_function1(static void lreply, int, n, char *, fmt)
 {
     va_list args;
     char buf[STR_SIZE];
@@ -271,22 +285,24 @@ arglist_function1(void lreply, int, n, char *, fmt)
 
     if (printf("%s\r\n", buf) < 0)
     {
-	dbprintf(("! error %d (%s) in printf\n", errno, strerror(errno)));
+	dbprintf(("%s: ! error %d (%s) in printf\n",
+		  debug_prefix_time(NULL), errno, strerror(errno)));
 	uncompress_remove = remove_files(uncompress_remove);
 	exit(1);
     }
     if (fflush(stdout) != 0)
     {
-	dbprintf(("! error %d (%s) in fflush\n", errno, strerror(errno)));
+	dbprintf(("%s: ! error %d (%s) in fflush\n",
+		  debug_prefix_time(NULL), errno, strerror(errno)));
 	uncompress_remove = remove_files(uncompress_remove);
 	exit(1);
     }
 
-    dbprintf(("< %s\n", buf));
+    dbprintf(("%s: < %s\n", debug_prefix_time(NULL), buf));
 }
 
 /* send one line of a multi-line response */
-arglist_function1(void fast_lreply, int, n, char *, fmt)
+printf_arglist_function1(static void fast_lreply, int, n, char *, fmt)
 {
     va_list args;
     char buf[STR_SIZE];
@@ -298,7 +314,8 @@ arglist_function1(void fast_lreply, int, n, char *, fmt)
 
     if (printf("%s\r\n", buf) < 0)
     {
-	dbprintf(("! error %d (%s) in printf\n", errno, strerror(errno)));
+	dbprintf(("%s: ! error %d (%s) in printf\n",
+		  debug_prefix_time(NULL), errno, strerror(errno)));
 	uncompress_remove = remove_files(uncompress_remove);
 	exit(1);
     }
@@ -314,6 +331,7 @@ char *host;
 {
     struct stat dir_stat;
     char *fn;
+    host_t *ihost;
 
     if (config_name == NULL) {
 	reply(501, "Must set config before setting host.");
@@ -334,10 +352,16 @@ char *host;
 #endif
 
     /* check that the config actually handles that host */
-    /* assume in index dir already */
+    ihost = lookup_host(host);
+    if(ihost == NULL) {
+	reply(501, "Host %s is not in your disklist.", host);
+	return -1;
+    }
+
+    /* assume an index dir already */
     fn = getindexfname(host, NULL, NULL, 0);
     if (stat (fn, &dir_stat) != 0 || !S_ISDIR(dir_stat.st_mode)) {
-	reply(501, "No index records for host: %s. Invalid?", host);
+	reply(501, "No index records for host: %s. Have you enabled indexing?", host);
 	amfree(fn);
 	return -1;
     }
@@ -352,14 +376,22 @@ char *disk;
 {
     char *fn;
     struct stat dir_stat;
+    disk_t *idisk;
 
     if (config_name == NULL || dump_hostname == NULL) {
 	reply(501, "Must set config,host before setting disk.");
 	return -1;
     }
 
-    fn = getindexfname(dump_hostname, disk, NULL, 0);
+    /* check that the config actually handles that disk */
+    idisk = lookup_disk(dump_hostname, disk);
+    if(idisk == NULL) {
+	reply(501, "Disk %s:%s is not in your disklist.", dump_hostname, disk);
+	return -1;
+    }
 
+    /* assume an index dir already */
+    fn = getindexfname(dump_hostname, disk, NULL, 0);
     if (stat (fn, &dir_stat) != 0 || !S_ISDIR(dir_stat.st_mode)) {
 	reply(501, "No index records for disk: %s. Invalid?", disk);
 	amfree(fn);
@@ -419,6 +451,9 @@ char *config;
     }
     amfree(conf_tapelist);
 
+    output_find = find_dump(1, disk_list);
+    sort_find_result("DLKHB", &output_find);
+
     /* okay, now look for the index directory */
     conf_indexdir = getconf_str(CNF_INDEXDIR);
     if(*conf_indexdir == '/') {
@@ -439,7 +474,10 @@ char *config;
 
 int build_disk_table P((void))
 {
-    char date[100];
+    char date[3 * NUM_STR_SIZE + 2 + 1];
+    long last_datestamp;
+    int last_filenum;
+    int last_level;
     find_result_t *find_output;
 
     if (config_name == NULL || dump_hostname == NULL || disk_name == NULL) {
@@ -447,25 +485,38 @@ int build_disk_table P((void))
 	return -1;
     }
 
-    if(output_find == NULL) { /* do it the first time only */
-	output_find = find_dump(NULL,0,NULL);
-	sort_find_result("DLKHB", &output_find);
-    }
-
     clear_list();
-    for(find_output = output_find; find_output != NULL; 
+    last_datestamp = -1;
+    last_filenum = -1;
+    last_level = -1;
+    for(find_output = output_find;
+	find_output != NULL; 
 	find_output = find_output->next) {
 	if(strcasecmp(dump_hostname, find_output->hostname) == 0 &&
 	   strcmp(disk_name    , find_output->diskname) == 0 &&
 	   strcmp("OK"         , find_output->status)   == 0) {
+	    /*
+	     * The sort order puts holding disk entries first.	We want to
+	     * use them if at all possible, so ignore any other entries
+	     * for the same datestamp after we see a holding disk entry
+	     * (as indicated by a filenum of zero).
+	     */
+	    if(find_output->datestamp == last_datestamp &&
+	       find_output->level == last_level && last_filenum == 0) {
+		continue;
+	    }
+	    last_datestamp = find_output->datestamp;
+	    last_filenum = find_output->filenum;
+	    last_level = find_output->level;
 	    ap_snprintf(date, sizeof(date), "%04d-%02d-%02d",
 			find_output->datestamp/10000,
 			(find_output->datestamp/100) %100,
 			find_output->datestamp %100);
 	    add_dump(date, find_output->level, find_output->label, 
 		     find_output->filenum);
-	    dbprintf(("- %s %d %s %d\n", date, find_output->level, 
-		     find_output->label, find_output->filenum));
+	    dbprintf(("%s: - %s %d %s %d\n",
+		      debug_prefix_time(NULL), date, find_output->level, 
+		      find_output->label, find_output->filenum));
 	}
     }
     return 0;
@@ -553,7 +604,7 @@ char *dir;
 	    return -1;
 	}
 	amfree(filename_gz);
-	dbprintf(("f %s\n", filename));
+	dbprintf(("%s: f %s\n", debug_prefix_time(NULL), filename));
 	if ((fp = fopen(filename, "r")) == NULL) {
 	    reply(599, "System error %s", strerror(errno));
 	    amfree(filename);
@@ -566,6 +617,7 @@ char *dir;
 	    }
 	    amfree(filename);
 	    amfree(ldir);
+	    amfree(line);
 	    afclose(fp);
 	    return 0;
 	}
@@ -643,20 +695,38 @@ int  recursive;
     {
 	lreply(200, " Opaque recursive list of %s", dir);
 	for (dir_item = get_dir_list(); dir_item != NULL; 
-	     dir_item = dir_item->next)
-	    fast_lreply(201, " %s %d %-16s %s",
-			dir_item->dump->date, dir_item->dump->level,
-			dir_item->dump->tape, dir_item->path);
+	     dir_item = dir_item->next) {
+	    if(am_has_feature(their_features, fe_amindexd_fileno_in_ORLD)){
+		fast_lreply(201, " %s %d %-16s %d %s",
+			    dir_item->dump->date, dir_item->dump->level,
+			    dir_item->dump->tape, dir_item->dump->file,
+			    dir_item->path);
+	    }
+	    else {
+		fast_lreply(201, " %s %d %-16s %s",
+			    dir_item->dump->date, dir_item->dump->level,
+			    dir_item->dump->tape, dir_item->path);
+	    }
+	}
 	reply(200, " Opaque recursive list of %s", dir);
     }
     else
     {
 	lreply(200, " Opaque list of %s", dir);
 	for (dir_item = get_dir_list(); dir_item != NULL; 
-	     dir_item = dir_item->next)
-	    lreply(201, " %s %d %-16s %s",
-		   dir_item->dump->date, dir_item->dump->level,
-		   dir_item->dump->tape, dir_item->path);
+	     dir_item = dir_item->next) {
+	    if(am_has_feature(their_features, fe_amindexd_fileno_in_OLSD)){
+		lreply(201, " %s %d %-16s %d %s",
+			    dir_item->dump->date, dir_item->dump->level,
+			    dir_item->dump->tape, dir_item->dump->file,
+			    dir_item->path);
+	    }
+	    else {
+		lreply(201, " %s %d %-16s %s",
+			    dir_item->dump->date, dir_item->dump->level,
+			    dir_item->dump->tape, dir_item->path);
+	    }
+	}
 	reply(200, " Opaque list of %s", dir);
     }
     clear_dir_list();
@@ -732,12 +802,12 @@ char **argv;
     struct sockaddr_in his_addr;
     struct hostent *his_name;
     char *arg;
-    int arg_len;
     char *cmd;
     int len;
     int fd;
     int user_validated = 0;
     char *errstr = NULL;
+    char *pgm = "amindexd";			/* in case argv[0] is not set */
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -751,7 +821,21 @@ char **argv;
 
     safe_cd();
 
-    set_pname("amindexd");
+    /*
+     * When called via inetd, it is not uncommon to forget to put the
+     * argv[0] value on the config line.  On some systems (e.g. Solaris)
+     * this causes argv and/or argv[0] to be NULL, so we have to be
+     * careful getting our name.
+     */
+    if (argc >= 1 && argv != NULL && argv[0] != NULL) {
+	if((pgm = strrchr(argv[0], '/')) != NULL) {
+	    pgm++;
+	} else {
+	    pgm = argv[0];
+	}
+    }
+
+    set_pname(pgm);
 
 #ifdef FORCE_USERID
 
@@ -769,6 +853,22 @@ char **argv;
 
 #endif	/* FORCE_USERID */
 
+    dbopen();
+    startclock();
+    dbprintf(("%s: version %s\n", get_pname(), version()));
+
+    if (! (argc >= 1 && argv != NULL && argv[0] != NULL)) {
+	dbprintf(("%s: WARNING: argv[0] not defined: check inetd.conf\n",
+		  debug_prefix_time(NULL)));
+    }
+
+    {
+	int db_fd = dbfd();
+	if(db_fd != -1) {
+	    dup2(db_fd, 2);
+	}
+    }
+
     /* initialize */
 
     argc--;
@@ -787,9 +887,6 @@ char **argv;
 	argv++;
     }
 
-    dbopen();
-    dbprintf(("%s: version %s\n", get_pname(), version()));
-
     if(gethostname(local_hostname, sizeof(local_hostname)-1) == -1)
 	error("gethostname: %s", strerror(errno));
     local_hostname[sizeof(local_hostname)-1] = '\0';
@@ -807,7 +904,7 @@ char **argv;
 	 */
 	his_name = gethostbyname(local_hostname);
 	if(his_name == NULL) {
-	    error("gethostbyname: %s: %s\n", local_hostname, strerror(errno));
+	    error("gethostbyname(%s) failed\n", local_hostname);
 	}
 	assert(his_name->h_addrtype == AF_INET);
 	his_addr.sin_family = his_name->h_addrtype;
@@ -827,9 +924,10 @@ char **argv;
 	      htons(his_addr.sin_port));
     }
     if ((his_name = gethostbyaddr((char *)&(his_addr.sin_addr),
-				  sizeof(struct in_addr),
+				  sizeof(his_addr.sin_addr),
 				  AF_INET)) == NULL) {
-	error("gethostbyaddr: %s", strerror(errno));
+	error("gethostbyaddr(%s): hostname lookup failed",
+	      inet_ntoa(his_addr.sin_addr));
     }
     fp = s = his_name->h_name;
     ch = *s++;
@@ -843,6 +941,9 @@ char **argv;
     amfree(disk_name);
     amfree(target_date);
 
+    our_features = am_init_feature_set();
+    their_features = am_set_default_feature_set();
+
     if (config_name != NULL && is_config_valid(config_name) != -1) {
 	return 1;
     }
@@ -858,14 +959,17 @@ char **argv;
 	while(1) {
 	    if((part = agets(stdin)) == NULL) {
 		if(errno != 0) {
-		    dbprintf(("? read error: %s\n", strerror(errno)));
+		    dbprintf(("%s: ? read error: %s\n",
+			      debug_prefix_time(NULL), strerror(errno)));
 		} else {
-		    dbprintf(("? unexpected EOF\n"));
+		    dbprintf(("%s: ? unexpected EOF\n",
+			      debug_prefix_time(NULL)));
 		}
 		if(line) {
-		    dbprintf(("? unprocessed input:\n"));
+		    dbprintf(("%s: ? unprocessed input:\n",
+			      debug_prefix_time(NULL)));
 		    dbprintf(("-----\n"));
-		    dbprintf(("? %s\n", line));
+		    dbprintf(("%s\n", line));
 		    dbprintf(("-----\n"));
 		}
 		amfree(line);
@@ -896,7 +1000,7 @@ char **argv;
 	    strappend(line, "\n");
 	}
 
-	dbprintf(("> %s\n", line));
+	dbprintf(("%s: > %s\n", debug_prefix_time(NULL), line));
 
 	arg = NULL;
 	s = line;
@@ -918,13 +1022,6 @@ char **argv;
 	    if (ch) {
 		arg = s-1;
 		skip_non_whitespace(s, ch);
-		/*
-		 * Save the length of the next non-whitespace string
-		 * (e.g. a host name), but do not terminate it.  Some
-		 * commands want the rest of the line, whitespace or
-		 * not.
-		 */
-		arg_len = s-arg;
 	    }
 	}
 
@@ -966,6 +1063,49 @@ char **argv;
 		}
 	    }
 	    s[-1] = ch;
+	} else if (strcmp(cmd, "LISTDISK") == 0) {
+	    disk_t *disk;
+	    int nbdisk = 0;
+	    s[-1] = '\0';
+	    if (config_name == NULL || dump_hostname == NULL) {
+		reply(501, "Must set config, host before listdisk");
+	    }
+	    else if(arg) {
+		lreply(200, " List of disk for device %s on host %s", arg,
+		       dump_hostname);
+		for (disk = disk_list->head; disk!=NULL; disk = disk->next) {
+		    if(strcmp(disk->host->hostname, dump_hostname) == 0 &&
+		       ((disk->device && strcmp(disk->device, arg) == 0) ||
+			(!disk->device && strcmp(disk->name, arg) == 0))) {
+			fast_lreply(201, " %s", disk->name);
+			nbdisk++;
+		    }
+		}
+		if(nbdisk > 0) {
+		    reply(200, "List of disk for device %s on host %s", arg,
+			  dump_hostname);
+		}
+		else {
+		    reply(200, "No disk for device %s on host %s", arg,
+			  dump_hostname);
+		}
+	    }
+	    else {
+		lreply(200, " List of disk for host %s", dump_hostname);
+		for (disk = disk_list->head; disk!=NULL; disk = disk->next) {
+		    if(strcmp(disk->host->hostname, dump_hostname) == 0) {
+			fast_lreply(201, " %s", disk->name);
+			nbdisk++;
+		    }
+		}
+		if(nbdisk > 0) {
+		    reply(200, "List of disk for host %s", dump_hostname);
+		}
+		else {
+		    reply(200, "No disk for host %s", dump_hostname);
+		}
+	    }
+	    s[-1] = ch;
 	} else if (strcmp(cmd, "SCNF") == 0 && arg) {
 	    s[-1] = '\0';
 	    amfree(config_name);
@@ -981,6 +1121,20 @@ char **argv;
 		amfree(config_dir);
 	    }
 	    s[-1] = ch;
+	} else if (strcmp(cmd, "FEATURES") == 0 && arg) {
+	    char *our_feature_string = NULL;
+	    char *their_feature_string = NULL;
+	    s[-1] = '\0';
+	    am_release_feature_set(our_features);
+	    am_release_feature_set(their_features);
+	    our_features = am_init_feature_set();
+	    our_feature_string = am_feature_to_string(our_features);
+	    their_feature_string = newstralloc(target_date, arg);
+	    their_features = am_string_to_feature(their_feature_string);
+	    reply(200, "FEATURES %s", our_feature_string);
+	    amfree(our_feature_string);
+	    amfree(their_feature_string);
+	    s[-1] = ch;
 	} else if (strcmp(cmd, "DATE") == 0 && arg) {
 	    s[-1] = '\0';
 	    target_date = newstralloc(target_date, arg);
@@ -1005,6 +1159,7 @@ char **argv;
 	    reply(500, "Command not recognised/incorrect: %s", cmd);
 	}
     }
+    amfree(line);
 
     uncompress_remove = remove_files(uncompress_remove);
     free_find_result(&output_find);
