@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amlabel.c,v 1.18.2.15 2000/10/11 20:42:40 martinea Exp $
+ * $Id: amlabel.c,v 1.18.2.15.4.3.2.4 2003/01/01 23:28:53 martinea Exp $
  *
  * write an Amanda label on a tape
  */
@@ -38,9 +38,6 @@
 #include <vtblc.h>
 #endif /* HAVE_LIBVTBLC */
 
-char *config_name = NULL;
-char *config_dir = NULL;
-
 /* local functions */
 
 void usage P((char *argv0));
@@ -70,6 +67,8 @@ int main(argc, argv)
     int have_changer;
     int force, tape_ok;
     tape_t *tp;
+    tapetype_t *tape;
+    long tt_blocksize_kb;
     int slotcommand;
     uid_t uid_me;
     uid_t uid_dumpuser;
@@ -125,7 +124,7 @@ int main(argc, argv)
     config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
     conffile = stralloc2(config_dir, CONFFILE_NAME);
     if (read_conffile(conffile)) {
-	error("could not read config file \"%s\"", conffile);
+	error("errors processing config file \"%s\"", conffile);
     }
     conf_tapelist = getconf_str(CNF_TAPELIST);
     if (*conf_tapelist == '/') {
@@ -165,6 +164,8 @@ int main(argc, argv)
 	if(!force)
 	    error("label %s already on a tape\n",label);
     }
+    tape = lookup_tapetype(getconf_str(CNF_TAPETYPE));
+    tt_blocksize_kb = tape->blocksize;
 
     if((have_changer = changer_init()) == 0) {
 	if(slotcommand) {
@@ -223,7 +224,8 @@ int main(argc, argv)
     else {
 	/* got an amanda tape */
 	printf(" %s",oldlabel);
-	if(!match(labelstr, oldlabel)) {
+	if(strcmp(oldlabel, FAKE_LABEL) != 0
+	   && match(labelstr, oldlabel) == 0) {
 	    printf(", tape is in another amanda configuration");
 	    if(!force)
 		tape_ok=0;
@@ -237,6 +239,8 @@ int main(argc, argv)
 	}
 	printf("\n");
     }
+    amfree(oldlabel);
+    amfree(olddatestamp);
 	
     printf("rewinding"); fflush(stdout);
 
@@ -259,14 +263,16 @@ int main(argc, argv)
 
 #ifdef HAVE_LINUX_ZFTAPE_H
 	if (is_zftape(tapename) == 1){
-	    if((errstr = tapefd_wrlabel(fd, "X", label)) != NULL) {
+	    errstr = tapefd_wrlabel(fd, "X", label, tt_blocksize_kb * 1024);
+	    if(errstr != NULL) {
 		putchar('\n');
 		error(errstr);
 	    }
 	}
 	else
 #endif /* HAVE_LINUX_ZFTAPE_H */
-	if((errstr = tape_wrlabel(tapename, "X", label)) != NULL) {
+	errstr = tape_wrlabel(tapename, "X", label, tt_blocksize_kb * 1024);
+	if(errstr != NULL) {
 	    putchar('\n');
 	    error(errstr);
 	}
@@ -279,14 +285,16 @@ int main(argc, argv)
 
 #ifdef HAVE_LINUX_ZFTAPE_H
 	if (is_zftape(tapename) == 1){
-	    if((errstr = tapefd_wrendmark(fd, "X")) != NULL) {
+	    errstr = tapefd_wrendmark(fd, "X", tt_blocksize_kb * 1024);
+	    if(errstr != NULL) {
 		putchar('\n');
 		error(errstr);
 	    }
 	}
 	else
 #endif /* HAVE_LINUX_ZFTAPE_H */
-	if((errstr = tape_wrendmark(tapename, "X")) != NULL) {
+	errstr = tape_wrendmark(tapename, "X", tt_blocksize_kb * 1024);
+	if(errstr != NULL) {
 	    putchar('\n');
 	    error(errstr);
 	}
@@ -380,24 +388,27 @@ int main(argc, argv)
 	    }
 
 	    if (strcmp("X", olddatestamp) != 0 ||
-		strcmp(label, oldlabel) != 0) {
+		(strcmp(oldlabel, FAKE_LABEL) != 0
+		 && strcmp(label, oldlabel) != 0)) {
 		putchar('\n');
 		error("read label %s back, timestamp %s (expected X), what now?",
 		      oldlabel, olddatestamp);
 	    }
+	    amfree(oldlabel);
+	    amfree(olddatestamp);
 
 	    /* write tape list */
 
+	    /* make a copy */
+       	    conf_tapelist_old = stralloc2(conf_tapelist, ".amlabel");
+	    if(write_tapelist(conf_tapelist_old)) {
+	        error("couldn't write tapelist: %s", strerror(errno));
+	    }
+	    amfree(conf_tapelist_old);
+
     	    /* XXX add cur_tape number to tape list structure */
 	    remove_tapelabel(label);
     	    add_tapelabel(0, label);
-       	    conf_tapelist_old = stralloc2(conf_tapelist, ".amlabel");
-	    if (rename(conf_tapelist, conf_tapelist_old) != 0) {
-		if(errno != ENOENT)
-		    error("could not rename \"%s\" to \"%s\": %s",
-			  conf_tapelist, conf_tapelist_old, strerror(errno));
-	    }
-	    amfree(conf_tapelist_old);
 	    if(write_tapelist(conf_tapelist)) {
 	        error("couldn't write tapelist: %s", strerror(errno));
 	    }
