@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: amlogroll.c,v 1.1.2.1 1999/09/11 16:55:25 jrj Exp $
+ * $Id: amlogroll.c,v 1.1.2.1.4.1.2.3 2003/10/27 17:55:39 martinea Exp $
  *
  * rename a live log file to the datestamped name.
  */
@@ -37,9 +37,6 @@
 
 char *datestamp;
 
-char *config_name = NULL;
-char *config_dir = NULL;
-
 void handle_start P((void));
 
 int main(argc, argv)
@@ -65,7 +62,7 @@ char **argv;
 	close(fd);
     }
 
-    set_pname("amreport");
+    set_pname("amlogroll");
 
     malloc_size_1 = malloc_inuse(&malloc_hist_1);
 
@@ -93,7 +90,7 @@ char **argv;
 
     conffile = stralloc2(config_dir, CONFFILE_NAME);
     if(read_conffile(conffile)) {
-        error("could not read config file \"%s\"", conffile);
+        error("errors processing config file \"%s\"", conffile);
     }
     amfree(conffile);
 
@@ -109,6 +106,7 @@ char **argv;
     if((logfile = fopen(logfname, "r")) == NULL) {
 	error("could not open log %s: %s", logfname, strerror(errno));
     }
+    amfree(logfname);
 
     erroutput_type |= ERR_AMANDALOG;
     set_logerror(logerror);
