@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amtape.c,v 1.22.2.8 2001/03/26 22:23:36 jrjackson Exp $
+ * $Id: amtape.c,v 1.22.2.6.4.5.2.4 2003/11/25 12:21:08 martinea Exp $
  *
  * tape changer interface program
  */
@@ -58,9 +58,6 @@ int taperscan_slot P((int rc, char *slotstr, char *device));
 int update_one_slot P((int rc, char *slotstr, char *device));
 void update_labeldb P((int argc, char **argv));
 
-char *config_name = NULL;
-char *config_dir = NULL;
-
 void usage()
 {
     fprintf(stderr, "Usage: amtape%s <conf> <command>\n", versionsuffix());
@@ -74,6 +71,7 @@ void usage()
     fprintf(stderr, "\t\tslot current         load tape from current slot\n");
     fprintf(stderr, "\t\tslot prev            load tape from previous slot\n");
     fprintf(stderr, "\t\tslot next            load tape from next slot\n");
+    fprintf(stderr, "\t\tslot advance         advance to next slot but do not load\n");
     fprintf(stderr, "\t\tslot first           load tape from first slot\n");
     fprintf(stderr, "\t\tslot last            load tape from last slot\n");
     fprintf(stderr, "\t\tlabel <label>        find and load labeled tape\n");
@@ -126,7 +124,7 @@ char **argv;
     config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
     conffile = stralloc2(config_dir, CONFFILE_NAME);
     if (read_conffile(conffile)) {
-	error("could not read config file \"%s\"", conffile);
+	error("errors processing config file \"%s\"", conffile);
     }
     conf_tapelist = getconf_str(CNF_TAPELIST);
     if (*conf_tapelist == '/') {
@@ -324,7 +322,8 @@ char *device;
     else {
 	fprintf(stderr, "%s: slot %s: date %-8s label %s",
 		get_pname(), slotstr, datestamp, label);
-	if(strcmp(label, searchlabel))
+	if(strcmp(label, FAKE_LABEL) != 0
+	   && strcmp(label, searchlabel) != 0)
 	    fprintf(stderr, " (wrong tape)\n");
 	else {
 	    fprintf(stderr, " (exact label match)\n");
@@ -496,7 +495,9 @@ char *device;
 	    /* got an amanda tape */
 	    fprintf(stderr, "%s: slot %s: date %-8s label %s",
 		    get_pname(), slotstr, datestamp, label);
-	    if(searchlabel != NULL && strcmp(label, searchlabel) == 0) {
+	    if(searchlabel != NULL
+	       && (strcmp(label, FAKE_LABEL) == 0
+		   || strcmp(label, searchlabel) == 0)) {
 		/* it's the one we are looking for, stop here */
 		fprintf(stderr, " (exact label match)\n");
 		found = 1;
@@ -510,8 +511,20 @@ char *device;
 		/* not an exact label match, but a labelstr match */
 		/* check against tape list */
 		tp = lookup_tapelabel(label);
-		if(tp != NULL && !reusable_tape(tp))
+		if(tp == NULL)
+		    fprintf(stderr, " (not in tapelist)\n");
+		else if(!reusable_tape(tp))
 		    fprintf(stderr, " (active tape)\n");
+		else if(got_match == 0 && tp->datestamp == 0) {
+		    got_match = 1;
+		    first_match = newstralloc(first_match, slotstr);
+		    first_match_label = newstralloc(first_match_label, label);
+		    fprintf(stderr, " (new tape)\n");
+		    found = 3;
+		    amfree(datestamp);
+		    amfree(label);
+		    return 1;
+		}
 		else if(got_match)
 		    fprintf(stderr, " (labelstr match)\n");
 		else {
@@ -555,11 +568,15 @@ char **argv;
     fprintf(stderr, "a new tape.\n");
 
     if (searchlabel != NULL)
-      changer_find(scan_init, taperscan_slot,searchlabel);
+      changer_find(scan_init, taperscan_slot, searchlabel);
     else
       changer_scan(scan_init, taperscan_slot);
 
-    if(found == 2) {
+    if(found == 3) {
+	fprintf(stderr, "%s: settling for new tape\n", get_pname());
+	searchlabel = newstralloc(searchlabel, first_match_label);
+    }
+    else if(found == 2) {
 	fprintf(stderr, "%s: %s: settling for first labelstr match\n",
 		get_pname(),
 		searchlabel? "gravity stacker": "looking only for new tape");
@@ -606,4 +623,6 @@ char **argv;
 	error("Could not load current slot.\n");
 
     printf("%s\n", device);
+    amfree(slot);
+    amfree(device);
 }
