@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amtrmidx.c,v 1.21.4.1 1999/09/08 23:27:44 jrj Exp $
+ * $Id: amtrmidx.c,v 1.21.4.1.4.2.2.5 2003/10/27 18:33:03 martinea Exp $
  *
  * trims number of index files to only those still in system.  Well
  * actually, it keeps a few extra, plus goes back to the last level 0
@@ -42,9 +42,6 @@
 #include "find.h"
 #include "version.h"
 
-char *config_name = NULL;
-char *config_dir = NULL;
-
 static int sort_by_name_reversed(a, b)
 const void *a;
 const void *b;
@@ -61,10 +58,7 @@ char **argv;
 {
     disk_t *diskp;
     disklist_t *diskl;
-    int no_keep;			/* files per system to keep */
     int i;
-    int level_position;			/* where (from end) is level in name */
-    int datestamp_position;
     char *conffile;
     char *conf_diskfile;
     char *conf_tapelist;
@@ -107,7 +101,7 @@ char **argv;
     config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
     conffile = stralloc2(config_dir, CONFFILE_NAME);
     if(read_conffile(conffile)) {
-	error("could not find config file \"%s\"", conffile);
+	error("errors processing config file \"%s\"", conffile);
     }
     amfree(conffile);
     conf_diskfile = getconf_str(CNF_DISKFILE);
@@ -131,7 +125,7 @@ char **argv;
     }
     amfree(conf_tapelist);
 
-    output_find = find_dump(NULL,0,NULL);
+    output_find = find_dump(1, diskl);
 
     conf_indexdir = getconf_str(CNF_INDEXDIR);
     if(*conf_indexdir == '/') {
@@ -140,13 +134,6 @@ char **argv;
 	conf_indexdir = stralloc2(config_dir, conf_indexdir);
     }
 
-    /* determine how many indices to keep */
-    no_keep = getconf_int(CNF_TAPECYCLE) + 1;
-    dbprintf(("Keeping %d index file%s\n", no_keep, (no_keep == 1) ? "" : "s"));
-
-    level_position = strlen(COMPRESS_SUFFIX);
-    datestamp_position = level_position + 9;
-
     /* now go through the list of disks and find which have indexes */
     time(&tmp_time);
     tmp_time -= 7*24*60*60;			/* back one week */
@@ -158,8 +145,6 @@ char **argv;
 	    char **names;
 	    int name_length;
 	    int name_count;
-	    char *path;
-	    int deleting;
 	    char *host;
 	    char *disk;
 
@@ -175,7 +160,9 @@ char **argv;
 	    amfree(host);
 	    amfree(disk);
 	    if((d = opendir(indexdir)) == NULL) {
-		error("could not open index directory \"%s\"", indexdir);
+		dbprintf(("could not open index directory \"%s\"", indexdir));
+		amfree(indexdir);
+		continue;
 	    }
 	    name_length = 100;
 	    names = (char **)alloc(name_length * sizeof(char *));
@@ -203,6 +190,7 @@ char **argv;
 		if(l > sizeof("YYYYMMDD_L")-1
 		    && strcmp (f->d_name + l, ".tmp") == 0) {
 		    struct stat sbuf;
+		    char *path;
 
 		    path = stralloc2(indexdir, f->d_name);
 		    if(lstat(path, &sbuf) != -1
@@ -235,17 +223,12 @@ char **argv;
 	     * Search for the first full dump past the minimum number
 	     * of index files to keep.
 	     */
-	    deleting = 0;
 	    for(i = 0; i < name_count; i++) {
-		if(i < no_keep) {
-		} else if(! deleting
-			  && names[i][sizeof("YYYYMMDD_L")-1-1] == '0') {
-		    deleting = 1;
-		} else if(deleting
-			  && !dump_exist(output_find,
+		if(!dump_exist(output_find,
 					 diskp->host->hostname,diskp->name,
 					 atoi(names[i]),
 					 names[i][sizeof("YYYYMMDD_L")-1-1] - '0')) {
+		    char *path;
 		    path = stralloc2(indexdir, names[i]);
 		    dbprintf(("rm %s\n", path));
 		    if(amtrmidx_debug == 0 && unlink(path) == -1) {
@@ -257,11 +240,13 @@ char **argv;
 		amfree(names[i]);
 	    }
 	    amfree(names);
+	    amfree(indexdir);
 	}
     }
 
     amfree(conf_indexdir);
     amfree(config_dir);
+    free_find_result(&output_find);
 
     dbclose();
 
