@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amtrmlog.c,v 1.1.2.3 2000/09/23 00:28:09 martinea Exp $
+ * $Id: amtrmlog.c,v 1.1.2.3.4.1.2.2 2003/01/01 23:28:53 martinea Exp $
  *
  * trims number of index files to only those still in system.  Well
  * actually, it keeps a few extra, plus goes back to the last level 0
@@ -42,9 +42,6 @@
 #include "find.h"
 #include "version.h"
 
-char *config_name = NULL;
-char *config_dir = NULL;
-
 int main(argc, argv)
 int argc;
 char **argv;
@@ -102,7 +99,7 @@ char **argv;
     config_dir = vstralloc(CONFIG_DIR, "/", config_name, "/", NULL);
     conffile = stralloc2(config_dir, CONFFILE_NAME);
     if(read_conffile(conffile)) {
-	error("could not find config file \"%s\"", conffile);
+	error("errors processing config file \"%s\"", conffile);
     }
     amfree(conffile);
     conf_diskfile = getconf_str(CNF_DISKFILE);
@@ -187,11 +184,16 @@ char **argv;
 	}
     }
     closedir(dir);
+    for (name = output_find_log; *name != NULL; name++) {
+	amfree(*name);
+    }
+    amfree(output_find_log);
     amfree(logname);
     amfree(oldfile);
     amfree(newfile);
     amfree(olddir);
     amfree(config_dir);
+    amfree(conf_logdir);
 
     dbclose();
 
