@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: diskfile.h,v 1.11.4.3 2000/10/10 21:33:23 martinea Exp $
+ * $Id: diskfile.h,v 1.11.4.3.4.1.2.9 2003/01/04 03:35:54 martinea Exp $
  *
  * interface for disklist file reading code
  */
@@ -34,6 +34,7 @@
 
 #include "amanda.h"
 #include "conffile.h"
+#include "amfeatures.h"
 
 typedef struct host_s {
     struct host_s *next;		/* next host */
@@ -44,6 +45,7 @@ typedef struct host_s {
     interface_t *netif;			/* network interface this host is on */
     time_t start_t;			/* start dump after this time */
     char *up;				/* generic user pointer */
+    am_feature_t *features;		/* feature set */
 } host_t;
 
 typedef struct disk_s {
@@ -53,29 +55,36 @@ typedef struct disk_s {
     host_t *host;			/* host list */
     struct disk_s *hostnext;
 
-    char *name;				/* device name for disk, eg "sd0g" */
+    char *name;				/* label name for disk */
+    char *device;			/* device name for disk, eg "sd0g" */
     char *dtype_name;			/* name of dump type   XXX shouldn't need this */
     char *program;			/* dump program, eg DUMP, GNUTAR */
-    char *exclude;			/* file exclude spec */
-    int exclude_list;			/* exclude list flag */
+    sl_t *exclude_file;			/* file exclude spec */
+    sl_t *exclude_list;			/* exclude list */
+    sl_t *include_file;			/* file include spec */
+    sl_t *include_list;			/* include list */
+    int exclude_optional;		/* exclude list are optional */
+    int include_optional;		/* include list are optional */
     long priority;			/* priority of disk */
     long dumpcycle;			/* days between fulls */
     long frequency;			/* XXX - not used */
     auth_t auth;			/* type of authentication (per system?) */
     int maxdumps;			/* max number of parallel dumps (per system) */
+    int maxpromoteday;			/* maximum of promote day */
     time_t start_t;			/* start this dump after this time */
     int strategy;			/* what dump strategy to use */
     int compress;			/* type of compression to use */
     float comprate[2];			/* default compression rates */
     /* flag options */
-    int record:1;			/* record dump in /etc/dumpdates ? */
-    int skip_incr:1;			/* incs done externally ? */
-    int skip_full:1;			/* fulls done externally ? */
-    int no_hold:1;			/* don't use holding disk ? */
-    int kencrypt:1;
-    int index:1;			/* produce an index ? */
+    unsigned int record:1;			/* record dump in /etc/dumpdates ? */
+    unsigned int skip_incr:1;			/* incs done externally ? */
+    unsigned int skip_full:1;			/* fulls done externally ? */
+    unsigned int no_hold:1;			/* don't use holding disk ? */
+    unsigned int kencrypt:1;
+    unsigned int index:1;			/* produce an index ? */
     int spindle;			/* spindle # - for parallel dumps */
     int inprogress;			/* being dumped now? */
+    int todo;
     void *up;				/* generic user pointer */
 } disk_t;
 
@@ -88,6 +97,7 @@ typedef struct disklist_s {
 
 disklist_t *read_diskfile P((char *filename));
 
+disk_t *add_disk P((char *hostname, char *diskname));
 host_t *lookup_host P((char *hostname));
 disk_t *lookup_disk P((char *hostname, char *diskname));
 
@@ -101,6 +111,8 @@ void remove_disk P((disklist_t *list, disk_t *disk));
 
 void dump_queue P((char *str, disklist_t q, int npr, FILE *f));
 
-char *optionstr P((disk_t *dp));
+char *optionstr P((disk_t *dp, am_feature_t *their_features, FILE *fdout));
+
+void match_disklist P((disklist_t *origqp, int sargc, char **sargv));
 
 #endif /* ! DISKFILE_H */
