@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: driverio.c,v 1.35.2.14 1999/11/12 00:16:27 oliva Exp $
+ * $Id: driverio.c,v 1.35.2.14.4.2.2.5 2003/01/01 23:28:54 martinea Exp $
  *
  * I/O-related functions for driver program
  */
@@ -36,22 +36,11 @@
 #include "infofile.h"
 #include "logfile.h"
 #include "token.h"
+#include "server_util.h"
 
 #define GLOBAL		/* the global variables defined here */
 #include "driverio.h"
 
-char *cmdstr[] = {
-    "BOGUS", "QUIT", "DONE",
-    "FILE-DUMP", "PORT-DUMP", "CONTINUE", "ABORT",	/* dumper cmds */
-    "FAILED", "TRY-AGAIN", "NO-ROOM", "RQ-MORE-DISK",
-    "ABORT-FINISHED",	/* dumper results */
-    "FATAL-TRY-AGAIN",
-    "START-TAPER", "FILE-WRITE", "PORT-WRITE",		/* taper cmds */
-    "PORT", "TAPE-ERROR", "TAPER-OK",			/* taper results */
-    NULL
-};
-
-
 void init_driverio()
 {
     dumper_t *dumper;
@@ -145,7 +134,11 @@ char *dumper_program;
 	aclose(fd[0]);
 	if(dup2(fd[1], 0) == -1 || dup2(fd[1], 1) == -1)
 	    error("%s dup2: %s", dumper->name, strerror(errno));
-	execle(dumper_program, "dumper", config_name, (char *)0, safe_env());
+	execle(dumper_program,
+	       dumper->name ? dumper->name : "dumper",
+	       config_name,
+	       (char *)0,
+	       safe_env());
 	error("exec %s (%s): %s", dumper_program,
 	      dumper->name, strerror(errno));
     default:	/* parent process */
@@ -176,7 +169,7 @@ int inparallel;
     }
 }
 
-tok_t getresult(fd, show, result_argc, result_argv, max_arg)
+cmd_t getresult(fd, show, result_argc, result_argv, max_arg)
 int fd;
 int show;
 int *result_argc;
@@ -184,7 +177,7 @@ char **result_argv;
 int max_arg;
 {
     int arg;
-    tok_t t;
+    cmd_t t;
     char *line;
 
     if((line = areads(fd)) == NULL) {
@@ -215,7 +208,7 @@ int max_arg;
 
     if(*result_argc < 1) return BOGUS;
 
-    for(t = BOGUS+1; t < LAST_TOK; t++)
+    for(t = (cmd_t)(BOGUS+1); t < LAST_TOK; t++)
 	if(strcmp(result_argv[1], cmdstr[t]) == 0) return t;
 
     return BOGUS;
@@ -223,7 +216,7 @@ int max_arg;
 
 
 int taper_cmd(cmd, /* optional */ ptr, destname, level, datestamp)
-tok_t cmd;
+cmd_t cmd;
 void *ptr;
 char *destname;
 int level;
@@ -233,39 +226,46 @@ char *datestamp;
     char number[NUM_STR_SIZE];
     disk_t *dp;
     int l, n, s;
+    char *features;
 
     switch(cmd) {
     case START_TAPER:
-	cmdline = vstralloc("START-TAPER ", (char *)ptr, "\n", NULL);
+	cmdline = vstralloc(cmdstr[cmd], " ", (char *)ptr, "\n", NULL);
 	break;
     case FILE_WRITE:
 	dp = (disk_t *) ptr;
 	ap_snprintf(number, sizeof(number), "%d", level);
-	cmdline = vstralloc("FILE-WRITE",
+	features = am_feature_to_string(dp->host->features);
+	cmdline = vstralloc(cmdstr[cmd],
 			    " ", disk2serial(dp),
 			    " ", destname,
 			    " ", dp->host->hostname,
+			    " ", features,
 			    " ", dp->name,
 			    " ", number,
 			    " ", datestamp,
 			    "\n", NULL);
+	amfree(features);
 	break;
     case PORT_WRITE:
 	dp = (disk_t *) ptr;
 	ap_snprintf(number, sizeof(number), "%d", level);
-	cmdline = vstralloc("PORT-WRITE",
+	features = am_feature_to_string(dp->host->features);
+	cmdline = vstralloc(cmdstr[cmd],
 			    " ", disk2serial(dp),
 			    " ", dp->host->hostname,
+			    " ", features,
 			    " ", dp->name,
 			    " ", number,
 			    " ", datestamp,
 			    "\n", NULL);
+	amfree(features);
 	break;
     case QUIT:
-	cmdline = stralloc("QUIT\n");
+	cmdline = stralloc2(cmdstr[cmd], "\n");
 	break;
     default:
-	assert(0);
+	error("Don't know how to send %s command to taper", cmdstr[cmd]);
     }
     /*
      * Note: cmdline already has a '\n'.
@@ -277,6 +277,7 @@ char *datestamp;
 	if((s = write(taper, cmdline + l, n - l)) < 0) {
 	    printf("writing taper command: %s\n", strerror(errno));
 	    fflush(stdout);
+	    amfree(cmdline);
 	    return 0;
 	}
     }
@@ -286,7 +287,7 @@ char *datestamp;
 
 int dumper_cmd(dumper, cmd, /* optional */ dp)
 dumper_t *dumper;
-tok_t cmd;
+cmd_t cmd;
 disk_t *dp;
 {
     char *cmdline = NULL;
@@ -297,24 +298,36 @@ disk_t *dp;
     char *o;
     int activehd=0;
     assignedhd_t **h=NULL;
+    char *device;
+    char *features;
 
     if(dp && sched(dp) && sched(dp)->holdp) {
 	h = sched(dp)->holdp;
 	activehd = sched(dp)->activehd;
     }
 
+    if(dp && dp->device) {
+	device = dp->device;
+    }
+    else {
+	device = "NODEVICE";
+    }
+
     switch(cmd) {
     case FILE_DUMP:
 	holdalloc(h[activehd]->disk)->allocated_dumpers++;
 	ap_snprintf(number, sizeof(number), "%d", sched(dp)->level);
 	ap_snprintf(chunksize, sizeof(chunksize), "%ld", h[0]->disk->chunksize);
 	ap_snprintf(use, sizeof(use), "%ld", h[0]->reserved );
-	o = optionstr(dp);
+	features = am_feature_to_string(dp->host->features);
+	o = optionstr(dp, dp->host->features, NULL);
 	cmdline = vstralloc(cmdstr[cmd],
 			    " ", disk2serial(dp),
 			    " ", sched(dp)->destname,
 			    " ", dp->host->hostname,
+			    " ", features,
 			    " ", dp->name,
+			    " ", device,
 			    " ", number,
 			    " ", sched(dp)->dumpdate,
 			    " ", chunksize,
@@ -322,21 +335,26 @@ disk_t *dp;
 			    " ", use,
 			    " |", o,
 			    "\n", NULL);
+	amfree(features);
 	amfree(o);
 	break;
     case PORT_DUMP:
 	ap_snprintf(number, sizeof(number), "%d", sched(dp)->level);
-	o = optionstr(dp);
+	features = am_feature_to_string(dp->host->features);
+	o = optionstr(dp, dp->host->features, NULL);
 	cmdline = vstralloc(cmdstr[cmd],
 			    " ", disk2serial(dp),
 			    " ", sched(dp)->destname,
 			    " ", dp->host->hostname,
+			    " ", features,
 			    " ", dp->name,
+			    " ", device,
 			    " ", number,
 			    " ", sched(dp)->dumpdate,
 			    " ", dp->program,
 			    " |", o,
 			    "\n", NULL);
+	amfree(features);
 	amfree(o);
 	break;
     case QUIT:
@@ -366,7 +384,7 @@ disk_t *dp;
 	}
 	break;
     default:
-	assert(0);
+	error("Don't know how to send %s command to dumper", cmdstr[cmd]);
     }
     /*
      * Note: cmdline already has a '\n'.
@@ -383,6 +401,7 @@ disk_t *dp;
 		printf("writing %s command: %s\n", dumper->name, 
 		       strerror(errno));
 		fflush(stdout);
+		amfree(cmdline);
 		return 0;
 	    }
 	}
