@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: find.c,v 1.6.2.4 1999/09/08 23:28:11 jrj Exp $
+ * $Id: find.c,v 1.6.2.4.4.2.2.5 2003/10/27 18:33:03 martinea Exp $
  *
  * controlling process for the Amanda backup system
  */
@@ -43,30 +43,20 @@ void search_holding_disk P((find_result_t **output_find));
 char *find_nicedate P((int datestamp));
 
 static char *find_sort_order = NULL;
+int dynamic_disklist = 0;
+disklist_t* find_diskqp = NULL;
 
-static int find_nhosts;
-static char *find_hostname;
-static int find_ndisks;
-static char **find_diskstrs;
-
-find_result_t *find_dump(lfind_hostname, lfind_ndisks, lfind_diskstrs)
-char *lfind_hostname;
-int lfind_ndisks;
-char **lfind_diskstrs;
+find_result_t *find_dump(dyna_disklist, diskqp)
+int dyna_disklist;
+disklist_t* diskqp;
 {
     char *conf_logdir, *logfile = NULL;
     int tape, maxtape, seq, logs;
     tape_t *tp;
     find_result_t *output_find = NULL;
 
-    find_hostname=lfind_hostname;
-    find_ndisks=lfind_ndisks;
-    find_diskstrs=lfind_diskstrs;
-    if(find_hostname == NULL)
-	find_nhosts=0;
-    else
-	find_nhosts=1;
-
+    dynamic_disklist = dyna_disklist;
+    find_diskqp = diskqp;
     conf_logdir = getconf_str(CNF_LOGDIR);
     if (*conf_logdir == '/') {
 	conf_logdir = stralloc(conf_logdir);
@@ -112,11 +102,12 @@ char **lfind_diskstrs;
 	if(access(logfile,R_OK) == 0) {
 	    logs += search_logfile(&output_find, tp->label, tp->datestamp, -1, logfile);
 	}
-	if(logs == 0)
+	if(logs == 0 && tp->datestamp != 0)
 	    printf("Warning: no log files found for tape %s written %s\n",
 		   tp->label, find_nicedate(tp->datestamp));
     }
     amfree(logfile);
+    amfree(conf_logdir);
 
     search_holding_disk(&output_find);
     return(output_find);
@@ -191,11 +182,12 @@ char **find_log()
 		logs++;
 	    }
 	}
-	if(logs == 0)
+	if(logs == 0 && tp->datestamp != 0)
 	    printf("Warning: no log files found for tape %s written %s\n",
 		   tp->label, find_nicedate(tp->datestamp));
     }
     amfree(logfile);
+    amfree(conf_logdir);
     *current_log = NULL;
     return(output_find_log);
 }
@@ -204,7 +196,8 @@ void search_holding_disk(output_find)
 find_result_t **output_find;
 {
     holdingdisk_t *hdisk;
-    holding_t     *holding_list, *dir;
+    sl_t  *holding_list;
+    sle_t *dir;
     char *sdirname = NULL;
     char *destname = NULL;
     char *hostname = NULL;
@@ -214,10 +207,10 @@ find_result_t **output_find;
     int level;
     disk_t *dp;
 
-    holding_list = pick_all_datestamp();
+    holding_list = pick_all_datestamp(1);
 
     for(hdisk = getconf_holdingdisks(); hdisk != NULL; hdisk = hdisk->next) {
-	for(dir = holding_list; dir != NULL; dir = dir->next) {
+	for(dir = holding_list->first; dir != NULL; dir = dir->next) {
 	    sdirname = newvstralloc(sdirname,
 				    hdisk->diskdir, "/", dir->name,
 				    NULL);
@@ -240,6 +233,9 @@ find_result_t **output_find;
 		if(get_amanda_names(destname, &hostname, &diskname, &level) != F_DUMPFILE) {
 		    continue;
 		}
+		if(level < 0 || level > 9)
+		    continue;
+
 		dp = NULL;
 		for(;;) {
 		    char *s;
@@ -249,10 +245,9 @@ find_result_t **output_find;
            		break;
 	            *s = '\0';
 		}
-		if ( dp == NULL )
-		    continue;
-		if(level < 0 || level > 9)
+		if ( dp == NULL ) {
 		    continue;
+		}
 
 		if(find_match(hostname,diskname)) {
 		    find_result_t *new_output_find =
@@ -274,6 +269,8 @@ find_result_t **output_find;
 	    closedir(workdir);
 	}	
     }
+    free_sl(holding_list);
+    holding_list = NULL;
     amfree(destname);
     amfree(sdirname);
     amfree(hostname);
@@ -364,7 +361,7 @@ find_result_t **output_find;
     }
     array_find_result[nb_result-1]->next=NULL;
     *output_find=array_find_result[0];
-
+    amfree(array_find_result);
 }
 
 void print_find_result(output_find)
@@ -449,6 +446,7 @@ find_result_t **output_find;
 	amfree(output_find_result->diskname);
 	amfree(output_find_result->label);
 	amfree(output_find_result->status);
+	prev = output_find_result;
     }
     if(prev != NULL) amfree(prev);
     output_find = NULL;
@@ -457,17 +455,8 @@ find_result_t **output_find;
 int find_match(host, disk)
 char *host, *disk;
 {
-    int d;
-
-    if(find_nhosts == 0) return 1;
-    if(strcmp(host, find_hostname)) return 0;
-    if(find_ndisks == 0) return 1;
-
-    for(d = 0; d < find_ndisks; d++) {
-	if(match(find_diskstrs[d], disk))
-	    return 1;
-    }
-    return 0;
+    disk_t *dp = lookup_disk(host,disk);
+    return (dp && dp->todo);
 }
 
 char *find_nicedate(datestamp)
@@ -549,8 +538,8 @@ char *label, *logfile;
 int datestamp, datestamp_aux;
 {
     FILE *logf;
-    char *host, *host_undo, host_undo_ch;
-    char *disk, *disk_undo, disk_undo_ch;
+    char *host, *host_undo;
+    char *disk, *disk_undo;
     int   datestampI;
     char *rest;
     char *ck_label;
@@ -558,6 +547,7 @@ int datestamp, datestamp_aux;
     int passlabel, ck_datestamp2;
     char *s;
     int ch;
+    disk_t *dp;
 
     if((logf = fopen(logfile, "r")) == NULL)
 	error("could not open logfile %s: %s", logfile, strerror(errno));
@@ -613,7 +603,6 @@ int datestamp, datestamp_aux;
 	    host = s - 1;
 	    skip_non_whitespace(s, ch);
 	    host_undo = s - 1;
-	    host_undo_ch = *host_undo;
 	    *host_undo = '\0';
 
 	    skip_whitespace(s, ch);
@@ -624,7 +613,6 @@ int datestamp, datestamp_aux;
 	    disk = s - 1;
 	    skip_non_whitespace(s, ch);
 	    disk_undo = s - 1;
-	    disk_undo_ch = *disk_undo;
 	    *disk_undo = '\0';
 
 	    skip_whitespace(s, ch);
@@ -657,6 +645,14 @@ int datestamp, datestamp_aux;
 		*s = '\0';
 	    }
 
+	    dp = lookup_disk(host,disk);
+	    if ( dp == NULL ) {
+		if (dynamic_disklist == 0) {
+		    continue;
+		}
+		dp = add_disk(host, disk);
+		enqueue_disk(find_diskqp , dp);
+	    }
 	    if(find_match(host, disk)) {
 		if(curprog == P_TAPER) {
 		    find_result_t *new_output_find =
@@ -677,7 +673,7 @@ int datestamp, datestamp_aux;
 		}
 		else if(curlog == L_FAIL) {	/* print other failures too */
 		    find_result_t *new_output_find =
-			alloc(sizeof(find_result_t));
+			(find_result_t *)alloc(sizeof(find_result_t));
 		    new_output_find->next=*output_find;
 		    new_output_find->datestamp=datestamp;
 		    new_output_find->datestamp_aux=datestamp_aux;
