@@ -24,15 +24,19 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: holding.c,v 1.17.2.12 2000/06/02 14:31:47 martinea Exp $
+ * $Id: holding.c,v 1.17.2.12.4.3.2.10 2003/06/14 13:47:05 martinea Exp $
  *
  * Functions to access holding disk
  */
 
 #include "amanda.h"
+#include "util.h"
 #include "holding.h"
 #include "fileheader.h"
+#include "util.h"
+#include "logfile.h"
 
+static sl_t *scan_holdingdisk P((sl_t *holding_list, char *diskdir, int verbose));
 
 int is_dir(fname)
 char *fname;
@@ -105,43 +109,9 @@ char *fname;
     return gotentry;
 }
 
-#define MAX_DIRS 26	/* so we can select them A .. Z */
 
-holding_t *insert_dirname(holding_list, name)
-holding_t **holding_list;
-char *name;
-{
-    holding_t *d, *p, *n;
-    int cmp;
-
-    for(p = NULL, d = *holding_list; d != NULL; p = d, d = d->next)
-	if((cmp = strcmp(name, d->name)) > 0) continue;
-	else if(cmp == 0) return d;
-	else break;
-    n = (holding_t *)alloc(sizeof(holding_t));
-    n->name = stralloc(name);
-    n->next = d;
-    if(p) p->next = n;
-    else *holding_list = n;
-    return n;
-}
-
-
-void free_holding_list(holding_list)
-holding_t *holding_list;
-{
-    holding_t *p, *n;
-    p = holding_list;
-    while(p != NULL) {
-	n = p;
-	p = p->next;
-	amfree(n->name);
-	amfree(n);
-    }
-}
-
-void scan_holdingdisk(holding_list, diskdir, verbose)
-holding_t **holding_list;
+sl_t *scan_holdingdisk(holding_list, diskdir, verbose)
+sl_t *holding_list;
 char *diskdir;
 int verbose;
 {
@@ -150,16 +120,19 @@ int verbose;
     char *entryname = NULL;
 
     if((topdir = opendir(diskdir)) == NULL) {
-	printf("Warning: could not open holding dir %s: %s\n",
-	       diskdir, strerror(errno));
-	return;
+	if(verbose && errno != ENOENT)
+	   printf("Warning: could not open holding dir %s: %s\n",
+		  diskdir, strerror(errno));
+	return holding_list;
     }
 
     /* find all directories of the right format  */
 
-    printf("Scanning %s...\n", diskdir);
+    if(verbose)
+	printf("Scanning %s...\n", diskdir);
     while((workdir = readdir(topdir)) != NULL) {
-	if(is_dot_or_dotdot(workdir->d_name)) {
+	if(is_dot_or_dotdot(workdir->d_name)
+	   || strcmp(workdir->d_name, "lost+found") == 0) {
 	    continue;
 	}
 	entryname = newvstralloc(entryname,
@@ -168,132 +141,240 @@ int verbose;
 	    printf("  %s: ", workdir->d_name);
 	}
 	if(!is_dir(entryname)) {
-	    if(verbose) {
+	    if(verbose)
 	        puts("skipping cruft file, perhaps you should delete it.");
-	    }
 	} else if(!is_datestr(workdir->d_name)) {
-	    if(verbose) {
+	    if(verbose)
 	        puts("skipping cruft directory, perhaps you should delete it.");
-	    }
 	} else {
-	    if(insert_dirname(holding_list, workdir->d_name) == NULL) {
-	        if(verbose) {
-		    puts("too many non-empty Amanda dirs, can't handle this one.");
-		}
-	    } else {
-	        if(verbose) {
-		    puts("found Amanda directory.");
-		}
-	    }
+	    holding_list = insert_sort_sl(holding_list, workdir->d_name);
+	    if(verbose)
+		puts("found Amanda directory.");
 	}
     }
     closedir(topdir);
     amfree(entryname);
+    return holding_list;
 }
 
-holding_t *pick_all_datestamp()
-{
-    holdingdisk_t *hdisk;
-    holding_t *holding_list = NULL;
 
+sl_t *scan_holdingdir(holding_list, holdp, datestamp)
+sl_t *holding_list;
+holdingdisk_t *holdp;
+char *datestamp;
+{
+    DIR *workdir;
+    struct dirent *entry;
+    char *dirname = NULL;
+    char *destname = NULL;
+    disk_t *dp;
+    dumpfile_t file;
+
+    dirname = vstralloc(holdp->diskdir, "/", datestamp, NULL);
+    if((workdir = opendir(dirname)) == NULL) {
+	if(errno != ENOENT)
+	    log_add(L_INFO, "%s: could not open dir: %s",
+		    dirname, strerror(errno));
+	amfree(dirname);
+	return holding_list;
+    }
+    chdir(dirname);
+    while((entry = readdir(workdir)) != NULL) {
+	if(strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0)
+	    continue;
+
+	if(is_emptyfile(entry->d_name))
+	    continue;
+
+	destname = newvstralloc(destname,
+				dirname, "/", entry->d_name,
+				NULL);
+	get_dumpfile(destname, &file);
+	if( file.type != F_DUMPFILE) {
+	    if( file.type != F_CONT_DUMPFILE )
+		log_add(L_INFO, "%s: ignoring cruft file.", entry->d_name);
+	    continue;
+	}
+
+	dp = lookup_disk(file.name, file.disk);
+
+	if (dp == NULL) {
+	    log_add(L_INFO, "%s: disk %s:%s not in database, skipping it.",
+		    entry->d_name, file.name, file.disk);
+	    continue;
+	}
+
+	if(file.dumplevel < 0 || file.dumplevel > 9) {
+	    log_add(L_INFO, "%s: ignoring file with bogus dump level %d.",
+		    entry->d_name, file.dumplevel);
+	    continue;
+	}
+
+	holding_list = append_sl(holding_list, destname);
+    }
+    closedir(workdir);
+    amfree(dirname);
+    amfree(destname);
+    return holding_list;
+}
+
+
+sl_t *get_flush(dateargs, datestamp, amflush, verbose)
+sl_t *dateargs;
+char *datestamp;  /* don't do this date */
+int amflush, verbose;
+{
+    sl_t *holding_list;
+    sl_t *date_list;
+    sle_t *datearg;
+    sle_t *date, *next_date;
+    holdingdisk_t *hdisk;
+    char current_dir[1000];
+
+    getcwd(current_dir, 999);
+
+    holding_list = new_sl();
+
+    if(dateargs) {
+	int ok;
+
+	date_list = pick_all_datestamp(verbose);
+	for(date = date_list->first; date != NULL;) {
+	    next_date = date->next;
+	    ok = 0;
+	    for(datearg=dateargs->first; datearg != NULL && ok==0;
+		datearg = datearg->next) {
+		ok = match_datestamp(datearg->name, date->name);
+	    }
+	    if(ok == 0) { /* remove dir */
+		remove_sl(date_list, date);
+	    }
+	    date = next_date;
+	}
+    }
+    else if (amflush) {
+	date_list = pick_datestamp(verbose);
+    }
+    else {
+	date_list = pick_all_datestamp(verbose);
+    }
+
+    for(date = date_list->first; date !=NULL; date = date->next) {
+	if(!datestamp || strcmp(datestamp,date->name) != 0) {
+	    for(hdisk = getconf_holdingdisks(); hdisk != NULL;
+						hdisk = hdisk->next) {
+		holding_list = scan_holdingdir(holding_list, hdisk, date->name);
+	    }
+	}
+    }
+
+    free_sl(date_list);
+    date_list = NULL;
+    chdir(current_dir);
+    return(holding_list);
+}
+
+
+sl_t *pick_all_datestamp(verbose)
+int verbose;
+{
+    sl_t *holding_list = NULL;
+    holdingdisk_t *hdisk;
+
+    holding_list = new_sl();
     for(hdisk = getconf_holdingdisks(); hdisk != NULL; hdisk = hdisk->next)
-	scan_holdingdisk(&holding_list, hdisk->diskdir, 1);
+	holding_list = scan_holdingdisk(holding_list, hdisk->diskdir, verbose);
 
     return holding_list;
 }
 
 
-holding_t *pick_datestamp()
+sl_t *pick_datestamp(verbose)
+int verbose;
 {
-    holding_t *holding_list;
-    holding_t *dir, **directories;
+    sl_t *holding_list;
+    sl_t *r_holding_list = NULL;
+    sle_t *dir;
+    char **directories = NULL;
     int i;
-    int ndirs;
-    char answer[1024], *result;
-    char max_char = '\0', *ch, chupper = '\0';
+    char *answer = NULL;
+    char *a;
+    int ch;
+    char max_char = '\0', chupper = '\0';
 
-    holding_list = pick_all_datestamp();
+    holding_list = pick_all_datestamp(verbose);
 
-    ndirs=0;
-    for(dir = holding_list; dir != NULL;
-	dir = dir->next) {
-	ndirs++;
-    }
-
-    if(ndirs == 0) {
+    if(holding_list->nb_element == 0) {
 	return holding_list;
     }
-    else if(ndirs == 1) {
+    else if(holding_list->nb_element == 1 || !verbose) {
 	return holding_list;
     }
     else {
-	directories = alloc((ndirs) * sizeof(holding_t *));
-	for(dir = holding_list, i=0; dir != NULL; dir = dir->next,i++) {
-	    directories[i] = dir;
+	directories = alloc((holding_list->nb_element) * sizeof(char *));
+	for(dir = holding_list->first, i=0; dir != NULL; dir = dir->next,i++) {
+	    directories[i] = dir->name;
 	}
 
 	while(1) {
 	    puts("\nMultiple Amanda directories, please pick one by letter:");
-	    for(dir = holding_list, i = 0; dir != NULL && i < 26; dir = dir->next, i++) {
-		printf("  %c. %s\n", 'A'+i, dir->name);
-		max_char = 'A'+i;
+	    for(dir = holding_list->first, max_char = 'A';
+		dir != NULL && max_char <= 'Z';
+		dir = dir->next, max_char++) {
+		printf("  %c. %s\n", max_char, dir->name);
 	    }
-	    printf("Select directories to flush [A..%c]: [ALL] ", 'A' + i - 1);
-	    result = fgets(answer, sizeof(answer), stdin);
-	    if(strlen(answer) == 1 || !strncasecmp(answer,"ALL",3)) {
-		amfree(directories);
-		return(holding_list);
+	    max_char--;
+	    printf("Select directories to flush [A..%c]: [ALL] ", max_char);
+	    fflush(stdout); fflush(stderr);
+	    amfree(answer);
+	    if ((answer = agets(stdin)) == NULL) {
+		clearerr(stdin);
+		continue;
 	    }
-	    else {
-		i=1;
-		for(ch = answer; *ch != '\0'; ch++) {
-		    chupper = toupper(*ch);
-		    if(!((chupper >= 'A' && chupper <= max_char) ||
-			 chupper == ' ' || chupper != ',' || chupper != '\n'))
-			i=0;
+	    a = answer;
+	    while ((ch = *a++) != '\0' && isspace(ch)) {}
+	    if(ch == '\0' || strncasecmp(a, "ALL", 3) == 0) {
+		break;
+	    }
+	    do {
+		if (isspace(ch) || ch == ',') {
+		    continue;
 		}
-		if(i==1) {
-		    holding_t *r_holding_list, *hlist, *p;
-		    r_holding_list = p = NULL;
-		    for(ch = answer; *ch != '\0'; ch++) {
-			chupper = toupper(*ch);
-			if(chupper >= 'A' && chupper <= max_char) {
-			    hlist = malloc(sizeof(holding_t));
-			    hlist->next = NULL;
-			    hlist->name = stralloc(directories[chupper-'A']->name);
-			    if(p == NULL)
-				r_holding_list = p = hlist;
-			    else {
-				p->next = hlist;
-				p = hlist;
-			    }
-			}
-		    }
-		    amfree(directories);
-		    free_holding_list(holding_list);
-		    return(r_holding_list);
+		chupper = toupper(ch);
+		if (chupper < 'A' || chupper > max_char) {
+		    free_sl(r_holding_list);
+		    r_holding_list = NULL;
+		    break;
 		}
+		r_holding_list = append_sl(r_holding_list,
+					   directories[chupper - 'A']);
+	    } while ((ch = *a++) != '\0');
+	    if (r_holding_list && ch == '\0') {
+		free_sl(holding_list);
+		holding_list = r_holding_list;
+		break;
 	    }
-
 	}
     }
-
-    return NULL;
+    amfree(directories);
+    amfree(answer);
+    return holding_list;
 }
 
+
 filetype_t get_amanda_names(fname, hostname, diskname, level)
 char *fname, **hostname, **diskname;
 int *level;
 {
     dumpfile_t file;
-    char buffer[TAPE_BLOCK_BYTES];
+    char buffer[DISK_BLOCK_BYTES];
     int fd;
     *hostname = *diskname = NULL;
 
     if((fd = open(fname, O_RDONLY)) == -1)
 	return F_UNKNOWN;
 
-    if(read(fd, buffer, sizeof(buffer)) != sizeof(buffer)) {
+    if(fullread(fd, buffer, sizeof(buffer)) != sizeof(buffer)) {
 	aclose(fd);
 	return F_UNKNOWN;
     }
@@ -315,7 +396,7 @@ void get_dumpfile(fname, file)
 char *fname;
 dumpfile_t *file;
 {
-    char buffer[TAPE_BLOCK_BYTES];
+    char buffer[DISK_BLOCK_BYTES];
     int fd;
 
     fh_init(file);
@@ -323,7 +404,7 @@ dumpfile_t *file;
     if((fd = open(fname, O_RDONLY)) == -1)
 	return;
 
-    if(read(fd, buffer, sizeof(buffer)) != sizeof(buffer)) {
+    if(fullread(fd, buffer, sizeof(buffer)) != sizeof(buffer)) {
 	aclose(fd);
 	return;
     }
@@ -339,7 +420,7 @@ char *holding_file;
 {
     int fd;
     int buflen;
-    char buffer[TAPE_BLOCK_BYTES];
+    char buffer[DISK_BLOCK_BYTES];
     dumpfile_t file;
     char *filename;
     long size=0;
@@ -352,7 +433,7 @@ char *holding_file;
 	    amfree(filename);
 	    return -1;
 	}
-	buflen=fill_buffer(fd, buffer, sizeof(buffer));
+	buflen = fullread(fd, buffer, sizeof(buffer));
 	parse_file_header(buffer, &file, buflen);
 	close(fd);
 	if(stat(filename, &finfo) == -1) {
@@ -372,7 +453,7 @@ char *holding_file;
 {
     int fd;
     int buflen;
-    char buffer[TAPE_BLOCK_BYTES];
+    char buffer[DISK_BLOCK_BYTES];
     dumpfile_t file;
     char *filename;
 
@@ -383,7 +464,7 @@ char *holding_file;
 	    amfree(filename);
 	    return 0;
 	}
-	buflen=fill_buffer(fd, buffer, sizeof(buffer));
+	buflen = fullread(fd, buffer, sizeof(buffer));
 	parse_file_header(buffer, &file, buflen);
 	close(fd);
 	unlink(filename);
@@ -400,7 +481,7 @@ int complete;
 {
     int fd;
     int buflen;
-    char buffer[TAPE_BLOCK_BYTES];
+    char buffer[DISK_BLOCK_BYTES];
     dumpfile_t file;
     char *filename;
     char *filename_tmp = NULL;
@@ -414,7 +495,14 @@ int complete;
 	    amfree(filename_tmp);
 	    return 0;
 	}
-	buflen=fill_buffer(fd, buffer, sizeof(buffer));
+	buflen = fullread(fd, buffer, sizeof(buffer));
+	if (buflen == 0) {
+	    fprintf(stderr,"rename_tmp_holding: %s: empty file?\n", filename_tmp);
+	    amfree(filename);
+	    amfree(filename_tmp);
+	    close(fd);
+	    return 0;
+	}
 	parse_file_header(buffer, &file, buflen);
 	close(fd);
 	if(complete == 0 ) {
@@ -427,8 +515,8 @@ int complete;
 
 	    }
 	    file.is_partial = 1;
-	    write_header(buffer, &file, sizeof(buffer));
-	    write(fd, buffer, sizeof(buffer));
+	    build_header(buffer, &file, sizeof(buffer));
+	    fullwrite(fd, buffer, sizeof(buffer));
 	    close(fd);
 	}
 	if(rename(filename_tmp, filename) != 0) {
@@ -442,3 +530,83 @@ int complete;
     amfree(filename_tmp);
     return 1;
 }
+
+
+void cleanup_holdingdisk(diskdir, verbose)
+char *diskdir;
+int verbose;
+{
+    DIR *topdir;
+    struct dirent *workdir;
+
+    if((topdir = opendir(diskdir)) == NULL) {
+	if(verbose && errno != ENOENT)
+	    printf("Warning: could not open holding dir %s: %s\n",
+		   diskdir, strerror(errno));
+	return;
+   }
+
+    /* find all directories of the right format  */
+
+    if(verbose)
+	printf("Scanning %s...\n", diskdir);
+    chdir(diskdir);
+    while((workdir = readdir(topdir)) != NULL) {
+	if(is_dot_or_dotdot(workdir->d_name)
+	   || strcmp(workdir->d_name, "lost+found") == 0) {
+	    continue;
+	}
+	if(verbose)
+	    printf("  %s: ", workdir->d_name);
+	if(!is_dir(workdir->d_name)) {
+	    if(verbose)
+	        puts("skipping cruft file, perhaps you should delete it.");
+	}
+	else if(!is_datestr(workdir->d_name)) {
+	    if(verbose)
+	        puts("skipping cruft directory, perhaps you should delete it.");
+	}
+	else if(rmdir(workdir->d_name) == 0) {
+	    if(verbose)
+	        puts("deleted empty Amanda directory.");
+ 	}
+     }
+     closedir(topdir);
+}
+
+
+int mkholdingdir(diskdir)
+char *diskdir;
+{
+    struct stat stat_hdp;
+    int success = 1;
+
+    if (mkpdir(diskdir, 0770, (uid_t)-1, (gid_t)-1) != 0 && errno != EEXIST) {
+	log_add(L_WARNING, "WARNING: could not create parents of %s: %s",
+		diskdir, strerror(errno));
+	success = 0;
+    }
+    else if (mkdir(diskdir, 0770) != 0 && errno != EEXIST) {
+	log_add(L_WARNING, "WARNING: could not create %s: %s",
+		diskdir, strerror(errno));
+	success = 0;
+    }
+    else if (stat(diskdir, &stat_hdp) == -1) {
+	log_add(L_WARNING, "WARNING: could not stat %s: %s",
+		diskdir, strerror(errno));
+	success = 0;
+    }
+    else {
+	if (!S_ISDIR((stat_hdp.st_mode))) {
+	    log_add(L_WARNING, "WARNING: %s is not a directory",
+		    diskdir);
+	    success = 0;
+	}
+	else if (access(diskdir,W_OK) != 0) {
+	    log_add(L_WARNING, "WARNING: directory %s is not writable",
+		    diskdir);
+	    success = 0;
+	}
+    }
+    return success;
+}
