@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: logfile.c,v 1.17.4.1 1999/09/08 23:28:20 jrj Exp $
+ * $Id: logfile.c,v 1.17.4.1.4.2.2.3 2003/01/01 23:28:56 martinea Exp $
  *
  * common log file writing routine
  */
@@ -40,6 +40,7 @@ char *logtype_str[] = {
     "FATAL",		/* program died for some reason, used by error() */
     "ERROR", "WARNING",	"INFO", "SUMMARY",	 /* information messages */
     "START", "FINISH",				   /* start/end of a run */
+    "DISK",							 /* disk */
     "SUCCESS", "FAIL", "STRANGE",		    /* the end of a dump */
     "STATS",						   /* statistics */
     "MARKER",					  /* marker for reporter */
@@ -47,7 +48,7 @@ char *logtype_str[] = {
 };
 
 char *program_str[] = {
-    "UNKNOWN", "planner", "driver", "reporter", "dumper", "taper", "amflush"
+    "UNKNOWN", "planner", "driver", "amreport", "dumper", "taper", "amflush"
 };
 
 int curlinenum;
@@ -78,10 +79,9 @@ char *msg;
     log_add(L_FATAL, "%s", msg);
 }
 
-arglist_function(void log_add, logtype_t, typ)
+printf_arglist_function1(void log_add, logtype_t, typ, char *, format)
 {
     va_list argp;
-    char *format;
     int saved_errout;
     char *leader = NULL;
     char linebuf[STR_SIZE];
@@ -98,8 +98,7 @@ arglist_function(void log_add, logtype_t, typ)
 	leader = vstralloc(logtype_str[(int)typ], " ", get_pname(), " ", NULL);
     }
 
-    arglist_start(argp, typ);
-    format = arglist_val(argp, char *);
+    arglist_start(argp, format);
     ap_vsnprintf(linebuf, sizeof(linebuf)-1, format, argp);
 						/* -1 to allow for '\n' */
     arglist_end(argp);
@@ -191,6 +190,7 @@ char *datestamp;
 
     amfree(fname);
     amfree(logfile);
+    amfree(conf_logdir);
 }
 
 
@@ -205,6 +205,7 @@ static void open_log()
 	conf_logdir = stralloc2(config_dir, conf_logdir);
     }
     logfile = vstralloc(conf_logdir, "/log", NULL);
+    amfree(conf_logdir);
 
     logfd = open(logfile, O_WRONLY|O_CREAT|O_APPEND, 0600);
 
