@@ -23,7 +23,7 @@
  * Authors: the Amanda Development Team.  Its members are listed in a
  * file named AUTHORS, in the root directory of this distribution.
  */
-/* $Id: taper.c,v 1.47.2.16 2001/03/09 19:28:24 jrjackson Exp $
+/* $Id: taper.c,v 1.47.2.14.4.8.2.18 2004/02/13 14:11:26 martinea Exp $
  *
  * moves files from holding disk to tape, or from a socket to tape
  */
@@ -39,7 +39,9 @@
 #include "version.h"
 #include "arglist.h"
 #include "token.h"
+#include "amfeatures.h"
 #include "fileheader.h"
+#include "server_util.h"
 #ifdef HAVE_LIBVTBLC
 #include <vtblc.h>
 #include <strings.h>
@@ -64,7 +66,6 @@ static vtbl_lbls vtbl_entry[MAX_VOLUMES];
  * XXX advance to next tape first in next_tape
  * XXX label is being read twice?
  */
-#define MAX_ARGS	7
 
 /* NBUFS replaced by conf_tapebufs */
 /* #define NBUFS		20 */
@@ -77,15 +78,14 @@ int conf_tapebufs;
 
 
 
-typedef enum { BOGUS, START_TAPER, FILE_WRITE, PORT_WRITE, QUIT } cmd_t;
-
 #define EMPTY 1
 #define FILLING 2
 #define FULL 3
 
 typedef struct buffer_s {
     long status;
-    char buffer[TAPE_BLOCK_BYTES];
+    unsigned int size;
+    char *buffer;
 } buffer_t;
 
 #define nextbuf(p)    ((p) == buftable+conf_tapebufs-1? buftable : (p)+1)
@@ -96,14 +96,9 @@ int main P((int main_argc, char **main_argv));
 void file_reader_side P((int rdpipe, int wrpipe));
 void tape_writer_side P((int rdpipe, int wrpipe));
 
-/* misc helper functions */
-cmd_t getcmd P((int *argcp, char ***argvp));
-void putresult P((char *format, ...))
-    __attribute__ ((format (printf, 1, 2)));
-
 /* shared-memory routines */
-buffer_t *attach_buffers P((void));
-void detach_buffers P((buffer_t *bufp));
+char *attach_buffers P((unsigned int size));
+void detach_buffers P((char *bufp));
 void destroy_buffers P((void));
 
 /* synchronization pipe routines */
@@ -135,7 +130,8 @@ int bufdebug = 1;
 int bufdebug = 0;
 #endif
 
-buffer_t *buftable;
+char *buffers = NULL;
+buffer_t *buftable = NULL;
 
 char *procname = "parent";
 
@@ -145,10 +141,16 @@ int filenum;
 char *errstr = NULL;
 int tape_fd = -1;
 char *tapedev = NULL;
+char *tapetype = NULL;
+tapetype_t *tt = NULL;
+long tt_blocksize;
+long tt_blocksize_kb;
+long buffer_size;
+int tt_file_pad;
 static unsigned long malloc_hist_1, malloc_size_1;
 static unsigned long malloc_hist_2, malloc_size_2;
-char *config_name = NULL;
-char *config_dir = NULL;
+
+am_feature_t *their_features = NULL;
 
 int runtapes, cur_tape, have_changer, tapedays;
 char *labelstr, *conf_tapelist;
@@ -169,6 +171,11 @@ char **main_argv;
     int p2c[2], c2p[2];		/* parent-to-child, child-to-parent pipes */
     char *conffile;
     int fd;
+    unsigned int size;
+    int i;
+    int j;
+    int page_size;
+    char *first_buffer;
 
     for(fd = 3; fd < FD_SETSIZE; fd++) {
 	/*
@@ -219,7 +226,7 @@ char **main_argv;
 
     conffile = stralloc2(config_dir, CONFFILE_NAME);
     if(read_conffile(conffile)) {
-	error("could not find config file \"%s\"", conffile);
+	error("errors processing config file \"%s\"", conffile);
     }
     amfree(conffile);
 
@@ -234,6 +241,8 @@ char **main_argv;
     }
 
     tapedev	= getconf_str(CNF_TAPEDEV);
+    tapetype    = getconf_str(CNF_TAPETYPE);
+    tt		= lookup_tapetype(tapetype);
 #ifdef HAVE_LIBVTBLC
     rawtapedev = getconf_str(CNF_RAWTAPEDEV);
 #endif /* HAVE_LIBVTBLC */
@@ -243,6 +252,12 @@ char **main_argv;
     runtapes	= getconf_int(CNF_RUNTAPES);
     cur_tape	= 0;
 
+    conf_tapebufs = getconf_int(CNF_TAPEBUFS);
+
+    tt_blocksize_kb = tt->blocksize;
+    tt_blocksize = tt_blocksize_kb * 1024;
+    tt_file_pad = tt->file_pad;
+
     if(interactive) {
 	fprintf(stderr,"taper: running in interactive test mode\n");
 	fflush(stderr);
@@ -255,7 +270,64 @@ char **main_argv;
 
     /* create shared memory segment */
 
-    buftable = attach_buffers();
+#if defined(HAVE_GETPAGESIZE)
+    page_size = getpagesize();
+    fprintf(stderr, "%s: page size is %d\n", get_pname(), page_size);
+#else
+    page_size = 1024;
+    fprintf(stderr, "%s: getpagesize() not available, using %d\n",
+	    get_pname(),
+	    page_size);
+#endif
+    buffer_size = am_round(tt_blocksize, page_size);
+    fprintf(stderr, "%s: buffer size is %ld\n", get_pname(), buffer_size);
+    while(conf_tapebufs > 0) {
+	size  = page_size;
+	size += conf_tapebufs * buffer_size;
+	size += conf_tapebufs * sizeof(buffer_t);
+	if((buffers = attach_buffers(size)) != NULL) {
+	    break;
+	}
+	log_add(L_INFO, "attach_buffers: (%d tapebuf%s: %d bytes) %s",
+			conf_tapebufs,
+			(conf_tapebufs == 1) ? "" : "s",
+			size,
+			strerror(errno));
+	conf_tapebufs--;
+    }
+    if(buffers == NULL) {
+	error("cannot allocate shared memory");
+    }
+    i = (buffers - (char *)0) & (page_size - 1);  /* page boundary offset */
+    if(i != 0) {
+	first_buffer = buffers + page_size - i;
+	fprintf(stderr, "%s: shared memory at %p, first buffer at %p\n",
+		get_pname(),
+		buffers,
+		first_buffer);
+    } else {
+	first_buffer = buffers;
+    }
+    buftable = (buffer_t *)(first_buffer + conf_tapebufs * buffer_size);
+    memset(buftable, 0, conf_tapebufs * sizeof(buffer_t));
+    if(conf_tapebufs < 10) {
+	j = 1;
+    } else if(conf_tapebufs < 100) {
+	j = 2;
+    } else {
+	j = 3;
+    }
+    for(i = 0; i < conf_tapebufs; i++) {
+	buftable[i].buffer = first_buffer + i * buffer_size;
+	fprintf(stderr, "%s: buffer[%0*d] at %p\n",
+		get_pname(),
+		j, i,
+		buftable[i].buffer);
+    }
+    fprintf(stderr, "%s: buffer structures at %p for %d bytes\n",
+	    get_pname(),
+	    buftable,
+	    (int)(conf_tapebufs * sizeof(buffer_t)));
 
     /* fork off child writer process, parent becomes reader process */
 
@@ -291,7 +363,7 @@ char **main_argv;
 void read_file P((int fd, char *handle,
 		  char *host, char *disk, char *datestamp, 
 		  int level, int port_flag));
-int taper_fill_buffer P((int fd, char *buffer, int size));
+int taper_fill_buffer P((int fd, buffer_t *bp, int buflen));
 void dumpbufs P((char *str1));
 void dumpstatus P((buffer_t *bp));
 
@@ -299,15 +371,19 @@ void file_reader_side(rdpipe, wrpipe)
 int rdpipe, wrpipe;
 {
     cmd_t cmd;
-    int argc;
-    char **argv;
-    char *handle = NULL, *hostname = NULL, *diskname = NULL, *result;
+    struct cmdargs cmdargs;
+    char *handle = NULL;
+    char *filename = NULL;
+    char *hostname = NULL;
+    char *diskname = NULL;
+    char *result = NULL;
     char *datestamp = NULL;
     char tok;
-    char *q;
+    char *q = NULL;
     int level, fd, data_port, data_socket, wpid;
     struct stat stat_file;
     int tape_started;
+    int a;
 
     procname = "reader";
     syncpipe_init(rdpipe, wrpipe);
@@ -315,16 +391,16 @@ int rdpipe, wrpipe;
     /* must get START_TAPER before beginning */
 
     startclock();
-    cmd = getcmd(&argc, &argv);
+    cmd = getcmd(&cmdargs);
     total_wait = stopclock();
 
-    if(cmd != START_TAPER || argc != 2) {
-	error("error [file_reader_side cmd %d argc %d]", cmd, argc);
+    if(cmd != START_TAPER || cmdargs.argc != 2) {
+	error("error [file_reader_side cmd %d argc %d]", cmd, cmdargs.argc);
     }
 
     /* pass start command on to tape writer */
 
-    taper_datestamp = newstralloc(taper_datestamp, argv[2]);
+    taper_datestamp = newstralloc(taper_datestamp, cmdargs.argv[2]);
 
     tape_started = 0;
     syncpipe_put('S');
@@ -335,7 +411,7 @@ int rdpipe, wrpipe;
     tok = syncpipe_get();
     switch(tok) {
     case 'S':
-	putresult("TAPER-OK\n");
+	putresult(TAPER_OK, "\n");
 	tape_started = 1;
 	/* start is logged in writer */
 	break;
@@ -343,7 +419,7 @@ int rdpipe, wrpipe;
 	/* no tape, bail out */
 	result = syncpipe_getstr();
 	q = squotef("[%s]", result ? result : "(null)");
-	putresult("TAPE-ERROR %s\n", q);
+	putresult(TAPE_ERROR, "%s\n", q);
 	amfree(q);
 	log_add(L_ERROR,"no-tape [%s]", result);
 	amfree(result);
@@ -357,7 +433,7 @@ int rdpipe, wrpipe;
 
     while(1) {
 	startclock();
-	cmd = getcmd(&argc, &argv);
+	cmd = getcmd(&cmdargs);
 	if(cmd != QUIT && !tape_started) {
 	    error("error [file_reader_side cmd %d without tape ready]", cmd);
 	}
@@ -366,22 +442,57 @@ int rdpipe, wrpipe;
 	switch(cmd) {
 	case PORT_WRITE:
 	    /*
-	     * PORT-WRITE <handle> <host> <disk> <lev> <datestamp>
+	     * PORT-WRITE
+	     *   handle
+	     *   hostname
+	     *   features
+	     *   diskname
+	     *   level
+	     *   datestamp
 	     */
-	    if(argc != 6) {
-		error("error [file_reader_side PORT-WRITE argc != 6: %d]",
-		      argc);
+	    cmdargs.argc++;			/* true count of args */
+	    a = 2;
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper PORT-WRITE: not enough args: handle]");
+	    }
+	    handle = newstralloc(handle, cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper PORT-WRITE: not enough args: hostname]");
+	    }
+	    hostname = newstralloc(hostname, cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper PORT-WRITE: not enough args: features]");
+	    }
+	    am_release_feature_set(their_features);
+	    their_features = am_string_to_feature(cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper PORT-WRITE: not enough args: diskname]");
+	    }
+	    diskname = newstralloc(diskname, cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper PORT-WRITE: not enough args: level]");
+	    }
+	    level = atoi(cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper PORT-WRITE: not enough args: datestamp]");
+	    }
+	    datestamp = newstralloc(datestamp, cmdargs.argv[a++]);
+
+	    if(a != cmdargs.argc) {
+		error("error [taper file_reader_side PORT-WRITE: too many args: %d != %d]",
+		      cmdargs.argc, a);
 	    }
-	    handle = stralloc(argv[2]);
-	    hostname = stralloc(argv[3]);
-	    diskname = stralloc(argv[4]);
-	    level = atoi(argv[5]);
-	    datestamp = stralloc(argv[6]);
 
 	    data_port = 0;
 	    data_socket = stream_server(&data_port,
-					DEFAULT_SIZE,
-					DATABUF_SIZE*2);	
+					-1,
+					STREAM_BUFSIZE);	
 	    if(data_socket < 0) {
 		char *m;
 
@@ -390,75 +501,93 @@ int rdpipe, wrpipe;
 			      "]",
 			      NULL);
 		q = squote(m);
-		putresult("TAPE-ERROR %s %s\n", handle, q);
-		amfree(q);
+		putresult(TAPE_ERROR, "%s %s\n", handle, q);
 		amfree(m);
-		amfree(handle);
-		amfree(hostname);
-		amfree(diskname);
 		break;
 	    }
-	    putresult("PORT %d\n", data_port);
+	    putresult(PORT, "%d\n", data_port);
 
 	    if((fd = stream_accept(data_socket, CONNECT_TIMEOUT,
-				   DEFAULT_SIZE, sizeof(buftable->buffer))) == -1) {
+				   -1, NETWORK_BLOCK_BYTES)) == -1) {
 		q = squote("[port connect timeout]");
-		putresult("TAPE-ERROR %s %s\n", handle, q);
-		amfree(q);
+		putresult(TAPE_ERROR, "%s %s\n", handle, q);
 		aclose(data_socket);
-		amfree(handle);
-		amfree(hostname);
-		amfree(diskname);
 		break;
 	    }
 	    read_file(fd, handle, hostname, diskname, datestamp, level, 1);
 	    aclose(data_socket);
-	    amfree(handle);
-	    amfree(hostname);
-	    amfree(diskname);
 	    break;
 
 	case FILE_WRITE:
 	    /*
-	     * FILE-WRITE <sn> <fname> <hst> <dsk> <lev> <datestamp>
+	     * FILE-WRITE
+	     *   handle
+	     *   filename
+	     *   hostname
+	     *   features
+	     *   diskname
+	     *   level
+	     *   datestamp
 	     */
-	    if(argc != 7) {
-		error("error [file_reader_side FILE-WRITE argc != 7: %d]",
-		      argc);
+	    cmdargs.argc++;			/* true count of args */
+	    a = 2;
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper FILE-WRITE: not enough args: handle]");
+	    }
+	    handle = newstralloc(handle, cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper FILE-WRITE: not enough args: filename]");
+	    }
+	    filename = newstralloc(filename, cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper FILE-WRITE: not enough args: hostname]");
+	    }
+	    hostname = newstralloc(hostname, cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper FILE-WRITE: not enough args: features]");
+	    }
+	    am_release_feature_set(their_features);
+	    their_features = am_string_to_feature(cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper FILE-WRITE: not enough args: diskname]");
+	    }
+	    diskname = newstralloc(diskname, cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper FILE-WRITE: not enough args: level]");
+	    }
+	    level = atoi(cmdargs.argv[a++]);
+
+	    if(a >= cmdargs.argc) {
+		error("error [taper FILE-WRITE: not enough args: datestamp]");
+	    }
+	    datestamp = newstralloc(datestamp, cmdargs.argv[a++]);
+
+	    if(a != cmdargs.argc) {
+		error("error [taper file_reader_side FILE-WRITE: too many args: %d != %d]",
+		      cmdargs.argc, a);
 	    }
 
-	    handle = stralloc(argv[2]);
-	    hostname = stralloc(argv[4]);
-	    diskname = stralloc(argv[5]);
-	    level = atoi(argv[6]);
-	    datestamp = stralloc(argv[7]);
-
-	    if(stat(argv[3],&stat_file)!=0) {
+	    if(stat(filename,&stat_file)!=0) {
 		q = squotef("[%s]", strerror(errno));
-		putresult("TAPE-ERROR %s %s\n", handle, q);
-		amfree(q);
-		amfree(handle);
-		amfree(hostname);
-		amfree(diskname);
+		putresult(TAPE_ERROR, "%s %s\n", handle, q);
 		break;
 	    }
-	    if((fd = open(argv[3], O_RDONLY)) == -1) {
+	    if((fd = open(filename, O_RDONLY)) == -1) {
 		q = squotef("[%s]", strerror(errno));
-		putresult("TAPE-ERROR %s %s\n", handle, q);
-		amfree(q);
-		amfree(handle);
-		amfree(hostname);
-		amfree(diskname);
+		putresult(TAPE_ERROR, "%s %s\n", handle, q);
 		break;
 	    }
 	    read_file(fd, handle, hostname, diskname, datestamp, level, 0);
-	    amfree(handle);
-	    amfree(hostname);
-	    amfree(diskname);
 	    break;
 
 	case QUIT:
-	    putresult("QUITTING\n");
+	    putresult(QUITTING, "\n");
 	    fprintf(stderr,"taper: DONE [idle wait: %s secs]\n",
 		    walltime_str(total_wait));
 	    fflush(stderr);
@@ -472,7 +601,7 @@ int rdpipe, wrpipe;
 		fflush(stderr);
 	    }
 
-	    detach_buffers(buftable);
+	    detach_buffers(buffers);
 	    destroy_buffers();
 	    amfree(datestamp);
 	    amfree(label);
@@ -491,14 +620,22 @@ int rdpipe, wrpipe;
 	    exit(0);
 
 	default:
-	    handle = stralloc(argv[1]);
-	    q = squote(handle);
-	    putresult("BAD-COMMAND %s\n", q);
-	    amfree(q);
-	    amfree(handle);
+	    if(cmdargs.argc >= 1) {
+		q = squote(cmdargs.argv[1]);
+	    } else if(cmdargs.argc >= 0) {
+		q = squote(cmdargs.argv[0]);
+	    } else {
+		q = stralloc("(no input?)");
+	    }
+	    putresult(BAD_COMMAND, "%s\n", q);
 	    break;
 	}
     }
+    amfree(q);
+    amfree(handle);
+    amfree(filename);
+    amfree(hostname);
+    amfree(diskname);
 }
 
 void dumpbufs(str1)
@@ -533,7 +670,7 @@ buffer_t *bp;
 {
     char pn[2];
     char bt[NUM_STR_SIZE];
-    char status[NUM_STR_SIZE];
+    char status[NUM_STR_SIZE + 1];
     char *str = NULL;
 
     pn[0] = procname[0];
@@ -541,7 +678,8 @@ buffer_t *bp;
     ap_snprintf(bt, sizeof(bt), "%d", (int)(bp-buftable));
 
     switch(bp->status) {
-    case FULL:		status[0] = 'F'; status[1] = '\0'; break;
+    case FULL:		ap_snprintf(status, sizeof(status), "F%d", bp->size);
+			break;
     case FILLING:	status[0] = 'f'; status[1] = '\0'; break;
     case EMPTY:		status[0] = 'E'; status[1] = '\0'; break;
     default:
@@ -561,12 +699,13 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 {
     buffer_t *bp;
     char tok;
-    int rc, err, opening, closing, bufnum;
+    int rc, err, opening, closing, bufnum, need_closing;
     long filesize;
     times_t runtime;
     char *strclosing = NULL;
     char *str;
     int header_read = 0;
+    int buflen;
     dumpfile_t file;
 
     char *q = NULL;
@@ -582,7 +721,9 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 
     filesize = 0;
     closing = 0;
+    need_closing = 0;
     err = 0;
+    fh_init(&file);
 
     if(bufdebug) {
 	fprintf(stderr, "taper: r: start file\n");
@@ -591,15 +732,19 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 
     for(bp = buftable; bp < buftable + conf_tapebufs; bp++) {
 	bp->status = EMPTY;
-	if(interactive || bufdebug) dumpstatus(bp);
     }
 
     bp = buftable;
+    if(interactive || bufdebug) dumpstatus(bp);
 
     /* tell writer to open tape */
 
     opening = 1;
     syncpipe_put('O');
+    syncpipe_putstr(datestamp);
+    syncpipe_putstr(hostname);
+    syncpipe_putstr(diskname);
+    syncpipe_putint(level);
 
     startclock();
 
@@ -623,6 +768,13 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 		fflush(stderr);
 	    }
 
+	    if(need_closing) {
+		syncpipe_put('C');
+		closing = 1;
+		need_closing = 0;
+		break;
+	    }
+
 	    if(closing) break;	/* ignore extra read tokens */
 
 	    assert(!opening);
@@ -651,7 +803,7 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 		errstr = newstralloc(errstr,
 				     "[fatal buffer mismanagement bug]");
 		q = squote(errstr);
-		putresult("TRY-AGAIN %s %s\n", handle, q);
+		putresult(TRYAGAIN, "%s %s\n", handle, q);
 		amfree(q);
 		log_add(L_INFO, "retrying %s:%s.%d on new tape: %s",
 		        hostname, diskname, level, errstr);
@@ -667,81 +819,104 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 	    }
 
 	    bp->status = FILLING;
+	    buflen = header_read ? tt_blocksize : DISK_BLOCK_BYTES;
 	    if(interactive || bufdebug) dumpstatus(bp);
-	    if((rc = taper_fill_buffer(fd, bp->buffer, 
-				       sizeof(bp->buffer))) < 0) {
-		err = (rc < 0)? errno : 0;
+	    if((rc = taper_fill_buffer(fd, bp, buflen)) < 0) {
+		err = errno;
 		closing = 1;
 		strclosing = newvstralloc(strclosing,"Can't read data: ",NULL);
 		syncpipe_put('C');
-	    }
-	    else {
-		if(rc == 0) { /* switch to next file */
+	    } else {
+		if(rc < buflen) { /* switch to next file */
 		    int save_fd;
+		    struct stat stat_file;
+
 		    save_fd = fd;
 		    close(fd);
-		    if(file.cont_filename[0] == '\0' ||  /* no more file */
-		       strlen(file.cont_filename) ==0) { /* no more file */
+		    if(file.cont_filename[0] == '\0') {	/* no more file */
 			err = 0;
-			closing = 1;
-			syncpipe_put('C');
-		    }
-		    else if((fd = open(file.cont_filename,O_RDONLY)) == -1) {
+			need_closing = 1;
+		    } else if(stat(file.cont_filename, &stat_file) != 0) {
 			err = errno;
-			closing = 1;
+			need_closing = 1;
+			strclosing = newvstralloc(strclosing,"can't stat: ",file.cont_filename,NULL);
+		    } else if((fd = open(file.cont_filename,O_RDONLY)) == -1) {
+			err = errno;
+			need_closing = 1;
 			strclosing = newvstralloc(strclosing,"can't open: ",file.cont_filename,NULL);
-			syncpipe_put('C');
-			rc = 0;
-		    }
-		    else if((fd != save_fd) && dup2(fd, save_fd) == -1) {
+		    } else if((fd != save_fd) && dup2(fd, save_fd) == -1) {
 			err = errno;
-			closing = 1;
+			need_closing = 1;
 			strclosing = newvstralloc(strclosing,"can't dup2: ",file.cont_filename,NULL);
-			syncpipe_put('C');
-		    }
-		    else {
+		    } else {
+			buffer_t bp1;
+			int rc1;
+
+			bp1.status = EMPTY;
+			bp1.size = DISK_BLOCK_BYTES;
+			bp1.buffer = malloc(DISK_BLOCK_BYTES);
+
 			if(fd != save_fd) {
 			    close(fd);
 			    fd = save_fd;
 			}
-			if((rc = taper_fill_buffer(fd, bp->buffer, 
-						   sizeof(bp->buffer))) <= 0) {
-			    err = (rc < 0)? errno : 0;
-			    closing = 1;
-			    strclosing = newvstralloc(strclosing,"Can't read header: ",file.cont_filename,NULL);
-			    syncpipe_put('C');
-			}
-			else {
-			    parse_file_header(bp->buffer, &file, rc);
 
-			    if((rc = taper_fill_buffer(fd, bp->buffer, 
-						   sizeof(bp->buffer))) <= 0) {
-				err = (rc < 0)? errno : 0;
-				closing = 1;
-				if(rc<0)
-			    	    strclosing = newvstralloc(strclosing,"Can't read data: ",NULL);
-				syncpipe_put('C');
+			rc1 = taper_fill_buffer(fd, &bp1, DISK_BLOCK_BYTES);
+			if(rc1 <= 0) {
+			    amfree(bp1.buffer);
+			    err = (rc1 < 0) ? errno : 0;
+			    need_closing = 1;
+			    strclosing = newvstralloc(strclosing,
+						      "Can't read header: ",
+						      file.cont_filename,
+						      NULL);
+			} else {
+			    parse_file_header(bp1.buffer, &file, rc1);
+
+			    amfree(bp1.buffer);
+			    bp1.buffer = bp->buffer + rc;
+
+			    rc1 = taper_fill_buffer(fd, &bp1, tt_blocksize - rc);
+			    if(rc1 <= 0) {
+				err = (rc1 < 0) ? errno : 0;
+				need_closing = 1;
+				if(rc1 < 0) {
+			    	    strclosing = newvstralloc(strclosing,
+							      "Can't read data: ",
+							      file.cont_filename,
+							      NULL);
+				}
+			    }
+			    else {
+				rc += rc1;
+				bp->size = rc;
 			    }
 			}
 		    }
 		}
 		if(rc > 0) {
+		    bp->status = FULL;
 		    if(header_read == 0) {
 			char *cont_filename;
 
 			parse_file_header(bp->buffer, &file, rc);
 			cont_filename = stralloc(file.cont_filename);
-			memset(file.cont_filename,'\0',sizeof(file.cont_filename));
-			write_header(bp->buffer,&file,rc);
+			file.cont_filename[0] = '\0';
+			file.blocksize = tt_blocksize;
+			build_header(bp->buffer, &file, tt_blocksize);
 
-			/* add CONT_FILENAME to header */
+			/* add CONT_FILENAME back to in-memory header */
 			strncpy(file.cont_filename, cont_filename, 
 				sizeof(file.cont_filename));
+			if(interactive || bufdebug) dumpstatus(bp);
+			bp->size = tt_blocksize; /* output a full tape block */
 			header_read = 1;
+			amfree(cont_filename);
+		    }
+		    else {
+			filesize += am_round(rc, 1024) / 1024;
 		    }
-		    bp->status = FULL;
 		    if(interactive || bufdebug) dumpstatus(bp);
-			filesize += rc / 1024;
 		    if(bufdebug) {
 			fprintf(stderr,"taper: r: put W%d\n",(int)(bp-buftable));
 			fflush(stderr);
@@ -750,6 +925,11 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 		    syncpipe_putint(bp-buftable);
 		    bp = nextbuf(bp);
 		}
+		if(need_closing && rc <= 0) {
+		    syncpipe_put('C');
+		    need_closing = 0;
+		    closing = 1;
+		}
 	    }
 	    break;
 
@@ -764,13 +944,13 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 
 	    q = squote(errstr);
 	    if(tok == 'T') {
-		putresult("TRY-AGAIN %s %s\n", handle, q);
+		putresult(TRYAGAIN, "%s %s\n", handle, q);
 		log_add(L_INFO, "retrying %s:%s.%d on new tape: %s",
 		        hostname, diskname, level, errstr);
 	    } else {
-		putresult("TAPE-ERROR %s %s\n", handle, q);
-		log_add(L_FAIL, "%s %s %d [out of tape]",
-			hostname, diskname, level);
+		putresult(TAPE_ERROR, "%s %s\n", handle, q);
+		log_add(L_FAIL, "%s %s %s %d [out of tape]",
+			hostname, diskname, datestamp, level);
 		log_add(L_ERROR,"no-tape [%s]", errstr);
 	    }
 	    amfree(q);
@@ -804,10 +984,10 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 				          "[input: ", strerror(err), "]",
 				          NULL);
 		q = squote(errstr);
-		putresult("TAPE-ERROR %s %s\n", handle, q);
+		putresult(TAPE_ERROR, "%s %s\n", handle, q);
 		amfree(q);
-		log_add(L_FAIL, "%s %s %d %s",
-			hostname, diskname, level, errstr);
+		log_add(L_FAIL, "%s %s %s %d %s",
+			hostname, diskname, datestamp, level, errstr);
 		str = syncpipe_getstr();	/* reap stats */
 		amfree(str);
 	    } else {
@@ -829,7 +1009,7 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
 				      NULL);
 		amfree(str);
 		q = squote(errstr);
-		putresult("DONE %s %s %d %s\n",
+		putresult(DONE, "%s %s %d %s\n",
 			  handle, label, filenum, q);
 		amfree(q);
 		log_add(L_SUCCESS, "%s %s %s %d %s",
@@ -903,42 +1083,37 @@ void read_file(fd, handle, hostname, diskname, datestamp, level, port_flag)
     }
 }
 
-int taper_fill_buffer(fd, buffer, size)
-int fd, size;
-char *buffer;
+int taper_fill_buffer(fd, bp, buflen)
+int fd;
+buffer_t *bp;
+int buflen;
 {
     char *curptr;
     int spaceleft, cnt;
 
-    curptr = buffer;
-    spaceleft = size;
+    curptr = bp->buffer;
+    bp->size = 0;
+    spaceleft = buflen;
 
     do {
 	cnt = read(fd, curptr, spaceleft);
 	switch(cnt) {
 	case 0:	/* eof */
-	    if(spaceleft == size) {
-		if(interactive) fputs("r0", stderr);
-		return 0;
-	    }
-	    else {
-		/* partial buffer, zero rest */
-		memset(curptr, '\0', spaceleft);
-		if(interactive) fputs("rP", stderr);
-		return size;
-	    }
+	    if(interactive) fputs("r0", stderr);
+	    return bp->size;
 	case -1:	/* error on read, punt */
 	    if(interactive) fputs("rE", stderr);
 	    return -1;
 	default:
 	    spaceleft -= cnt;
 	    curptr += cnt;
+	    bp->size += cnt;
 	}
 
     } while(spaceleft > 0);
 
     if(interactive) fputs("R", stderr);
-    return size;
+    return bp->size;
 }
 
 
@@ -960,8 +1135,12 @@ void tape_writer_side(getp, putp)
 int getp, putp;
 {
     char tok;
-    int tape_started, out_open;
+    int tape_started;
     char *str;
+    char *hostname;
+    char *diskname;
+    char *datestamp;
+    int level;
 
 #ifdef HAVE_LIBVTBLC
     char *vol_label;
@@ -972,7 +1151,6 @@ int getp, putp;
     syncpipe_init(getp, putp);
 
     tape_started = 0;
-    out_open = 0;
     idlewait = times_zero;
 
     while(1) {
@@ -1012,6 +1190,17 @@ int getp, putp;
 	    break;
 
 	case 'O':		/* open-output */
+	    datestamp = syncpipe_getstr();
+	    tapefd_setinfo_datestamp(tape_fd, datestamp);
+	    amfree(datestamp);
+	    hostname = syncpipe_getstr();
+	    tapefd_setinfo_host(tape_fd, hostname);
+	    amfree(hostname);
+	    diskname = syncpipe_getstr();
+	    tapefd_setinfo_disk(tape_fd, diskname);
+	    amfree(diskname);
+	    level = syncpipe_getint();
+	    tapefd_setinfo_level(tape_fd, level);
 	    write_file();
 	    break;
 
@@ -1088,7 +1277,6 @@ void write_file()
 	    fflush(stderr);
 	}
 	syncpipe_put('R'); syncpipe_putint(i);
-
     }
 
     /*
@@ -1140,6 +1328,10 @@ void write_file()
 	 */
 
 	while(full_buffers) {
+	    if(tt_file_pad && bp->size < tt_blocksize) {
+		memset(bp->buffer+bp->size, 0, tt_blocksize - bp->size);
+		bp->size = tt_blocksize;
+	    }
 	    if(!write_buffer(bp)) goto tape_error;
 	    full_buffers--;
 	    bp = nextbuf(bp);
@@ -1175,6 +1367,10 @@ void write_file()
 		    syncpipe_putstr("writer-side buffer mismatch");
 		    goto error_ack;
 		}
+		if(tt_file_pad && bp->size < tt_blocksize) {
+		    memset(bp->buffer+bp->size, 0, tt_blocksize - bp->size);
+		    bp->size = tt_blocksize;
+		}
 		if(!write_buffer(bp)) goto tape_error;
 		bp = nextbuf(bp);
 	    }
@@ -1244,16 +1440,6 @@ void write_file()
     return;
 }
 
-/*
- * Define the following to test tape error handling.  An EIO error will
- * be simulated after the number of blocks in block_counter.  This works
- * especially well with tapedev set to /dev/null.
- */
-#undef FAKE_TAPE_ERROR
-#if defined(FAKE_TAPE_ERROR)
-static int block_counter = 10;
-#endif
-
 int write_buffer(bp)
 buffer_t *bp;
 {
@@ -1265,14 +1451,8 @@ buffer_t *bp;
     }
 
     startclock();
-    rc = tapefd_write(tape_fd, bp->buffer, sizeof(bp->buffer));
-#if defined(FAKE_TAPE_ERROR)
-    if(--block_counter <= 0) {
-	rc = -1;
-	errno = EIO;
-    }
-#endif
-    if(rc == sizeof(bp->buffer)) {
+    rc = tapefd_write(tape_fd, bp->buffer, bp->size);
+    if(rc == bp->size) {
 #if defined(NEED_RESETOFS)
 	static double tape_used_modulus_2gb = 0;
 
@@ -1300,8 +1480,7 @@ buffer_t *bp;
 	}
 	syncpipe_put('R'); syncpipe_putint(bp-buftable);
 	return 1;
-    }
-    else {
+    } else {
 	errstr = newvstralloc(errstr,
 			      "writing file: ",
 			      (rc != -1) ? "short write" : strerror(errno),
@@ -1313,71 +1492,6 @@ buffer_t *bp;
 }
 
 
-/*
- * ========================================================================
- * MISC HELPER FUNCTIONS
- *
- */
-cmd_t
-getcmd(argcp, argvp)
-    int *argcp;
-    char ***argvp;
-{
-    /*
-     * WARNING: this routine stores the decoded strings in a static buffer.
-     * Subsequent calls will overwrite previous values, so if you want
-     * them to hang around, the calling routine had better copy them to
-     * storage it has allocated for that purpose.
-     */
-    char *line;
-    static char *argv[MAX_ARGS+1];
-    int argc;
-
-    if(interactive) {
-	fprintf(stderr, "%s> ", get_pname());
-	fflush(stderr);
-    }
-
-    if((line = agets(stdin)) == NULL) return QUIT;
-
-    argc = split(line, argv, sizeof(argv) / sizeof(argv[0]), " ");
-
-#ifdef DEBUG
-    {
-      int arg;
-      fprintf(stderr, "argc = %d\n", argc);
-      for(arg = 0; arg <= argc; arg++)
-	fprintf(stderr, "argv[%d] = \"%s\"\n", arg, argv[arg]);
-    }
-#endif
-
-    amfree(line);
-
-    *argvp = argv;
-    *argcp = argc;
-
-    /* not enough commands for a table lookup */
-
-    if(argc < 1) return BOGUS;
-    if(strcmp(argv[1],"START-TAPER") == 0) return START_TAPER;
-    if(strcmp(argv[1],"FILE-WRITE") == 0) return FILE_WRITE;
-    if(strcmp(argv[1],"PORT-WRITE") == 0) return PORT_WRITE;
-    if(strcmp(argv[1],"QUIT") == 0) return QUIT;
-    return BOGUS;
-}
-
-
-arglist_function(void putresult, char *, format)
-{
-    va_list argp;
-
-    arglist_start(argp, format);
-    vprintf(format, argp);
-    fflush(stdout);
-    arglist_end(argp);
-}
-
-
 /*
  * ========================================================================
  * SHARED-MEMORY BUFFER SUBSYSTEM
@@ -1388,20 +1502,23 @@ arglist_function(void putresult, char *, format)
 
 int shmid = -1;
 
-buffer_t *attach_buffers()
+char *attach_buffers(size)
+    unsigned int size;
 {
-    buffer_t *result;
+    char *result;
 
-    conf_tapebufs = getconf_int(CNF_TAPEBUFS);
-    shmid = shmget(IPC_PRIVATE, sizeof(buffer_t)*conf_tapebufs, IPC_CREAT|0700);
-    
-    if(shmid == -1)
-	error("shmget: (%d tapebufs) %s", sizeof(buffer_t)*conf_tapebufs,strerror(errno));
+    shmid = shmget(IPC_PRIVATE, size, IPC_CREAT|0700);
+    if(shmid == -1) {
+	return NULL;
+    }
 
-    result = (buffer_t *)shmat(shmid, (SHM_ARG_TYPE *)NULL, 0);
+    result = (char *)shmat(shmid, (SHM_ARG_TYPE *)NULL, 0);
+
+    if(result == (char *)-1) {
+	int save_errno = errno;
 
-    if(result == (buffer_t *)-1) {
 	destroy_buffers();
+	errno = save_errno;
 	error("shmat: %s", strerror(errno));
     }
 
@@ -1410,17 +1527,19 @@ buffer_t *attach_buffers()
 
 
 void detach_buffers(bufp)
-     buffer_t *bufp;
+    char *bufp;
 {
-    if(shmdt((SHM_ARG_TYPE *)bufp) == -1)
+    if(shmdt((SHM_ARG_TYPE *)bufp) == -1) {
 	error("shmdt: %s", strerror(errno));
+    }
 }
 
 void destroy_buffers()
 {
     if(shmid == -1) return;	/* nothing to destroy */
-    if(shmctl(shmid, IPC_RMID, NULL) == -1)
+    if(shmctl(shmid, IPC_RMID, NULL) == -1) {
 	error("shmctl: %s", strerror(errno));
+    }
 }
 
 #else
@@ -1440,37 +1559,40 @@ void destroy_buffers()
 #endif
 
 int shmfd = -1;
+unsigned int saved_size;
 
-buffer_t *attach_buffers()
+char *attach_buffers(size)
+    unsigned int size;
 {
-    buffer_t *shmbuf;
+    char *shmbuf;
 
-    conf_tapebufs = getconf_int(CNF_TAPEBUFS);
 #ifdef ZERO_FILE
     shmfd = open(ZERO_FILE, O_RDWR);
-    if(shmfd == -1)
+    if(shmfd == -1) {
 	error("attach_buffers: could not open %s: %s",
-	      ZERO_FILE, strerror(errno));
+	      ZERO_FILE,
+	      strerror(errno));
+    }
 #endif
 
-    shmbuf = (buffer_t *) mmap((void *) 0,
-			       sizeof(buffer_t)*conf_tapebufs,
-			       PROT_READ|PROT_WRITE,
-			       MAP_ANON|MAP_SHARED,
-			       shmfd, 0);
-    if((long)shmbuf == -1)
-	error("attach_buffer: mmap: %s", strerror(errno));
+    saved_size = size;
+    shmbuf = (char *) mmap((void *) 0,
+			   size,
+			   PROT_READ|PROT_WRITE,
+			   MAP_ANON|MAP_SHARED,
+			   shmfd, 0);
 
     return shmbuf;
 }
 
 void detach_buffers(bufp)
-buffer_t *bufp;
+char *bufp;
 {
-    if(munmap((void *)bufp, sizeof(buffer_t)*conf_tapebufs) == -1)
+    if(munmap((void *)bufp, saved_size) == -1) {
 	error("detach_buffers: munmap: %s", strerror(errno));
+    }
 
-    if(shmfd != -1) aclose(shmfd);
+    aclose(shmfd);
 }
 
 void destroy_buffers()
@@ -1667,9 +1789,16 @@ int label_tape()
     amfree(olddatestamp);
 
     /* check against tape list */
-    if (strcmp(tapedev, "/dev/null") != 0) {
+    if (strcmp(label, FAKE_LABEL) != 0) {
 	tp = lookup_tapelabel(label);
-	if(tp != NULL && !reusable_tape(tp)) {
+	if(tp == NULL) {
+	    errstr = newvstralloc(errstr,
+				  "label ", label,
+		" match labelstr but it not listed in the tapelist file",
+				  NULL);
+	    return 0;
+	}
+	else if(tp != NULL && !reusable_tape(tp)) {
 	    errstr = newvstralloc(errstr,
 			          "cannot overwrite active tape ", label,
 			          NULL);
@@ -1696,7 +1825,12 @@ int label_tape()
 	return 0;
     }
 
-    if((result = tapefd_wrlabel(tape_fd, taper_datestamp, label)) != NULL) {
+    tapefd_setinfo_length(tape_fd, tt->length);
+
+    tapefd_setinfo_datestamp(tape_fd, taper_datestamp);
+    tapefd_setinfo_disk(tape_fd, label);
+    result = tapefd_wrlabel(tape_fd, taper_datestamp, label, tt_blocksize);
+    if(result != NULL) {
 	errstr = newstralloc(errstr, result);
 	return 0;
     }
@@ -1716,9 +1850,7 @@ int label_tape()
     /* write tape list */
 
     /* XXX add cur_tape number to tape list structure */
-    if (strcmp(tapedev, "/dev/null") != 0) {
-	remove_tapelabel(label);
-	add_tapelabel(atoi(taper_datestamp), label);
+    if (strcmp(label, FAKE_LABEL) != 0) {
 
 	if(cur_tape == 0) {
 	    conf_tapelist_old = stralloc2(conf_tapelist, ".yesterday");
@@ -1729,12 +1861,13 @@ int label_tape()
 	    conf_tapelist_old = vstralloc(conf_tapelist,
 					".today.", cur_str, NULL);
 	}
-	if(rename(conf_tapelist, conf_tapelist_old) != 0) {
-	    if(errno != ENOENT)
-		error("could not rename \"%s\" to \"%s\": %s",
-		      conf_tapelist, conf_tapelist_old, strerror(errno));
+	if(write_tapelist(conf_tapelist_old)) {
+	    error("could not write tapelist: %s", strerror(errno));
 	}
 	amfree(conf_tapelist_old);
+
+	remove_tapelabel(label);
+	add_tapelabel(atoi(taper_datestamp), label);
 	if(write_tapelist(conf_tapelist)) {
 	    error("could not write tapelist: %s", strerror(errno));
 	}
@@ -1742,7 +1875,7 @@ int label_tape()
 
     log_add(L_START, "datestamp %s label %s tape %d",
 	    taper_datestamp, label, cur_tape);
-    if (first_call && strcmp(tapedev, "/dev/null") == 0) {
+    if (first_call && strcmp(label, FAKE_LABEL) == 0) {
 	first_call = 0;
 	log_add(L_WARNING, "tapedev is %s, dumps will be thrown away", tapedev);
     }
@@ -1812,7 +1945,8 @@ int writerror;
 		goto common_exit;
 	    }
 
-	    if((result = tapefd_wrendmark(tape_fd, taper_datestamp)) != NULL) {
+	    result = tapefd_wrendmark(tape_fd, taper_datestamp, tt_blocksize);
+	    if(result != NULL) {
 		errstr = newstralloc(errstr, result);
 		rc = 1;
 		goto common_exit;
@@ -1955,13 +2089,6 @@ common_exit:
 
 int write_filemark()
 {
-#if defined(FAKE_TAPE_ERROR)
-    if(--block_counter <= 0) {
-	errno = EIO;
-	errstr = newstralloc2(errstr, "writing filemark: ", strerror(errno));
-	return 0;
-    }
-#endif
     if(tapefd_weof(tape_fd, 1) == -1) {
 	errstr = newstralloc2(errstr, "writing filemark: ", strerror(errno));
 	return 0;
@@ -2029,7 +2156,9 @@ int taperscan_slot(rc, slotstr, device)
 		    get_pname(), slotstr, scan_datestamp, label);
 	    fflush(stderr);
 	    amfree(scan_datestamp);
-	    if(searchlabel != NULL && strcmp(label, searchlabel) == 0) {
+	    if(searchlabel != NULL
+	       && (strcmp(label, FAKE_LABEL) == 0
+		   || strcmp(label, searchlabel) == 0)) {
 		/* it's the one we are looking for, stop here */
 		fprintf(stderr, " (exact label match)\n");
 		fflush(stderr);
@@ -2045,10 +2174,24 @@ int taperscan_slot(rc, slotstr, device)
 		/* not an exact label match, but a labelstr match */
 		/* check against tape list */
 		tp = lookup_tapelabel(label);
-		if(tp != NULL && !reusable_tape(tp)) {
+		if(tp == NULL) {
+		    fprintf(stderr, "(not in tapelist)\n");
+		    fflush(stderr);
+		}
+		else if(!reusable_tape(tp)) {
 		    fprintf(stderr, " (active tape)\n");
 		    fflush(stderr);
 		}
+		else if(got_match == 0 && tp->datestamp == 0) {
+		    got_match = 1;
+		    first_match = newstralloc(first_match, slotstr);
+		    first_match_label = newstralloc(first_match_label, label);
+		    fprintf(stderr, " (new tape)\n");
+		    fflush(stderr);
+		    found = 3;
+		    found_device = newstralloc(found_device, device);
+		    return 1;
+		}
 		else if(got_match) {
 		    fprintf(stderr, " (labelstr match)\n");
 		    fflush(stderr);
@@ -2084,11 +2227,11 @@ char *taper_scan()
     got_match = 0;
 
     if (searchlabel != NULL)
-      changer_find(scan_init, taperscan_slot,searchlabel);
+      changer_find(scan_init, taperscan_slot, searchlabel);
     else
       changer_scan(scan_init, taperscan_slot);
 
-    if(found == 2)
+    if(found == 2 || found == 3)
 	searchlabel = first_match_label;
     else if(!found && got_match) {
 	searchlabel = first_match_label;
